SystemOrganization addCategory: #'Math-Complex'!
SystemOrganization addCategory: #'Math-ComplexTests'!
SystemOrganization addCategory: #'Math-DHB-Numerical'!
SystemOrganization addCategory: #'Math-DHB-Numerical-Tests'!
SystemOrganization addCategory: #'Math-Quaternion'!
SystemOrganization addCategory: #'Math-QuaternionTests'!
SystemOrganization addCategory: #'Math-Random'!
SystemOrganization addCategory: #'Math-Random-Tests'!
SystemOrganization addCategory: #'Math-KDTree'!

!Number class methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/11/2012 13:53'!
random
		"Answers a random number between 0 and the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^ DhbMitchellMooreGenerator new floatValue! !

!Number methodsFor: '*Math-Complex-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToComplex: rcvr andSend: selector
	"If I am involved in arithmetic with a Complex number, convert me to a Complex number."
	^ rcvr perform: selector with: self asComplex! !

!Number methodsFor: '*Math-Quaternion-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToQuaternion: rcvr andSend: selector 
	"If I am involved in arithmetic with a Quaternion, convert me to a Quaternion."
	^ rcvr perform: selector with: self asQuaternion! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
addPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self! !

!Number methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arCosh
	"Answer receiver's area hyperbolic cosine.
	That is the inverse function of cosh."

	^self asFloat arCosh! !

!Number methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arSinh
	"Answer receiver's area hyperbolic sine.
	That is the inverse function of sinh."

	^self asFloat arSinh! !

!Number methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arTanh
	"Answer receiver's area hyperbolic tangent.
	That is the inverse function of tanh."

	^self asFloat arTanh! !

!Number methodsFor: '*Math-Complex-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asComplex
	"Answer a Complex number that represents value of the the receiver."

	^ Complex real: self imaginary: 0! !

!Number methodsFor: '*Math-Quaternion-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asQuaternion
	^Quaternion 
		qr: self
		qi: 0
		qj: 0
		qk: 0! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
beta: aNumber
		"Computes the beta function of the receiver and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self logBeta: aNumber) exp! !

!Number methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
cosh
	"Answer receivers hyperbolic cosine."
	
	^self asFloat cosh! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
dividingPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: (1 / self)! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
equalsTo: aNumber
		"compare to Float>>closeTo:
		
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	^self relativelyEqualsTo: aNumber upTo: DhbFloatingPointMachine new defaultNumericalPrecision! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
errorFunction
		"Answer the error function for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^ DhbErfApproximation new value: self! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
gamma
		"Compute the Gamma function for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^ self > 1
		ifTrue: [ ^ DhbLanczosFormula new gamma: self]
		ifFalse:[ self < 0
						ifTrue: [ Float pi / ( ( Float pi * self) sin * ( 1 - self) gamma)]
						ifFalse:[ ( DhbLanczosFormula new gamma: ( self + 1)) / self]
					]! !

!Number methodsFor: '*Math-Complex-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
i
	^ Complex real: 0 imaginary: self! !

!Number methodsFor: '*Math-Complex-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
i: aNumber
	"Form a complex number with
		receiver as realPart
		aNumber as imaginaryPart
	this is the same as (self + aNumber i) but a little bit more efficient."

	aNumber isNumber ifFalse: [self error: 'Badly formed complex number'].
	^Complex real: self imaginary: aNumber! !

!Number methodsFor: '*Math-Quaternion-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
i: qi j: qj k: qk 
	^Quaternion 
		qr: self
		qi: qi
		qj: qj
		qk: qk! !

!Number methodsFor: '*Math-Quaternion-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
j
	^Quaternion 
		qr: 0
		qi: 0
		qj: self
		qk: 0! !

!Number methodsFor: '*Math-Quaternion-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
k
	^Quaternion 
		qr: 0
		qi: 0
		qj: 0
		qk: self! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
logBeta: aNumber
		"Computes the logarithm of the beta function of the receiver and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^self logGamma + aNumber logGamma - ( self + aNumber) logGamma! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
logGamma
		"Computes the log of the Gamma function (for positive numbers only)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^self > 1
		ifTrue: [ DhbLanczosFormula new logGamma: self]
		ifFalse:[ self > 0
						ifTrue: [ ( DhbLanczosFormula new logGamma: ( self + 1) ) - self ln ]
						ifFalse: [ ^self error: 'Argument for the log gamma function must be positive']
					]! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productWithVector: aVector
		"Answers a new vector product of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aVector collect: [ :each | each * self]! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answers a random number distributed between 0 and the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^self class random * self! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
relativelyEqualsTo: aNumber upTo: aSmallNumber
		"compare to Float>>closeTo:
		generally called from Number>>equalsTo:
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	| norm |
	norm := self abs max: aNumber abs.
	^norm <= DhbFloatingPointMachine new defaultNumericalPrecision
		or: [ (self - aNumber) abs < ( aSmallNumber * norm)]! !

!Number methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sinc
	"Answer receivers cardinal sine."
	
	^ self isZero
		ifTrue: [self class one]
		ifFalse: [self sin / self]! !

!Number methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sinh
	"Answer receivers hyperbolic sine"
	
	^self asFloat sinh! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
subtractToPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self negated! !

!Number methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
tanh
	"Answer receivers hyperbolic tangent"
	
	^self asFloat tanh! !

!Number methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
timesPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: self! !

!Integer methodsFor: '*Math' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asVector
	"Convert the receiver to a vector. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 11/2/99"

	| aVector index |
	aVector := DhbVector new: self size.
	index := 0.
	self do: [:each | aVector at: (index := index + 1) put: each].
	^aVector! !

!Integer methodsFor: '*Math' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
choose: anInteger
"answer the binomial coefficient"

((anInteger > self) not and: (anInteger < 0) not)
	ifTrue:
		[^ self factorial / ((anInteger factorial) * ((self - anInteger) factorial))]
	ifFalse:
		[^ 0]! !

!Integer methodsFor: '*Math' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random integer between 0 and the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^ DhbMitchellMooreGenerator new integerValue: self! !

!Collection methodsFor: '*Math' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asVector
	"Convert the receiver to a vector. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 11/2/99"

	| aVector index |
	aVector := DhbVector new: self size.
	index := 0.
	self do: [:each | aVector at: (index := index + 1) put: each].
	^aVector! !

!OrderedCollection methodsFor: '*Math' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
removeAtIndex: anIndex
	"Remove the element of the collection at position anIndex.  Answer the object removed."

	| obj |
	obj := self at: anIndex.
	self removeIndex: anIndex + firstIndex - 1.
	^obj! !

!Float methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arCosh
	"Answer receiver's area hyperbolic cosine.
	That is the inverse function of cosh."

	self < 1 
		ifTrue: 
			[^DomainError signal: 'Receiver must be greater or equal to 1'].
	^self + 1 = self 
		ifTrue: [self abs ln + 2 ln]
		ifFalse: [((self squared - 1) sqrt + self) ln]! !

!Float methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arSinh
	"Answer receiver's area hyperbolic sine.
	That is the inverse function of sinh."

	self = 0.0 ifTrue: [^self].	"Handle negativeZero"  
	^self + 1 = self 
		ifTrue: [(self abs ln + 2 ln) * self sign]
		ifFalse: [((self squared + 1) sqrt + self) ln]! !

!Float methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arTanh
	"Answer receiver's area hyperbolic tangent.
	That is the inverse function of tanh."

	self = 0.0 ifTrue: [^self].	"Handle negativeZero"
	self abs = 1 ifTrue: [^self copySignTo: Float infinity].
	self abs > 1 
		ifTrue: 
			[^DomainError signal: 'Receiver must be between -1.0 and 1.0'].
	^((1 + self) / (1 - self)) ln / 2! !

!Float methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
cosh
	"Answer receivers hyperbolic cosine."
	
	| ex |
	ex := self exp.
	^(ex + ex reciprocal) / 2! !

!Float methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sinh
	"Answer receivers hyperbolic sine"
	
	| ex |
	ex := self exp.
	^(ex - ex reciprocal) / 2! !

!Float methodsFor: '*Math-Number-Extensions-mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
tanh
	"Answer hyperbolic tangent of receiver.
	Trivial implementation is:
		^self sinh/self cosh
	This implementation takes care not to overflow."

	| ex emx |
	self > 20.0 ifTrue: [^1.0].
	self < -20.0 ifTrue: [^-1.0].
	ex := self exp.
	emx := ex reciprocal.
	^(ex - emx) / (ex + emx)! !

Array variableSubclass: #DhbVector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
* aNumberOrMatrixOrVector
		"Answers the product of the receiver with the argument.
		 The argument can be a number, matrix or vector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aNumberOrMatrixOrVector productWithVector: self! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
+ aVector
		"Answers the sum of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| answer n |
	answer := self class new: self size.
	n := 0.
	self with: aVector do:
		[ :a :b | 
		  n := n + 1. 
		  answer at: n put: ( a + b).
		].
	^answer! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
- aVector
		"Answers the difference of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| answer n |
	answer := self class new: self size.
	n := 0.
	self with: aVector do:
		[ :a :b | 
		  n := n + 1. 
		  answer at: n put: ( a - b).
		].
	^answer! !

!DhbVector methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aVectorOrAnArray
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	1 to: self size do: [ :n | self at: n put: ( ( self at: n) + ( aVectorOrAnArray at: n))].! !

!DhbVector methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulateNegated: aVectorOrAnArray
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	1 to: self size do: [ :n | self at: n put: ( ( self at: n) - ( aVectorOrAnArray at: n))].! !

!DhbVector methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asVector
		"Answer self since the receiver is a vector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self! !

!DhbVector methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
dimension
		"Answer the dimension of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self size! !

!DhbVector methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
negate
		"Inverse the sign of all components of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	1 to: self size do: [ :n | self at: n put: (self at: n) negated].! !

!DhbVector methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
norm
		"Answer the norm of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^(self * self) sqrt! !

!DhbVector methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
normalized
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30-Dec-99 "
	^(1 / self norm) * self! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromDouble: aDouble 
	^self * aDouble! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromFixedPoint: aFixedPoint 
	^self * aFixedPoint! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromFloat: aFloat 
	^self * aFloat! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromFraction: aFraction 
	^self * aFraction! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromInteger: anInteger 
	^self * anInteger! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productWithMatrix: aMatrix
		"Answers the product of aMatrix with the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix rowsCollect: [ :each | each * self]! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productWithVector: aVector
		"Answers the scalar product of aVector with the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	^self inject: 0
			into: [ :sum :each | n := n + 1. (aVector at: n) * each + sum]! !

!DhbVector methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
scalarProduct: aVector
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/5/99 "
	| product n |
	n := 0.
	product := self collect: [ :each | n := n + 1. (aVector at: n) * each].
	n := product size.
	[ n > 1]
		whileTrue:[ | i j|
					i := 1.
					j := n.
					[ i < j]
						whileTrue: [ product at: i put: ( product at: i) + ( product at: j).
									 j := j - 1.
									 i := i + 1.
								   ].
					n := i min: j.
				  ].
	^product at: 1! !

!DhbVector methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
scaleBy: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	1 to: self size do: [ :n | self at: n put: ( ( self at: n) * aNumber)].! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
tensorProduct: aVector
		"Answers the tensor product of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	self dimension = aVector dimension
		ifFalse:[ ^self error: 'Vector dimensions mismatch to build tensor product'].
	^DhbSymmetricMatrix rows: ( self collect: [ :a | aVector collect: [ :b | a * b]])! !

Array variableSubclass: #NNStore
	instanceVariableNames: 'freeIndex maxDistance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-KDTree'!

!NNStore commentStamp: 'WernerKassens 5/5/2011 14:40' prior: 0!
NNStore stores neighbours together with distances and uses the distances for sorting. 

can easily be subclassed if one needs an Array sorted depending on some outside data. see StupidNN for a simple example and comment on "withAll:" or just use "newFrom:".!

!NNStore class methodsFor: 'instance creation' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
new
"a NNStore  of size 0 can lead to errors that are probably unnecessary to catch"
^self new: 1  ! !

!NNStore class methodsFor: 'instance creation' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
new: anInt
^((super new: anInt) atAllPut: {Float infinity. nil}  )initialize  ! !

!NNStore class methodsFor: 'instance creation' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
newFrom: aCollectionWithSortingIndex
"example:
( NNStore newFrom:#(#(3 1.0) #(0 '2')) ) data.   #('2' 1.0)"
^((super newFrom: aCollectionWithSortingIndex )freeIndex: aCollectionWithSortingIndex size +1)sort  ! !

!NNStore class methodsFor: 'instance creation' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
withAll: aCollection
"this is the peferable form of instance creation for subclasses since freeIndex is correctly initialized this way. then 'sortFor:' must be overwritten and called at least once (an example is StupidNN). the other possibility is to use 'new:' and to fill NNStore subclass with 'add:' . the advantage is, that NNStore has not to be filled completely this way and the sorting logic can be applied outside of the subclass.
you cant enter the sorting index with this method. if you want to do that, use 'newFrom:' instead"
^(super withAll: (aCollection collect: [:coll| {nil. coll}]))freeIndex: aCollection size +1  ! !

!NNStore methodsFor: 'adding' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
add: distAndNeighbour
"if NNStore is full, further data will be added at the last position and the last data will be overwrittten"
	self at: (self isFull ifTrue: [ self size ] ifFalse: [ freeIndex ]) put: distAndNeighbour.
	freeIndex := freeIndex + 1.
	self isFull ifTrue: [ self sort ].! !

!NNStore methodsFor: 'accessing' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
completeData
"return the sorted data with distance (at first place)"
^self isFull ifTrue: [self] ifFalse: [(self copyFrom: 1 to: freeIndex  -1)sort: [:a :b| a first < b first ] ] .
! !

!NNStore methodsFor: 'copying' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
copyEmpty
	^ self species new: (self size min: (freeIndex -1 max: 1))! !

!NNStore methodsFor: 'accessing' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
data
"return the sorted data"
^self completeData  collect: [:e|e at:2] as:Array  ! !

!NNStore methodsFor: 'accessing' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
freeIndex: anInt
"method is necessary for subclass initialization (see withAll:)"
freeIndex :=anInt .! !

!NNStore methodsFor: 'initialize-release' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
initialize 
freeIndex :=1.
maxDistance :=Float infinity .! !

!NNStore methodsFor: 'testing' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
isEmpty 
^freeIndex =1! !

!NNStore methodsFor: 'testing' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
isFull
^freeIndex >self size ! !

!NNStore methodsFor: 'accessing' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
maxDistance 
"the maximum sorting index, if NNStore is full, if not then Infinity"
^maxDistance ! !

!NNStore methodsFor: 'sorting' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
sort
self sort: [:a :b| a first < b first ] .
maxDistance := self last first ! !

!NNStore methodsFor: 'sorting' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
sortFor: whatever
"should be overwritten by subclasses"

"self do:[:e|e at: 1 put: somethingDependingOnWhatever ]."
self sort .! !

NNStore variableSubclass: #StupidNN
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-KDTree'!

!StupidNN commentStamp: 'WernerKassens 5/5/2011 16:32' prior: 0!
StupidNN is a naive nearest neighbour search. KDTree is faster though if you have to do several searches.!

!StupidNN methodsFor: 'evaluating' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
nnSearch: aCollection i:anInt
"search for i nearest neighbours of vector aCollection "
|d|
self sortFor: aCollection   .
d :=self data .
anInt =1 ifTrue:[^d first ].
^d size >anInt ifTrue: [d copyFrom: 1 to: anInt]ifFalse: [d]  
  ! !

!StupidNN methodsFor: 'sorting' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
sortFor: aCollection
"sort nearest neighbours of aCollection "
|i|
self do:[:e| i :=aCollection -(e at:2) . e at: 1 put: (i*i)sum ].
self sort .! !

!IntegerTest methodsFor: '*Math-Extensions-Tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testChoose
"Abramowitz and Stegun Section 24.1.1 II B 'Checks'  p. 822"
	| sum n r s |
	n := 100 random.
	 r := 100 random.
	s := (n - r) abs + 1.
	sum := 0.
	((0 to: n) do: [:m | sum := sum + ((r choose: m) * (s choose: (n - m)))]).
	self should: [sum =  ((r + s) choose: n)].
	
	n := 100 random.
	r := 100 random + n.
	sum := 0.
	(0 to: n) do: [:m | sum := (-1 raisedTo: n - m)  * (r choose: m) + sum].
	self should: [sum = ((r - 1) choose: n)].
	
	n :=  100 random.
	self should: [(n choose: 0) = (n choose: n)].
	self should: [(n choose: 0) = 1].
	self should: [(n choose: 1) = n]. ! !

TestCase subclass: #BernoulliGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random-Tests'!

!BernoulliGeneratorTest commentStamp: '<historical>' prior: 0!
A BernoulliGeneratorTest is a test class for testing the behavior of BernoulliGenerator!

!BernoulliGeneratorTest methodsFor: 'tests-accessing' stamp: 'DanielUber 5/12/2012 21:28'!
testGenerator
	| g bern |
	g := LCRandom new.
	bern := BernoulliGenerator new.
	self should: [bern generator isKindOf: BernoulliGenerator defaultGenerator].
	bern generator: g.
	self should: [bern generator isKindOf: LCRandom]
	! !

!BernoulliGeneratorTest methodsFor: 'tests-accessing' stamp: 'DanielUber 5/12/2012 17:13'!
testNext! !

!BernoulliGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 5/12/2012 17:17'!
testNextYieldsOneOrZero
	| gen |
	gen := BernoulliGenerator fair.
	self should: [ | x | x := gen next.
				(x = 0) or: [x = 1]].! !

!BernoulliGeneratorTest methodsFor: 'tests-accessing' stamp: 'DanielUber 5/12/2012 17:18'!
testOneProbabilityGivesOneNext
	| g |
	g := BernoulliGenerator withProbability: 1.
	self should: [g next = 1].! !

!BernoulliGeneratorTest methodsFor: 'tests-accessing' stamp: 'DanielUber 5/12/2012 17:17'!
testZeroProbabilityGivesZeroNext
	| g |
	g := BernoulliGenerator withProbability: 0.0.
	self should: [g next = 0]! !

TestCase subclass: #BinomialGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random-Tests'!

!BinomialGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testBinomialGeneratorWithSuccessProbabilityOfOneAlwaysReturnNumberOfTrials
	| g numberOfTrials |
	g := BinomialGenerator new.
	numberOfTrials := 10.
	g
		n: numberOfTrials;
		p: 1.0.
	g generator: LCRandom new.
	self assert: g next = numberOfTrials.
	g generator: LehmerRandom new.
	self assert: g next = numberOfTrials.
	g generator: MersenneTwisterRandom new.
	self assert: g next = numberOfTrials! !

!BinomialGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testBinomialGeneratorWithSuccessProbabilityOfZeroAlwaysReturnZero
	| g numberOfTrials |
	g := BinomialGenerator new.
	numberOfTrials := 10.
	g
		n: numberOfTrials;
		p: 0.0.
	g generator: LCRandom new.
	self assert: g next = 0.
	g generator: LehmerRandom new.
	self assert: g next = 0.
	g generator: MersenneTwisterRandom new.
	self assert: g next = 0! !

TestCase subclass: #ComplexTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-ComplexTests'!

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testAbs
	"self run: #testAbs"
	"self debug: #testAbs"
	
	| c |
	c := (6 - 6 i).
	self assert: c abs  = 72 sqrt.
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/12/2012 21:42'!
testAbsSecure
	"self run: #testAbsSecure"
	"using equalsTo since absSecure returns a slightly different Float"
	| c |
	c := (6 - 6 i).
	self should: [c absSecure  equalsTo: 72 sqrt].
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/12/2012 21:46'!
testAdaptToCollectionAndSend
	"self run: #testAbsSecure"
	"using equalsTo since absSecure returns a slightly different Float"
	| c arr |
	c := (6 - 6 i).
	arr := #(0 1 2 3 4 5).
	self should: [(arr + c at: 1) = c ].
	self should: [(arr * c at: 2) = c].
		! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/12/2012 21:51'!
testAddPolynomial	
	| c  poly |
	c := (6 - 6 i).
	poly :=	DhbPolynomial coefficients: #(1 1 1).
	self should: [(poly + c at: 0 ) = (7 - 6 i)].
	self should: [(c + poly at: 0) = (7 - 6 i) ].! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testAdding
	"self run: #testAdding"
	
	| c |
	c := (5 - 6 i) + (-5 + 8 i).     "Complex with Complex"
	self assert: (c =  (0 + 2 i)).! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testArCosh
	| c |
	c := (2.5 + 0 i).
	self assert: (c arCosh real closeTo: c real arCosh).
	self assert: (c arCosh imaginary closeTo: 0).
	#(-0.5 -2 -3 0 0.5 2 3) do: [:real |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:imag |
		c := real + imag i.
		self assert: (c arCosh cosh real closeTo: c real).
		self assert: (c arCosh cosh imaginary closeTo: c imaginary).
		self deny: c arCosh real negative]]! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testArSinh
	| c |
	c := (2.5 + 0 i).
	self assert: (c arSinh real closeTo: c real arSinh).
	self assert: (c arSinh imaginary closeTo: 0).
	#(-0.5 -2 -3 0 0.5 2 3) do: [:real |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:imag |
		c := real + imag i.
		self assert: (c arSinh sinh real closeTo: c real).
		self assert: (c arSinh sinh imaginary closeTo: c imaginary)]]! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testArTanh
	| c |
	c := (0.5 + 0 i).
	self assert: (c arTanh real closeTo: c real arTanh).
	self assert: (c arTanh imaginary closeTo: 0).
	#(-0.5 -2 -3 0 0.5 2 3) do: [:real |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:imag |
		c := real + imag i.
		self assert: (c arTanh tanh real closeTo: c real).
		self assert: (c arTanh tanh imaginary closeTo: c imaginary)]]! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testArcCos
	| c |
	c := (0.5 + 0 i).
	self assert: (c arcCos real closeTo: c real arcCos).
	self assert: (c arcCos imaginary closeTo: 0).
	#(-0.5 -2 -3 0 0.5 2 3) do: [:real |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:imag |
		c := real + imag i.
		self assert: (c arcCos cos real closeTo: c real).
		self assert: (c arcCos cos imaginary closeTo: c imaginary)]]! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testArcCosPlusArcSin
	| c |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:real |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:imag |
		c := real + imag i.
		self assert: ((c arcCos + c arcSin) real closeTo: Float halfPi).
		self assert: ((c arcCos + c arcSin) imaginary closeTo: 0.0)]]! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testArcSin
	| c |
	c := (0.5 + 0 i).
	self assert: (c arcSin real closeTo: c real arcSin).
	self assert: (c arcSin imaginary closeTo: 0).
	#(-0.5 -2 -3 0 0.5 2 3) do: [:real |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:imag |
		c := real + imag i.
		self assert: (c arcSin sin real closeTo: c real).
		self assert: (c arcSin sin imaginary closeTo: c imaginary)]]! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testArcTan
	| c |
	c := (0.5 + 0 i).
	self assert: (c arcTan real closeTo: c real arcTan).
	self assert: (c arcTan imaginary closeTo: 0).
	#(-0.5 -2 -3 0 0.5 2 3) do: [:real |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:imag |
		c := real + imag i.
		self assert: (c arcTan tan real closeTo: c real).
		self assert: (c arcTan tan imaginary closeTo: c imaginary)]]! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/12/2012 22:03'!
testArcTanDenominator
	| c1 c2 |
	
	c1 :=  1 i.
	c2 := 0.
	self should: [(c1 arcTan: c1) = (Float pi / 4)].
	self should: [(c2 arcTan: c1 ) = 0].
	self should: [(c2 arcTan: c1* c1) = Float pi].
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testArg
	"self run: #testArg"
	"self debug: #testArg"
	
	| c |
	c := (0 + 5 i) .
	self assert: c arg  = (Float pi/ 2).
	! !

!ComplexTest methodsFor: 'testing - bugs' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testBug1

	self assert: (0.5 * (2+0i) ln) exp = (0.5 * 2 ln) exp.! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testComplexAsComplex
	| ineg |
	ineg := -1 i.
	self assert: ineg asComplex == ineg.! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testComplexCollection
	"self run: #testComplexCollection"
	"self debug: #testComplexCollection"
	
	| array array2 |
	array := Array with: 1 + 2i with:  3 + 4i with: 5 + 6i.
	array2 := 2 * array.
	array with:  array2 do: [:one :two | self assert: (2 * one) = two ] ! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testConjugated
	
	| c cc |
	c := (5 - 6 i).
	cc := c conjugated.
	self assert: cc real = c real.
	self assert: cc imaginary = c imaginary negated.! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testConversion
	"self run: #testConversion"
	"self debug: #testConversion"
	
	self assert: ((1 + 2i) + 1) =  (2 + 2 i).
	self assert: (1 + (1 + 2i)) =  (2 + 2 i).
	self assert: ((1 + 2i) + 1.0) =  (2.0 + 2 i).
	self assert: (1.0 + (1 + 2i)) =  (2.0 + 2 i).
	self assert: ((1 + 2i) + (2/3)) = ((5/3) + 2 i ).
	self assert: ((2/3) + (1 + 2i)) = ((5/3) + 2 i )! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testCos
	| c c2 |
	c := (2 + 0 i).
	self assert: (c cos real closeTo: c real cos).
	self assert: (c cos imaginary closeTo: 0).
	c := (2 + 3 i).
	c2 := c i exp + c i negated exp / 2.
	self assert: (c cos real closeTo: c2 real).
	self assert: (c cos imaginary closeTo: c2 imaginary).! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testCos2PlusSin2
	| c |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:real |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:imag |
		c := real + imag i.
		self assert: ((c cos squared + c sin squared) real closeTo: 1).
		self assert: ((c cos squared + c sin squared) imaginary closeTo: 0.0)]]! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testCosh
	| c c2 |
	c := (2 + 0 i).
	self assert: (c cosh real closeTo: c real cosh).
	self assert: (c cosh imaginary closeTo: 0).
	c := (2 + 3 i).
	c2 := c exp + c negated exp / 2.
	self assert: (c cosh real closeTo: c2 real).
	self assert: (c cosh imaginary closeTo: c2 imaginary).
	c2 := c i cos.
	self assert: (c cosh real closeTo: c2 real).
	self assert: (c cosh imaginary closeTo: c2 imaginary).! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testCosh2MinusSinh2
	| c |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:real |
	#(-0.5 -2 -3 0 0.5 2 3) do: [:imag |
		c := real + imag i.
		self assert: ((c cosh squared - c sinh squared) real closeTo: 1).
		self assert: ((c cosh squared - c sinh squared) imaginary closeTo: 0.0)]]! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testCreation
	"self run: #testCreation"
	
	| c |
	c := 5 i.
	self assert: (c real = 0).
	self assert: (c imaginary = 5).
	
	c := 6 + 7 i.
	self assert: (c real = 6).
	self assert: ( c imaginary = 7).
	
	c := 5.6 - 8 i.
	self assert: (c real = 5.6).
	self assert: (c imaginary = -8).
	
	c := Complex real: 10 imaginary: 5.
	self assert: (c real = 10).
	self assert: (c imaginary = 5).
	
	c := Complex abs: 5 arg: (Float pi/2).
	self assert: (c real rounded = 0).
	self assert: (c imaginary = 5).
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/12/2012 22:12'!
testDividingPolynomial
	| c poly |
	c := (4 + 4 i).
	poly := DhbPolynomial coefficients: #(1 0 1).
	self should: [(poly / c) = ((1/c) * poly)]
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testDivision1
	"self run: #testDivision1"
	"self debug: #testDivision1"
	
	| c1 c2 quotient |
	c1 := 2.0e252 + 3.0e70 i.
	c2 := c1.
	quotient := c1 / c2.
 	self deny: (quotient - 1) isZero.
	
	"This test fails due to the wonders of floating point arithmetic. 
	 Please have a look at Complex>>divideSecureBy: and #divideFastAndSecureBy:
	how this can be avoided."
	
! !

!ComplexTest methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testEquality
	"self run: #testEquality"
	"self debug: #testEquality"
	
	self assert: 0i = 0.
	self assert: (2 - 5i) = ((1 -4 i) + (1 - 1i)).
	self assert: 0i isZero.
	self deny: (1 + 3 i) = 1.
	self deny: (1 + 3 i) = (1 + 2i).

"Some more stuff"
	self deny: (1 i) = nil.
	self deny: nil = (1 i).

	self deny: (1 i) = #(1 2 3).
	self deny: #(1 2 3) = (1 i).

	self deny: (1 i) = 0.
	self deny: 0 = (1 i).

	self assert:  (1 + 0 i) = 1.
	self assert:  1 = (1+ 0 i).

	self assert:  (1 + 0 i) = 1.0.
	self assert:  1.0 = (1+ 0 i).

	self assert:  (1/2 + 0 i) = (1/2).
	self assert:  (1/2) = (1/2+ 0 i).! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testHash
	| aComplex |
	aComplex := 2 - 3 i.
	self assert: aComplex copy hash = aComplex hash.! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testLn
	self assert: (Float e + 0 i) ln = Float e ln "See Bug 1815 on Mantis"! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/12/2012 22:15'!
testLog
	self assert: ((Float e + 0 i) log: Float e) = Float e ln. "See Bug 1815 on Mantis"
	self should: [((2 + 0 i ) log: 2) = 1].! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMultiplyByI
	
	| c |
	c := (5 - 6 i).
	self assert: (c * 1i = c i)! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testNegated
	"self run: #testNegated"
	"self debug: #testNegated"
	
	| c |
	c := (2 + 5 i) .
	self assert: c negated  = (-2 - 5i).
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/13/2012 02:18'!
testNew
	| c |
	c := Complex new.
	self should: [ c real = 0].
	self should: [c imaginary = 0].! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testNumberAsComplex
	| minusOne |
	minusOne := -1 asComplex.
	self assert: minusOne real = -1.
	self assert: minusOne imaginary = 0.
	self assert: minusOne conjugated = minusOne.
	self shouldnt: [minusOne sqrt] raise: Error! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testOne
	| one |
	one := Complex one.
	self assert: one isComplex.
	self assert: one real = 1.
	self assert: one imaginary = 0.! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/12/2012 22:19'!
testPrintOn
	| c |
	c := (1 + 2 i).
	Transcript show: 'ComplexTest>>testPrintOn' ; cr; show: c ; cr.
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/13/2012 02:13'!
testProductWithVector
| v c |
c := (1 + 1 i).
v := DhbVector new: 1.
v at: 1 put: 1.
"Next line won't work without DhbVector>>adaptToComplex:andSend:"
"self should: [((c * v) at: 1) = c ]."
self should: [(v * c at: 1) = c].
! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testRaisedTo
	
	| c c3 |
	c := (5 - 6 i).
	c3 := (c raisedTo: 0.2) raisedTo: 5.
	self assert: (c3 real closeTo: c real).
	self assert: (c3 imaginary closeTo: c imaginary).! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testRaisedToInteger
	
	| c c3 |
	c := (5 - 6 i).
	c3 := (c * c * c).
	self assert: (c3 = (c raisedToInteger: 3)).
	self assert: (c3 reciprocal = (c raisedToInteger: -3)).! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testReciprocal
	"self run: #testReciprocal"
	"self debug: #testReciprocal"
	
	| c |
	c := (2 + 5 i).
	self assert: c reciprocal  = ((2/29) - (5/29)i).
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testReciprocalError
	"self run: #testReciprocalError"
	"self debug: #testReciprocalError"
	
	| c |
	c := (0 i).
	self should: [c reciprocal] raise: ZeroDivide
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testSecureDivision1
	"self run: #testSecureDivision1"
	"self debug: #testSecureDivision1"
	
	| c1 c2 quotient |
	c1 := 2.0e252 + 3.0e70 i.
	c2 := c1.
	quotient := c1 divideSecureBy: c2.
	self assert: (quotient - 1) isZero.
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testSecureDivision2
	"self run: #testSecureDivision2"
	"self debug: #testSecureDivision2"
	
	| c1 c2 quotient |
 	c1 := 2.0e252 + 3.0e70 i.
 	c2 := c1.
 	quotient := c1 divideFastAndSecureBy: c2.
	self assert: (quotient - 1) isZero.
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testSin
	| c c2 |
	c := (2 + 0 i).
	self assert: (c sin real closeTo: c real sin).
	self assert: (c sin imaginary closeTo: 0).
	c := 2 + 3 i.
	c2 := c i exp - c i negated exp / 2 i.
	self assert: (c sin real closeTo: c2 real).
	self assert: (c sin imaginary closeTo: c2 imaginary).! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testSinh
	| c c2 |
	c := (2 + 0 i).
	self assert: (c sinh real closeTo: c real sinh).
	self assert: (c sinh imaginary closeTo: 0).
	c := 2 + 3 i.
	c := c cosh squared - c sinh squared.
	self assert: (c real closeTo: 1).
	self assert: (c imaginary closeTo: 0).
	c2 := c exp - c negated exp / 2.
	self assert: (c sinh real closeTo: c2 real).
	self assert: (c sinh imaginary closeTo: c2 imaginary).
	c2 := c i sin i negated.
	self assert: (c sinh real closeTo: c2 real).
	self assert: (c sinh imaginary closeTo: c2 imaginary).! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testSquared
	"self run: #testSquared"
	"self debug: #testSquared"
	
	| c c2 |
	c := (6 - 6 i).
	c2 := (c squared).
	self assert: c2 imaginary = -72.
	self assert: c2 real = 0.! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/13/2012 02:16'!
testSubtractToPolynomial
	| c poly |
	poly := DhbPolynomial coefficients: #(1 2 3).
	c := (1 + 3 i).
	self should: [(c - poly  at: 0 ) = 3 i].
	self should: [(poly - c at: 0) = -3 i ].! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testTan
	| c c2 |
	c := (2 + 0 i).
	self assert: (c tan real closeTo: c real tan).
	self assert: (c tan imaginary closeTo: 0).
	c := 2 + 3 i.
	c2 := c sin / c cos.
	self assert: (c2 real closeTo: c tan real).
	self assert: (c2 imaginary closeTo: c tan imaginary).! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testTanh
	| c c2 |
	c := (2 + 0 i).
	self assert: (c tanh real closeTo: c real tanh).
	self assert: (c tanh imaginary closeTo: 0).
	c := 2 + 3 i.
	c2 := c sinh / c cosh.
	self assert: (c2 real closeTo: c tanh real).
	self assert: (c2 imaginary closeTo: c tanh imaginary).! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/13/2012 02:19'!
testTimesPolynomial
	| c poly |
	c := 1 + 1 i.
	poly := DhbPolynomial coefficients: #( 1 ).
	self should: [(c * poly at: 0) = c ].
	self should: [(poly * c at: 0) = c ].
	! !

!ComplexTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testZero
	| z |
	z := Complex zero.
	self assert: z isZero.
	self assert: z isComplex.
	self assert: z real isZero.
	self assert: z imaginary isZero.! !

TestCase subclass: #ConstantGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random-Tests'!

!ConstantGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 5/13/2012 02:23'!
testConstantGenerator
	| g |
	g := ConstantGenerator new.
	g constant: 1.
	self assert: g next = g next.
	self assert: g next = g next.
	self assert: g next = 1.
	
	g := ConstantGenerator constant: 2.
	self assert: g next = 2.! !

TestCase subclass: #DhbNumericalMethodsTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical-Tests'!

!DhbNumericalMethodsTestCase methodsFor: 'privateMethods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulateAround: aVector size: aNumber into: aCollection
	"Private - Generate a random point around the given center and insert it into the collection.
					aNumber is the sigma for the distance to the center"
	| r phi psi localVector |

	r := ( DhbNormalDistribution new: 0 sigma: aNumber) random.
	phi := Float pi random.
	psi := Float pi random.
	localVector := DhbVector new: 3.
	localVector at: 1 put: ( phi sin * psi sin * r );
					   at: 2 put: ( phi cos * psi sin * r );
					   at: 3 put: ( psi cos * r ).
	aCollection add: (localVector + aVector).! !

!DhbNumericalMethodsTestCase methodsFor: 'privateMethods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
generatedPoints: anInteger
	"Private - Generate random points into aCollection. 3 clusters are used"
	| centers results |
	centers := Array new: 3.
	centers at: 1 put: #( 200 200  200) asVector;
				 at: 2 put: #(-200 200  200) asVector;
				 at: 3 put: #( 200 200 -200) asVector.
	results := OrderedCollection new.
	anInteger timesRepeat: [ self accumulateAround: ( centers at: (3 random + 1)) size: 1 into: results].
	^results! !

!DhbNumericalMethodsTestCase methodsFor: 'privateMethods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setUp
	"Reset the seed of the random numbers (to get consistent results)"
	DhbMitchellMooreGenerator reset: 0.
	"Reset the FloatingPointMachine to make coverage consistent"
	DhbFloatingPointMachine reset.! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testBeta
	"Code example 2.14"
	| value |

	value := 2.5 gamma * 5.5 gamma / 8 gamma.
	self should: [ ((2.5 beta: 5.5)  - value) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testBetaLog
	"Code example 2.15"
	| value |
	value := ( 2.5 gamma * 5.5 gamma / 8 gamma) ln.
	self should: [ ((2.5 logBeta: 5.5)  - value) abs < 1.0e-13].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testBissection
	"Code Example 5.1"
	| zeroFinder result |
	zeroFinder := DhbBisectionZeroFinder function: [ :x | x errorFunction - 0.9].
	zeroFinder setPositiveX: 10.0; setNegativeX: 0.0.
	result := zeroFinder evaluate.
	self should: [ zeroFinder hasConverged].
	self should: [ (result -1.28155193291605) abs < 1.0e-14]! !

!DhbNumericalMethodsTestCase methodsFor: 'data mining' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testClusterCovariance

	| dataServer clusters finder |

	dataServer := DhbMemoryBasedDataServer new.
	dataServer data: ( self generatedPoints: 1000).
	finder := DhbClusterFinder new: 5 server: dataServer type: DhbCovarianceCluster.
	finder minimumRelativeClusterSize: 0.1.
	clusters := finder evaluate.
	self should: [ clusters size = 3].! !

!DhbNumericalMethodsTestCase methodsFor: 'data mining' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testClusterEuclidean

	| dataServer clusters finder |
	dataServer := DhbMemoryBasedDataServer new.
	dataServer data: ( self generatedPoints: 1000).
	finder := DhbClusterFinder new: 5 server: dataServer type: DhbEuclideanCluster.
	finder minimumRelativeClusterSize: 0.15.
	clusters := finder evaluate.
	self should: [ clusters size = 3].! !

!DhbNumericalMethodsTestCase methodsFor: 'data mining' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testCovarianceAccumulation
	"Code example 12.2"
	| accumulator average covarianceMatrix |
	accumulator := DhbCovarianceAccumulator new: 3.
	#( 
			(1 2 3)
			( 2 3 4)
			( 1 3 2)
			( 4 3 1)
			( 1 3 1)
			( 1 4 2)
			( 3 1 2)
			( 3 4 2)
		)
	do: [ :x | accumulator accumulate: x asVector].
	average := accumulator average.
	self should: [ ( average at: 1) equalsTo: 2.0].
	self should: [ ( average at: 2) equalsTo: 2.875].
	self should: [ ( average at: 3) equalsTo: 2.125].
	covarianceMatrix := accumulator covarianceMatrix.
	self should: [ ((covarianceMatrix rowAt: 1) at: 1) equalsTo: 1.25].
	self should: [ ((covarianceMatrix rowAt: 1) at: 2) equalsTo: -0.125].
	self should: [ ((covarianceMatrix rowAt: 2) at: 1) equalsTo: -0.125].
	self should: [ ((covarianceMatrix rowAt: 1) at: 3) equalsTo: -0.25].
	self should: [ ((covarianceMatrix rowAt: 3) at: 1) equalsTo: -0.25].
	self should: [ ((covarianceMatrix rowAt: 2) at: 2) equalsTo: 0.859375].
	self should: [ ((covarianceMatrix rowAt: 2) at: 3) equalsTo: -0.109375].
	self should: [ ((covarianceMatrix rowAt: 3) at: 2) equalsTo: -0.109375].
	self should: [ ((covarianceMatrix rowAt: 3) at: 3) equalsTo: 0.859375].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testDeterminant

	| m |
	m := DhbMatrix rows: #( (3 2 4) (2 -5 -1) ( 1 -2 2)).
	self should: [ m determinant = -42].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testEigenvalues
	"Code Example 8.15"
	| m charPol roots eigenvalues finder |
	m := DhbMatrix rows: #( ( 3 -2 0) (-2 7 1) (0 1 5)).
	charPol := DhbPolynomial coefficients: #(82 -66 15 -1).
	roots := charPol roots asSortedCollection asArray reverse.
	finder := DhbJacobiTransformation matrix: m.
	finder desiredPrecision: 1.0e-09.
	eigenvalues := finder evaluate.
	self should: [ eigenvalues size = 3].
	self should: [ ((roots at: 1) - (eigenvalues at: 1)) abs < 1.0e-09].
	self should: [ ((roots at: 2) - (eigenvalues at: 2)) abs < 1.0e-09].
	self should: [ ((roots at: 3) - (eigenvalues at: 3)) abs < 1.0e-09].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testEigenvaluesLargest
	"Code Example 8.13"
	| m charPol roots eigenvalue finder |
	m := DhbMatrix rows: #( ( 3 -2 0) (-2 7 1) (0 1 5)).
	charPol := DhbPolynomial coefficients: #(82 -66 15 -1).
	roots := charPol roots asSortedCollection asArray reverse.
	finder := DhbLargestEigenValueFinder matrix: m.
	finder desiredPrecision: 1.0e-08.
	eigenvalue := finder evaluate.
	self should: [ ((roots at: 1) - eigenvalue) abs < 1.0e-08].
	finder := finder nextLargestEigenValueFinder.
	eigenvalue := finder evaluate.
	self should: [ ((roots at: 2) - eigenvalue) abs < 1.0e-08].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testErrorFunctionCentile
	"Code example 2.5"
	| weight average stDev centile |
	weight := 2.85.
	average := 3.39.
	stDev := 0.44.
	centile := (( weight - average) / stDev) errorFunction * 100.
	self should: [ ( centile - 10.986012) abs < 0.000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testFTest

	| accC accMM confidenceLevel|
	accC := DhbStatisticalMoments new.
	#( 5.56 5.89 4.66 5.69 5.34 4.79 4.80 7.86 3.64 5.70 )
		do: [ :x | accC accumulate: x].
	accMM := DhbStatisticalMoments new.
	#( 7.48 6.75 3.77 5.71 7.25 4.73 6.23 5.60 5.94 4.58 )
		do: [ :x | accMM accumulate: x].
	confidenceLevel := accC fConfidenceLevel: accMM.
	self should: [ (accC average - 5.393) abs < 0.000000001].
	self should: [ (accC standardDeviation - 1.0990809292) abs < 0.000000001].
	self should: [ (accMM average - 5.804) abs < 0.000000001].
	self should: [ (accMM standardDeviation - 1.19415428) abs < 0.000000001].
	self should: [ (confidenceLevel - 79.8147614536) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testGamma
	"Code example 2.10"
	| value |
	value := Float pi sqrt * 3 / 4.
	self should: [ (2.5 gamma  - value) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testGammaDistribution
	| dist |
	dist := DhbGammaDistribution shape: 3.4 scale: 1.7.
	self should: [ (dist average - (3.4 * 1.7)) abs < 0.000000001].
	self should: [ (dist standardDeviation - (3.4 sqrt *1.7)) abs < 0.000000001].
	self should: [ ((dist value: 4.5) - 0.1446067652) abs < 0.000000001].
	self should: [ ((dist distributionValue: 4.5) - 0.3982869736) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testGammaLog
	"Code example 2.11"
	| value |
	value := 2.5 gamma ln.
	self should: [ (2.5 logGamma  - value) abs < 1.0e-13].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testGammaLow

	| value |
	value := Float pi sqrt / 2.
	self should: [ ((3/2) gamma  - value) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testGammaNegative

	| value |
	value := Float pi / ( 1.5 gamma * (Float pi / -2) sin).
	self should: [ ((-1/2) gamma  - value) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testHistogram
	| histogram |
	histogram := DhbHistogram new.
	histogram setRangeFrom: 0.0 to: 48.0 bins: 8.
	#( 36 13 27 16 33 24 4 20 15 23 37 23 31 15 47 22  6 15 41 22  14 14  31 42 3 42 22 8  37 41 )
		do: [ :x | histogram accumulate: x].
	histogram accumulate: -1; accumulate: 55; accumulate: 56.
	self should: [ histogram count = 30].
	self should: [ histogram underflow = 1].
	self should: [ histogram overflow = 2].
	self should: [ ( histogram countAt: 1) = 3].
	self should: [ ( histogram countAt: 8.5) = 4].
	self should: [ ( histogram countAt: 16) = 8].
	self should: [ ( histogram countAt: 23.5) = 4].
	self should: [ ( histogram countAt: 31) = 6].
	self should: [ ( histogram countAt: 38.5) = 4].
	self should: [ ( histogram countAt: 46) = 1].
	self should: [ (histogram average - 24.1333333333) abs < 0.000000001].
	self should: [ (histogram standardDeviation - 12.461619237603) abs < 0.000000001].
	self should: [ (histogram skewness - 0.116659884676) abs < 0.000000001].
	self should: [ (histogram kurtosis + 1.004665562311) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testIncompleteBetaFunction

	| function |
	function := DhbIncompleteBetaFunction shape: 2 shape: 5.
	self should: [ ( (function value: 0.8) - 0.9984) abs < 0.00001].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testIncompleteGammaFunction

	| function |
	function := DhbIncompleteGammaFunction shape: 2.
	self should: [ ( (function value: 2) - 0.59399414981) abs < 0.00000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testIntegrationRomberg

	| integrator ln2 ln3|
	integrator := DhbRombergIntegrator function: [ :x | 1.0 / x] from: 1 to: 2.
	ln2 := integrator evaluate.
	integrator from: 1 to: 3.
	ln3 := integrator evaluate.
	self should: [ ( 2.0 ln - ln2) abs < ( 2 * integrator precision)].
	self should: [ ( 3.0 ln - ln3) abs < ( 2 * integrator precision)].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testIntegrationSimpson

	| integrator ln2 ln3|
	integrator := DhbSimpsonIntegrator function: [ :x | 1.0 / x] from: 1 to: 2.
	ln2 := integrator evaluate.
	integrator from: 1 to: 3.
	ln3 := integrator evaluate.
	self should: [ ( 2.0 ln - ln2) abs < integrator precision].
	self should: [ ( 3.0 ln - ln3) abs < integrator precision].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testIntegrationTrapeze
	"Code Example 6.1"
	| integrator ln2 ln3|
	integrator := DhbTrapezeIntegrator function: [ :x | 1.0 / x] from: 1 to: 2.
	ln2 := integrator evaluate.
	integrator from: 1 to: 3.
	ln3 := integrator evaluate.
	self should: [ ( 2.0 ln - ln2) abs < integrator precision].
	self should: [ ( 3.0 ln - ln3) abs < integrator precision].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testInterpolationBulirschStoer

	| interpolator |
	interpolator := DhbBulirschStoerInterpolator new.
	1 to: 45 by: 2 do:
		[ :x | interpolator add: x @ (x degreesToRadians sin)].
	self should: [ ((interpolator value: 8) - (8 degreesToRadians sin)) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testInterpolationLagrange
	"Code example 3.2"
	| interpolator |
	interpolator := DhbLagrangeInterpolator new.
	1 to: 45 by: 2 do:
		[ :x | interpolator add: x @ (x degreesToRadians sin)].
	self should: [ ((interpolator value: 8) - (8 degreesToRadians sin)) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testInterpolationLagrangeLinear
	"Code example 3.1"
	| interpolator |
	interpolator := DhbLagrangeInterpolator points: (Array with: 1 @ 2 with: 3 @ 1 ).
	self should: [ ((interpolator value: 2.2) - 1.4) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testInterpolationNeville

	| interpolator |

	interpolator := DhbNevilleInterpolator new.
	1 to: 45 by: 2 do:
		[ :x | interpolator add: x @ (x degreesToRadians sin)].
	self should: [ ((interpolator value: 8) - (8 degreesToRadians sin)) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testInterpolationNevilleLinear
	"Code example 3.1"
	| interpolator |
	interpolator := DhbNevilleInterpolator points: (Array with: 1 @ 2 with: 3 @ 1 ).
	self should: [ ((interpolator value: 2.2) - 1.4) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testInterpolationNewton

	| interpolator |
	interpolator := DhbNewtonInterpolator new.
	1 to: 45 by: 2 do:
		[ :x | interpolator add: x @ (x degreesToRadians sin)].
	self should: [ ((interpolator value: 8) - (8 degreesToRadians sin)) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testInterpolationNewtonLinear
	"Code example 3.1"
	| interpolator |
	interpolator := DhbNewtonInterpolator points: (Array with: 1 @ 2 with: 3 @ 1 ).
	self should: [ ((interpolator value: 2.2) - 1.4) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testInterpolationSpline

	| interpolator |
	interpolator := DhbSplineInterpolator new.
	1 to: 45 by: 2 do:
		[ :x | interpolator add: x @ (x degreesToRadians sin)].
	self should: [ ((interpolator value: 8) - (8 degreesToRadians sin)) abs < 1.0e-7].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testInterpolationSplineLinear
	"Code example 3.1"
	| interpolator |
	interpolator := DhbSplineInterpolator points: (Array with: 1 @ 2 with: 3 @ 1 ).
	self should: [ ((interpolator value: 2.2) - 1.4) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testLUPDecomposition
	"Code Example 8.10"
	| s sol1 sol2 |
	s := DhbLUPDecomposition equations:
							#( (3 2 4) (2 -5 -1) ( 1 -2 2)).
	sol1 := s solve: #(16 6 10).
	sol2 := s solve: #(7 10 9).
	self should: [ sol1 size = 3].
	self should: [ (sol1 at: 1) = 2].
	self should: [ (sol1 at: 2) = -1].
	self should: [ (sol1 at: 3) = 3].
	self should: [ sol2 size = 3].
	self should: [ (sol2 at: 1) = 1].
	self should: [ (sol2 at: 2) = -2].
	self should: [ (sol2 at: 3) = 2].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testLeastSquare
	"Code example 10.9"
	"Note: the seemingly large error on the fit results is due to the binning of the histogram."
	| count shape scale genDistr hist fit fittedDistr parameters |
	count := 10000.
	shape := 0.
	scale := 1.
	hist := DhbHistogram new.
	hist freeExtent: true.
	genDistr := DhbFisherTippettDistribution shape: shape scale: scale.
	count timesRepeat: [ hist accumulate: genDistr random].
	fit := DhbLeastSquareFit histogram: hist distributionClass: DhbFisherTippettDistribution.
	fittedDistr := fit evaluate.
	parameters := fittedDistr parameters.
	self should: [ ((parameters at: 1) - shape) abs < 0.1].
	self should: [ ((parameters at: 2) - scale) abs < 0.1].
	self should: [ ((parameters at: 3) - count) abs < 100].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testLeastSquarePolynomial
	"Code example 10.5"
	| fit estimation |
	fit := DhbPolynomialLeastSquareFit new: 3.
	fit
		add: (DhbWeightedPoint point: 1 @ 2.0);
		add: (DhbWeightedPoint point: 2 @ 21.0);
		add: (DhbWeightedPoint point: 3 @ 72.0);
		add: (DhbWeightedPoint point: 4 @ 173.0);
		add: (DhbWeightedPoint point: 5 @ 342.0);
		add: (DhbWeightedPoint point: 6 @ 597.0);
		add: (DhbWeightedPoint point: 7 @ 956.0);
		add: (DhbWeightedPoint point: 8 @ 1437.0);
		add: (DhbWeightedPoint point: 9 @ 2058.0);
		add: (DhbWeightedPoint point: 10 @ 2837.0).
	estimation := fit evaluate.
	self should: [ ((estimation value: 4.5)- 247.875) abs < 0.000000001].
	self should: [ ((estimation error: 4.5) -  5.215298e-1) abs < 0.00001].
	self should: [ ((estimation value: 7.15)- 1019.932625) abs < (estimation error: 7.15)].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testLinearEquations
	"Code Example 8.6"
	| s sol1 sol2 |
	s := DhbLinearEquationSystem equations:
							#( (3 2 4) (2 -5 -1) ( 1 -2 2))
				constants: #( (16 6 10)
									  (7 10 9)).
	sol1 := s solutionAt: 1.
	sol2 := s solutionAt: 2.
	self should: [ sol1 size = 3].
	self should: [ (sol1 at: 1) = 2].
	self should: [ (sol1 at: 2) = -1].
	self should: [ (sol1 at: 3) = 3].
	self should: [ sol2 size = 3].
	self should: [ (sol2 at: 1) = 1].
	self should: [ (sol2 at: 2) = -2].
	self should: [ (sol2 at: 3) = 2].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testLinearEquationsSingle

	| s sol |
	s := DhbLinearEquationSystem equations:
							#( (1 2 0) (3 5 4) (5 6 3)) constant: #(0.1 12.5 10.3).
	sol := s solution.
	self should: [ sol size = 3].
	self should: [ (sol at: 1) equalsTo: 0.5].
	self should: [ (sol at: 2) equalsTo: -0.2].
	self should: [ (sol at: 3) equalsTo:  3.0].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testLinearEquationsSingular

	| s sol |
	s := DhbLinearEquationSystem equations:
							#( (1 2 0) (10 12 6) (5 6 3)) constant: #(0.1 12.5 10.3).
	sol := s solution.
	self should: [ sol isNil].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testLinearRegression
	"Code example 10.5"
	| linReg estimation |
	linReg := DhbLinearRegression new.
	linReg
		add: 1 @ 0.72;
		add: 2 @ 3.25;
		add: 3 @ 5.75;
		add: 4 @ 8.21;
		add: 5 @ 10.71;
		add: 6 @ 13.38;
		add: 7 @ 15.82;
		add: 8 @ 18.39;
		add: 9 @ 20.72;
		add: 10 @ 23.38.
	self should: [ (linReg slope - 2.514727272727) abs < 0.000000000001].
	self should: [ (linReg intercept + 1.798) abs < 0.000000000001].
	self should: [ (linReg correlationCoefficient - 0.999966922113) abs < 0.000000000001].
	estimation := linReg asEstimatedPolynomial.
	self should: [ ((estimation value: 4.5)- 9.5182727272727) abs < 0.000000000001].
	self should: [ ((estimation value: 7.15)-  16.1823) abs < 0.000000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'precision' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMachinePrecision
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach machinePrecision > 0].
self should: [mach machinePrecision < 1].
self should: [mach negativeMachinePrecision > 0].
self should: [mach negativeMachinePrecision < 1].
self should: [(mach radix raisedTo: mach negativeMachinePrecision) = 1.0].
self should: [(mach radix raisedTo: mach machinePrecision) = 1.0].! !

!DhbNumericalMethodsTestCase methodsFor: 'precision' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMachinePrecisionIsFloatPrecision
"sanity check. Take logs to find the exponent, then compare to Float's method"

| prec |
prec := (DhbFloatingPointMachine new machinePrecision ln / DhbFloatingPointMachine new radix ln ) negated .
self should: [prec = Float precision].! !

!DhbNumericalMethodsTestCase methodsFor: 'precision' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMachinePrecisionLargestNumberGreaterThanAnyFractionOfItself
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach largestNumber * (1 - mach defaultNumericalPrecision) < mach largestNumber].
! !

!DhbNumericalMethodsTestCase methodsFor: 'precision' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMachinePrecisionLargestNumberIsLargest
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach largestNumber * (1 + mach defaultNumericalPrecision) = Float infinity].
! !

!DhbNumericalMethodsTestCase methodsFor: 'precision' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMachinePrecisionLargestNumberNotInfinity
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach largestNumber < Float infinity].! !

!DhbNumericalMethodsTestCase methodsFor: 'precision' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMachinePrecisionSmallNumberLargerThanSmallestNumber
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach smallestNumber < mach smallNumber].! !

!DhbNumericalMethodsTestCase methodsFor: 'precision' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMachinePrecisionSmallNumberNotZero
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach smallNumber > 0.0].! !

!DhbNumericalMethodsTestCase methodsFor: 'precision' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMachinePrecisionSmallestNumberNotZero
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach smallestNumber > 0.0].! !

!DhbNumericalMethodsTestCase methodsFor: 'data mining' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMahalanobisCenter
	"Code example 12.5"
	| center distance|
	center := DhbMahalanobisCenter new: 3.
	#( 
			(1 2 3)
			( 2 3 4)
			( 1 3 2)
			( 4 3 1)
			( 1 3 1)
			( 1 4 2)
			( 3 1 2)
			( 3 4 2)
		)
	do: [ :x | center accumulate: x asVector].
	center computeParameters.
	distance := center distanceTo: #(1 2 3) asVector.
	self should: [ distance equalsTo: 2.26602282704126].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMatrixInversionSmall

	| m c |
	m := DhbMatrix rows: #( (3 2 4) (2 -5 -1) ( 1 -2 2)).
	c := m inverse * m.
	self should: [ c numberOfRows = 3].
	self should: [ c numberOfColumns = 3].
	self should: [ ((c rowAt: 1) at: 1) = 1].
	self should: [ ((c rowAt: 1) at: 2) = 0].
	self should: [ ((c rowAt: 1) at: 3) = 0].
	self should: [ ((c rowAt: 2) at: 1) = 0].
	self should: [ ((c rowAt: 2) at: 2) = 1].
	self should: [ ((c rowAt: 2) at: 3) = 0].
	self should: [ ((c rowAt: 3) at: 1) = 0].
	self should: [ ((c rowAt: 3) at: 2) = 0].
	self should: [ ((c rowAt: 3) at: 3) = 1].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMatrixOperation
	"Code Example 8.1"
	| a b c |
	a := DhbMatrix rows: #( ( 1 0 1) (-1 -2 3)).
	b := DhbMatrix rows: #( ( 1 2 3) (-2 1 7) (5 6 7)).
	c := a * b.
	self should: [ c numberOfRows = 2].
	self should: [ c numberOfColumns = 3].
	self should: [ ((c rowAt: 1) at: 1) = 6].
	self should: [ ((c rowAt: 1) at: 2) = 8].
	self should: [ ((c rowAt: 1) at: 3) = 10].
	self should: [ ((c rowAt: 2) at: 1) = 18].
	self should: [ ((c rowAt: 2) at: 2) = 14].
	self should: [ ((c rowAt: 2) at: 3) = 4].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testMaximumLikelihood
	"Code example 10.11"
	"Note: the seemingly large error on the fit results is due to the binning of the histogram."
	| count shape scale genDistr hist fit fittedDistr parameters |
	count := 10000.
	shape := 0.
	scale := 1.
	hist := DhbHistogram new.
	hist freeExtent: true.
	genDistr := DhbFisherTippettDistribution shape: shape scale: scale.
	count timesRepeat: [ hist accumulate: genDistr random].
	fit := DhbMaximumLikekihoodHistogramFit histogram: hist distributionClass: DhbFisherTippettDistribution.
	fittedDistr := fit evaluate.
	parameters := fittedDistr parameters.
	self should: [ ((parameters at: 1) - shape) abs < 0.1].
	self should: [ ((parameters at: 2) - scale) abs < 0.1].
	self should: [ ((parameters at: 3) - count) abs < 100].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testNewtonZeroFinder
	"Code Example 5.3"
	| zeroFinder result |
	zeroFinder := DhbNewtonZeroFinder function: [ :x | x errorFunction - 0.9].
	zeroFinder initialValue: 1.0.
	result := zeroFinder evaluate.
	self should: [ zeroFinder hasConverged].
	self should: [ (result -1.28155193867885) abs < zeroFinder precision]! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testNormalDistribution
	| dist |
	dist := DhbNormalDistribution new: 3.4 sigma: 1.7.
	self should: [ (dist average - 3.4) abs < 0.000000001].
	self should: [ (dist standardDeviation - 1.7) abs < 0.000000001].
	self should: [ ((dist value: 4.5) - 0.1903464693) abs < 0.000000001].
	self should: [ ((dist distributionValue: 4.5) - 0.7412031298) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'optimization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testOptimize
	"General optimizer to test genetic algorithm"
	| fBlock finder result |
	fBlock := [ :x | |r| r := x * x. r = 0 ifTrue: [ 1] ifFalse: [ r sqrt sin / r] ].
	finder := DhbMultiVariableGeneralOptimizer maximizingFunction: fBlock.
	finder desiredPrecision: 1.0e-6.
	finder origin: #( 0.5 1.0 0.5) asVector; range: #(2 2 2) asVector.
	result := finder evaluate.
	self should: [ finder precision < 1.0e-6].
	self should: [ (result at: 1) abs < 1.0e-6].
	self should: [ (result at: 2) abs < 1.0e-6].
	self should: [ (result at: 3) abs < 1.0e-6].! !

!DhbNumericalMethodsTestCase methodsFor: 'optimization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testOptimizeOneDimension
	"Code example 11.1"
	| distr finder maximum |
	distr := DhbGammaDistribution shape: 2 scale: 5.
	finder := DhbOneVariableFunctionOptimizer maximizingFunction: distr.
	finder desiredPrecision: 1.0e-6.
	maximum := finder evaluate.
	self should: [ ( maximum - 5) abs < 1.0e-6].
	self should: [ finder precision < 1.0e-6].! !

!DhbNumericalMethodsTestCase methodsFor: 'optimization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testOptimizePowell
	"Code example 11.3"
	| fBlock hillClimber educatedGuess result |
	fBlock := [ :x | (x * x) negated exp].
	educatedGuess := #( 0.5 1.0 0.5) asVector.
	hillClimber := DhbHillClimbingOptimizer maximizingFunction: fBlock.
	hillClimber initialValue: educatedGuess.
	hillClimber desiredPrecision: 1.0e-6.
	result := hillClimber evaluate.
	self should: [ hillClimber precision < 1.0e-6].
	self should: [ (result at: 1) abs < 1.0e-6].
	self should: [ (result at: 2) abs < 1.0e-6].
	self should: [ (result at: 3) abs < 1.0e-6].! !

!DhbNumericalMethodsTestCase methodsFor: 'optimization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testOptimizeSimplex
	"Code example 11.5"
	| fBlock simplex educatedGuess result |
	fBlock := [ :x | (x * x) negated exp].
	educatedGuess := #( 0.5 1.0 0.5) asVector.
	simplex := DhbSimplexOptimizer maximizingFunction: fBlock.
	simplex initialValue: educatedGuess.
	simplex desiredPrecision: 1.0e-6.
	result := simplex evaluate.
	self should: [ simplex precision < 1.0e-6].
	self should: [ (result at: 1) abs < 1.0e-6].
	self should: [ (result at: 2) abs < 1.0e-6].
	self should: [ (result at: 3) abs < 1.0e-6].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialAddition

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) + ( DhbPolynomial coefficients: #(-3 7 2 1)).
	self should: [ (polynomial at: 0) = -1].
	self should: [ (polynomial at: 1) = 4].
	self should: [ (polynomial at: 2) = 3].
	self should: [ (polynomial at: 3) = 1].
	self should: [ (polynomial at: 4) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialDerivative
	"Code example 2.3"
	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(-3 7 2 1)) derivative.
	self should: [ (polynomial at: 0) = 7].
	self should: [ (polynomial at: 1) = 4].
	self should: [ (polynomial at: 2) = 3].
	self should: [ (polynomial at: 3) = 0].
	self should: [ (polynomial at: 4) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialDivision

	| pol1 pol2 polynomial |
	pol1 := DhbPolynomial coefficients: #(2 -3 1).
	pol2 := DhbPolynomial coefficients: #(-6 23 -20 3 -1 1).
	polynomial := pol2 / pol1.
	self should: [ (polynomial at: 0) = -3].
	self should: [ (polynomial at: 1) = 7].
	self should: [ (polynomial at: 2) = 2].
	self should: [ (polynomial at: 3) = 1].
	self should: [ (polynomial at: 4) = 0].
	self should: [ (polynomial at: 5) = 0].
	self should: [ (polynomial at: 6) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialEvaluation
	"Code example 2.2"
	| polynomial |
	polynomial := DhbPolynomial coefficients: #(2 -3 1).
	self should: [0 = (polynomial value: 1)].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialIntegral
	"Code example 2.3"
	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(-3 7 2 1)) integral.
	self should: [ (polynomial at: 0) = 0].
	self should: [ (polynomial at: 1) = -3].
	self should: [ (polynomial at: 2) = (7/2)].
	self should: [ (polynomial at: 3) = (2/3)].
	self should: [ (polynomial at: 4) = (1/4)].
	self should: [ (polynomial at: 5) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialIntegralWithConstant
	"Code example 2.3"
	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(-3 7 2 1)) integral: 5.
	self should: [ (polynomial at: 0) = 5].
	self should: [ (polynomial at: 1) = -3].
	self should: [ (polynomial at: 2) = (7/2)].
	self should: [ (polynomial at: 3) = (2/3)].
	self should: [ (polynomial at: 4) = (1/4)].
	self should: [ (polynomial at: 5) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialMultiplication
	"Code example 2.3"
	| pol1 pol2 polynomial |
	pol1 := DhbPolynomial coefficients: #(2 -3 1).
	pol2 := DhbPolynomial coefficients: #(-3 7 2 1).
	polynomial := pol1 * pol2.
	self should: [ (polynomial at: 0) = -6].
	self should: [ (polynomial at: 1) = 23].
	self should: [ (polynomial at: 2) = -20].
	self should: [ (polynomial at: 3) = 3].
	self should: [ (polynomial at: 4) = -1].
	self should: [ (polynomial at: 5) = 1].
	self should: [ (polynomial at: 6) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialNumberAddition

	| polynomial |
	polynomial := 2 + ( DhbPolynomial coefficients: #(2 -3 1)).
	self should: [ (polynomial at: 0) = 4].
	self should: [ (polynomial at: 1) = -3].
	self should: [ (polynomial at: 2) = 1].
	self should: [ (polynomial at: 3) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialNumberAdditionInverse

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) + 2.
	self should: [ (polynomial at: 0) = 4].
	self should: [ (polynomial at: 1) = -3].
	self should: [ (polynomial at: 2) = 1].
	self should: [ (polynomial at: 3) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialNumberDivision

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) / 2.
	self should: [ (polynomial at: 0) = 1].
	self should: [ (polynomial at: 1) = (-3 / 2)].
	self should: [ (polynomial at: 2) = (1/2)].
	self should: [ (polynomial at: 3) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialNumberMultiplication

	| polynomial |
	polynomial := 2 * ( DhbPolynomial coefficients: #(2 -3 1)).
	self should: [ (polynomial at: 0) = 4].
	self should: [ (polynomial at: 1) = -6].
	self should: [ (polynomial at: 2) = 2].
	self should: [ (polynomial at: 3) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialNumberMultiplicationInverse

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) * 2.
	self should: [ (polynomial at: 0) = 4].
	self should: [ (polynomial at: 1) = -6].
	self should: [ (polynomial at: 2) = 2].
	self should: [ (polynomial at: 3) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialNumberSubtraction

	| polynomial |
	polynomial := 2 - ( DhbPolynomial coefficients: #(2 -3 1)).
	self should: [ (polynomial at: 0) = 0].
	self should: [ (polynomial at: 1) = 3].
	self should: [ (polynomial at: 2) = -1].
	self should: [ (polynomial at: 3) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialNumberSubtractionInverse

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) - 2.
	self should: [ (polynomial at: 0) = 0].
	self should: [ (polynomial at: 1) = -3].
	self should: [ (polynomial at: 2) = 1].
	self should: [ (polynomial at: 3) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialRoots
	"Code Example 5.5"
	| polynomial roots |
	polynomial := DhbPolynomial coefficients: #( -10 -13 -2 1).
	roots := polynomial roots asSortedCollection asArray.
	self should: [ roots size = 3].
	self should: [ ( (roots at: 1) + 2) abs < DhbFloatingPointMachine new defaultNumericalPrecision].
	self should: [ ( (roots at: 2) + 1) abs < DhbFloatingPointMachine new defaultNumericalPrecision].
	self should: [ ( (roots at: 3) - 5) abs < DhbFloatingPointMachine new defaultNumericalPrecision].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPolynomialSubtraction

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) - ( DhbPolynomial coefficients: #(-3 7 2 1)).
	self should: [ (polynomial at: 0) = 5].
	self should: [ (polynomial at: 1) = -10].
	self should: [ (polynomial at: 2) = -1].
	self should: [ (polynomial at: 3) = -1].
	self should: [ (polynomial at: 4) = 0].! !

!DhbNumericalMethodsTestCase methodsFor: 'precision' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testShowParameters
"included for coverage, test has no effect"
self should: [DhbFloatingPointMachine new showParameters isKindOf: DhbFloatingPointMachine ]! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testStatisticalMoments
	"comment"
	| accumulator |
	accumulator := DhbStatisticalMoments new.
	#( 36 13 27 16 33 24 4 20 15 23 37 23 31 15 47 22  6 15 41 22  14 14  31 42 3 42 22 8  37 41 )
		do: [ :x | accumulator accumulate: x].
	self should: [ (accumulator average - 24.1333333333) abs < 0.000000001].
	self should: [ (accumulator standardDeviation - 12.461619237603) abs < 0.000000001].
	self should: [(accumulator skewness - 0.116659884676) abs < 0.000000001].
	self should: [ (accumulator kurtosis + 1.004665562311) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testStatisticalMomentsFast

	| accumulator |
	accumulator := DhbFastStatisticalMoments new.
	#( 36 13 27 16 33 24 4 20 15 23 37 23 31 15 47 22  6 15 41 22  14 14  31 42 3 42 22 8  37 41 )
		do: [ :x | accumulator accumulate: x].
	self should: [ (accumulator average - 24.1333333333) abs < 0.000000001].
	self should: [ (accumulator standardDeviation - 12.461619237603) abs < 0.000000001].
	self should: [ (accumulator skewness - 0.116659884676) abs < 0.000000001].
	self should: [ (accumulator kurtosis + 1.004665562311) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testStatisticalMomentsFixed

	| accumulator |
	accumulator := DhbFixedStatisticalMoments new.
	#( 36 13 27 16 33 24 4 20 15 23 37 23 31 15 47 22  6 15 41 22  14 14  31 42 3 42 22 8  37 41 )
		do: [ :x | accumulator accumulate: x].
	self should: [ (accumulator average - 24.1333333333) abs < 0.000000001].
	self should: [ (accumulator standardDeviation - 12.461619237603) abs < 0.000000001].
	self should: [ (accumulator skewness - 0.116659884676) abs < 0.000000001].
	self should: [ (accumulator kurtosis + 1.004665562311) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testTTest

	| accC accMM confidenceLevel|
	accC := DhbStatisticalMoments new.
	#( 5.56 5.89 4.66 5.69 5.34 4.79 4.80 7.86 3.64 5.70 )
		do: [ :x | accC accumulate: x].
	accMM := DhbStatisticalMoments new.
	#( 7.48 6.75 3.77 5.71 7.25 4.73 6.23 5.60 5.94 4.58 )
		do: [ :x | accMM accumulate: x].
	confidenceLevel := accC tConfidenceLevel: accMM.
	self should: [ (accC average - 5.393) abs < 0.000000001].
	self should: [ (accC standardDeviation - 1.0990809292) abs < 0.000000001].
	self should: [ (accMM average - 5.804) abs < 0.000000001].
	self should: [ (accMM standardDeviation - 1.19415428) abs < 0.000000001].
	self should: [ (confidenceLevel - 56.6320739989) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'precision' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testUniqueInstance
| mach1 mach2 mach3 |
mach1 := DhbFloatingPointMachine new.
mach2 := DhbFloatingPointMachine new.
self should: [mach1 == mach2].
DhbFloatingPointMachine reset.
mach3 := DhbFloatingPointMachine new.
self shouldnt: [mach3 == mach2].
! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testVectorMatrixOperation
	"Code Example 8.1"
	| a u v |
	a := DhbMatrix rows: #( ( 1 0 1) (-1 -2 3)).
	u := #(1 2 3) asVector.
	v := a * u.
	self should: [ v size = 2].
	self should: [ (v at: 1) = 4].
	self should: [ (v at: 2) = 4].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testVectorOperation
	"Code Example 8.1"
	| u v w |
	u := #(1 2 3) asVector.
	v := #(3 4 5) asVector.
	w := 4 * u + (3 * v).
	self should: [ w size = 3].
	self should: [ (w at: 1) = 13].
	self should: [ (w at: 2) = 20].
	self should: [ (w at: 3) = 27].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testVectorOperationInverse

	| u v w |
	u := #(1 2 3) asVector.
	v := #(3 4 5) asVector.
	w := v * 4 - (3 * u).
	self should: [ w size = 3].
	self should: [ (w at: 1) = 9].
	self should: [ (w at: 2) = 10].
	self should: [ (w at: 3) = 11].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testVectorProduct
	"Code Example 8.1"
	| u v |
	u := #(1 2 3) asVector.
	v := #(3 4 5) asVector.
	self should: [ u * v = 26].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testVectorTransposeMatrixOperation
	"Code Example 8.1"
	| c v w |
	c := DhbMatrix rows: #( ( 6 8 10) (18 14 4)).
	v := #(4 4) asVector.
	w := c transpose * v.
	self should: [ w size = 3].
	self should: [ (w at: 1) = 96].
	self should: [ (w at: 2) = 88].
	self should: [ (w at: 3) = 56].! !

TestCase subclass: #GaussianGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random-Tests'!

!GaussianGeneratorTest methodsFor: 'as yet unclassified' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
checkDistributionOf: aGenerator withExpectedMeans: e andExpectedStandardDeviation: sd
	"fixme: this test ignores the standard deviation and occasionally fails"
	| data m |
	data := Set new.
	10000 timesRepeat: [ data add: aGenerator next ].
	m := data sum / data size.
	self assert: e - m <= 0.2! !

!GaussianGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testDistribution
	| mean standardDeviation g |
	mean := 147.
	standardDeviation := 17.
	g := (GaussianGenerator new)
		mean: mean;
		standardDeviation: standardDeviation.
	g generator: LCRandom new.
	self checkDistributionOf: g withExpectedMeans: mean andExpectedStandardDeviation: standardDeviation.
	"Replicated test in a different place"
	mean := 0.
	standardDeviation := 10 .
	g := (GaussianGenerator new)
		mean: mean;
		standardDeviation: standardDeviation;
		generator: LCRandom new.
	self checkDistributionOf: g withExpectedMeans: mean andExpectedStandardDeviation: standardDeviation.! !

TestCase subclass: #KDTreeTest
	instanceVariableNames: 'rand'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-KDTree'!

!KDTreeTest commentStamp: 'WernerKassens 5/3/2011 22:27' prior: 0!
KDTreeTest makes random checks of KDTree, comparing results with results of StupidNN. is not too fast.!

!KDTreeTest methodsFor: 'running' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
setUp
rand :=Random new.! !

!KDTreeTest methodsFor: 'tests' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
testKDTree1Dimension
|m tree stupid vec aResult bResult|
m :=(rand next:200) collect:   [:n| Array with: n]."1-dimensional data, just some numbers"
stupid :=StupidNN withAll: m.
tree :=KDTree  withAll: m.
1 to: 11do:[:v|vec :={1/ v asFloat }.
	aResult :=stupid nnSearch: vec  i:3.						"3 nearest numbers to vec"
	bResult :=tree nnSearch: vec  i:3.							"3 nearest numbers to vec using KDTree"
	self assert:  ( (aResult  with: bResult   collect: [:a :b|a hasEqualElements: b])  allSatisfy: [:e|e])  ]  .
m :=(1 to: 20) collect: [:index | Array with: (rand nextInt: 10)].	"only integers, obviously with multiples. test distances because nn are not necessarily the same"
stupid :=StupidNN withAll: m.
tree :=KDTree  withAll: m.
0 to: 11do:[:v|vec :={v}.
	aResult :=(vec first - (stupid nnSearch: vec  i:3))abs.		"distances between 3 nearest numbers and vec"
	bResult :=(vec first - (tree nnSearch: vec  i:3))abs.		"distances using KDTree"
	self assert:  ( (aResult  with: bResult   collect: [:a :b|a hasEqualElements: b])  allSatisfy: [:e|e]) ]  .
tree :=KDTree  withAll: #(#(1)).								"extreme case"
bResult :=tree nnSearch: #(2) i: 3.
self assert: (bResult size =1).
self assert: (bResult first = #(1) )   ! !

!KDTreeTest methodsFor: 'tests' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
testKDTree2Dimensions
|m tree stupid vec aResult bResult|
m :=(1 to: 2000) collect:   [:n| (rand next:2)].	"2-dimensional data"
stupid :=StupidNN withAll: m.
tree :=KDTree  withAll: m.
100 timesRepeat: [vec :=rand next:2.
	aResult :=stupid nnSearch: vec  i:3.
	bResult :=tree nnSearch: vec  i:3.
	self assert:  ( (aResult  with: bResult   collect: [:a :b|a hasEqualElements: b])  allSatisfy: [:e|e])  ] . 
m:= (1 to: 200) collect: [:index | Array with: (rand nextInt: 10) with: (rand nextInt: 10  )]. 	"integers"
stupid :=StupidNN withAll: m.
tree :=KDTree  withAll: m.
100 timesRepeat: [vec :=(rand next: 2)*11.
	aResult :=stupid nnSearch: vec  i:3.
	bResult :=tree nnSearch: vec  i:3.
	self assert:  ( (aResult  with: bResult   collect: [:a :b|a hasEqualElements: b])  allSatisfy: [:e|e]) ] . 
! !

!KDTreeTest methodsFor: 'tests' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
testKDTree4Dimensions
|m tree stupid vec aResult bResult|
m :=(1 to: 2000) collect:   [:n| (rand next:4)].		"4-dimensional data"
stupid :=StupidNN withAll: m.
tree :=KDTree  withAll: m.
50 timesRepeat: [vec :=(rand next: 4)- 0.5.
	aResult :=stupid nnSearch: vec  i:2.						"2 nearest neighbours"
	bResult :=tree nnSearch: vec  i:2.
	self assert:  ( (aResult  with: bResult   collect: [:a :b|a hasEqualElements: b])  allSatisfy: [:e|e])  ] . 
50 timesRepeat: [vec :=rand next: 4.
	aResult :=stupid nnSearch: vec  i:1.						"just the nearest neighbour"
	bResult :=tree nnSearch: vec  i:1.
	self assert: (aResult  hasEqualElements: bResult )    ] . 
! !

TestCase subclass: #LCRandomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random-Tests'!

!LCRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testNextBetweenZeroAndOne
	| g |
	g := LCRandom new.
	1000 timesRepeat:
		[ self should: [g next >= 0].
		self should: [g next <= 1]].! !

!LCRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPeekAlwaysReplyTheSameValue
	| g |
	g := LCRandom new.
	self assert: g peek = g peek! !

!LCRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPeekAnswersSameAsNext
	| g |
	g := LCRandom new.
	self assert: g peek = g next! !

TestCase subclass: #LehmerRandomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random-Tests'!

!LehmerRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testNextBetweenZeroAndOne
	| g |
	g := LehmerRandom new.
	1000 timesRepeat:
		[ self should: [g next > 0].
		self should: [g next < 1]].! !

!LehmerRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPeekAlwaysReplyTheSameValue
	| g |
	g := LehmerRandom new.
	self assert: g peek = g peek! !

!LehmerRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPeekAnswersSameAsNext
	| g |
	g := LehmerRandom new.
	self assert: g peek = g next! !

TestCase subclass: #MersenneTwisterRandomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random-Tests'!

!MersenneTwisterRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testNextBetweenZeroAndOne
	| g |
	g := MersenneTwisterRandom new.
	1000 timesRepeat:
		[ self should: [g next > 0].
		self should: [g next < 1]].! !

!MersenneTwisterRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPeekAlwaysReplyTheSameValue
	| g |
	g := MersenneTwisterRandom new.
	self assert: g peek = g peek! !

!MersenneTwisterRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testPeekAnswersSameAsNext
	| g |
	g := MersenneTwisterRandom new.
	self assert: g peek = g next! !

TestCase subclass: #TestQuaternion
	instanceVariableNames: 'q1234'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-QuaternionTests'!

!TestQuaternion commentStamp: '<historical>' prior: 0!
This class is for unit testing the Quaternion.!

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setUp
	q1234 := 1 i: 2 j: 3 k: 4! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testAbs
	self assert: (1 i: 1 j: 1 k: 1) abs = 2.
	self should: [(1.0e200 i: 1.0e200 j: 1.0e200 k: 1.0e200) abs = 2.0e200]
		description: 'An overflow should be avoided'.
	self 
		should: 
			[| big |
			big := (2.0 raisedTo: 53) - 1 timesTwoPower: 1023 - 52.
			(big i: big j: big k: big) abs = Float infinity]
		description: 'An overflow cannot be avoided'.
	self should: [(1.0e-200 i: 1.0e-200 j: 1.0e-200 k: 1.0e-200) abs = 2.0e-200]
		description: 'An underflow should be avoided'! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testCos
	| eps |
	eps := 1.0e-6.
	self assert: (q1234 cos - ((q1234 real cos * q1234 unreal cos) - (q1234 real sin * q1234 unreal sin))) abs < eps.
	self assert: ((1 + 2 i) cos - (1 + 2 j k) cos) abs < eps.! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testCosh
	| eps |
	eps := 1.0e-6.
	self assert: ((1 + 2 i) cosh - (1 + 2 j k) cosh) abs < eps! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testEquality
	self assert: q1234 = q1234 copy.
	self assert: (1 + 2 i) = (1 + 2 j k).
	self assert: (1 + 2 j k) = (1 + 2 i).
	self deny: (1 + 2 j) = (1 + 2 i).
	self deny: q1234 = 1.
	self assert: (1 k k) = -1.
	self assert: 1 = (-1 k k).
	self shouldnt: [q1234 = nil] raise: Error.
	self deny: q1234 = nil.! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testExp
	| eps |
	eps := 1.0e-6.
	self assert: (q1234 exp - (q1234 real exp * q1234 unreal exp)) abs < eps.
	self assert: ((1 + 2 i) exp - (1 + 2 j k) exp) abs < eps! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testFunctions
	| eps |
	eps := 1.0e-6.
	self assert: (q1234 ln exp - q1234) abs < eps! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testHash
	self assert: q1234 hash = q1234 copy hash.
	self assert: (1 + 2 i) hash = (1 + 2 j k) hash.
	self assert: (1 k k) hash = -1 hash.! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testLn
	| eps |
	eps := 1.0e-6.
	self assert: ((1 + 2 i) ln - (1 + 2 j k) ln) abs < eps! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testOne
	self assert: Quaternion one unreal isZero.
	self assert: Quaternion one real = 1.! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testProduct
	self  assert: (q1234) * 5 = (5 i: 10 j: 15 k: 20).
	self  assert: 5 * (q1234) = (5 i: 10 j: 15 k: 20).
	self assert: 1 i * 1 i = -1.
	self assert: 1 j * 1 j = -1.
	self assert: 1 k * 1 k = -1.
	self assert: 1 i * 1 j = 1 k.
	self assert: 1 j * 1 k = 1 i.
	self assert: 1 k * 1 i = 1 j.
	self assert: 1 j * 1 i = -1 k.
	self assert: 1 k * 1 j = -1 i.
	self assert: 1 i * 1 k = -1 j.
	self assert: 1 i i = -1.
	self assert: 1 j j = -1.
	self assert: 1 k k = -1.
	self assert: 1 i j = 1 k.
	self assert: 1 j k = 1 i.
	self assert: 1 k i = 1 j.
	self assert: 1 j i = -1 k.
	self assert: 1 k j = -1 i.
	self assert: 1 i k = -1 j.
	self  assert: (q1234) * 1 i = (q1234) i.
	self  assert: (q1234) * 1 j = (q1234) j.
	self  assert: (q1234) * 1 k = (q1234) k.

	self assert: (q1234 * 0) isZero.
	self assert: (0 / q1234) isZero.

	self assert: (q1234 * q1234 conjugated) = q1234 squaredNorm.
	self assert: (1 / q1234) = q1234 reciprocal.
	self assert: (q1234 / q1234) = 1.! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testRaisedTo
	| eps |
	eps := 1.0e-6.
	self assert: ((q1234 raisedTo: 3) - (q1234 ln * 3) exp) abs < eps.! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testSin
	| eps |
	eps := 1.0e-6.
	self assert: (q1234 sin - ((q1234 real sin * q1234 unreal cos) + (q1234 real cos * q1234 unreal sin))) abs < eps.
	self assert: ((1 + 2 i) sin - (1 + 2 j k) sin) abs < eps.! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testSinh
	| eps |
	eps := 1.0e-6.
	self assert: ((1 + 2 i) sinh - (1 + 2 j k) sinh) abs < eps! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testSum
	self  assert: 1 + 2 i + 3 j + 4 k = (q1234).
	self  assert: (1 + 2 i )+ (3 + 4 i) j = (q1234).
	self  assert: (q1234) + 5 = (6 i: 2 j: 3 k: 4).
	self  assert: 5 + (q1234) = (6 i: 2 j: 3 k: 4).
	self  assert: (q1234) - 9 = (-8 i: 2 j: 3 k: 4).
	self  assert: 9 - (q1234) = (8 i: -2 j: -3 k: -4).

	self assert: q1234 + q1234 conjugated  = (q1234 real * 2).
	self assert: q1234 + q1234 negated = (0).
	self assert: q1234 isZero not.
	self assert: (q1234 negated + q1234) isZero.! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testTan
	| eps |
	eps := 1.0e-6.
	self assert: ((1 + 2 i) tan - (1 + 2 j k) tan) abs < eps! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testTanh
	| eps |
	eps := 1.0e-6.
	self assert: ((1 + 2 i) tanh - (1 + 2 j k) tanh) abs < eps! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testUnreal
	self assert: q1234 unreal real = 0.
	self assert: q1234 unreal squared = (2 squared + 3 squared + 4 squared) negated.! !

!TestQuaternion methodsFor: 'running' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
testZero
	self assert: Quaternion zero isZero.
	self assert: Quaternion zero abs isZero.! !

Object subclass: #Complex
	instanceVariableNames: 'real imaginary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Complex'!

!Complex commentStamp: 'mk 10/31/2003 22:19' prior: 0!
I represent a complex number.

real			--	real part of the complex number
imaginary	--	imaginary part of the complex number

Complex number constructors:

	5 i
	6 + 7 i.
	5.6 - 8 i.
	Complex real: 10 imaginary: 5.
	Complex abs: 5 arg: (Float pi / 4)

Arithmetic operation with other complex or non-complex numbers work.

	(5 - 6 i) + (-5 + 8 i).			"Arithmetic between two complex numbers."
	5 * (5 - 6 i).				"Arithmetic between a non-complex and a complex number."
					
It is also possible to perform arithmetic operations between a complex number
and a array of (complex) numbers:

	2 * {1 + 2i.
	     3 + 4i.
	     5 + 6i}

	5 + 5i * {1 + 2i.
	          3.
	          5 + 6i}

It behaves analogously as it is with normal numbers and an array.

NOTE: Although Complex something similiar to the Smalltalk's Number class, it would
not be a good idea to make a Complex to be a subclass of a Number because:
- Number is subclass of Magnitude and Complex is certainly not a magnitude.
  Complex does not behave very well as a Magnitude. Operations such as
	<
	>
	<=
	>=
  do not have sense in case of complex numbers.
- Methods in the following Number methods' categories do not have sense for a Complex numbers
	trucation and round off
	testing
	intervals
	comparing
- However the following Number methods' categories do have sense for a Complex number
	arithmetic (with the exception of operation
		//
		\\
		quo:
		rem:	
	mathematical functions

Thus Complex is somewhat similar to a Number but it is not a subclass of it. Some operations
we would like to inherit (e.g. #abs, #negated, #reciprocal) but some of the Number operation
do not have sens to inherit or to overload. Classes are not always neat mechanism.

!!!!!! We had to COPY the implementation of the
		abs
		negated
		reciprocal
		log:
		isZero
		reciprocal
		...
	methods from the Number class to the Complex class. Awful solution. Now I begin to
	appreciate the Self.

Missing methods
	String | converting | asComplex
	Complex | mathematical functions | arcSin
	Complex | mathematical functions | arcCos
	Complex | mathematical functions | arcTan!

!Complex class methodsFor: 'instance creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
abs: aNumber1 arg: aNumber2
	^self
		real: aNumber1 * aNumber2 cos
		imaginary: aNumber1 * aNumber2 sin! !

!Complex class methodsFor: 'instance creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
	^ self real: 0 imaginary: 0! !

!Complex class methodsFor: 'instance creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
one
	^ self real: 1 imaginary: 0! !

!Complex class methodsFor: 'instance creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
real: aNumber1 imaginary: aNumber2
	^self basicNew	real: aNumber1 imaginary: aNumber2! !

!Complex class methodsFor: 'instance creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
zero
	^ self real: 0 imaginary: 0! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
* anObject
	"Answer the result of multiplying the receiver by aNumber."
	| a b c d newReal newImaginary |
	anObject isComplex
		ifTrue:
			[a := self real.
			b := self imaginary.
			c := anObject real.
			d := anObject imaginary.
			newReal := (a * c) - (b * d).
			newImaginary := (a * d) + (b * c).
			^ Complex real: newReal imaginary: newImaginary]
		ifFalse:
			[^ anObject adaptToComplex: self andSend: #*]! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
+ anObject
	"Answer the sum of the receiver and aNumber."
	| a b c d newReal newImaginary |
	anObject isComplex
		ifTrue:
			[a := self real.
			b := self imaginary.
			c := anObject real.
			d := anObject imaginary.
			newReal := a + c.
			newImaginary := b + d.
			^ Complex real: newReal imaginary: newImaginary]
		ifFalse:
			[^ anObject adaptToComplex: self andSend: #+]! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
- anObject
	"Answer the difference between the receiver and aNumber."
	| a b c d newReal newImaginary |
	anObject isComplex
		ifTrue:
			[a := self real.
			b := self imaginary.
			c := anObject real.
			d := anObject imaginary.
			newReal := a - c.
			newImaginary := b - d.
			^ Complex real: newReal imaginary: newImaginary]
		ifFalse:
			[^ anObject adaptToComplex: self andSend: #-]! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
/ anObject
	"Answer the result of dividing receiver by aNumber"
	| a b c d newReal newImaginary |
	anObject isComplex ifTrue:
		[a := self real.
		b := self imaginary.
		c := anObject real.
		d := anObject imaginary.
		newReal := ((a * c) + (b * d)) / ((c * c) + (d * d)).
		newImaginary := ((b * c) - (a * d)) / ((c * c) + (d * d)).
		^ Complex real: newReal imaginary: newImaginary].
	^ anObject adaptToComplex: self andSend: #/.! !

!Complex methodsFor: 'comparing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
= anObject
	anObject isNumber ifFalse: [^false].
	anObject isComplex
		ifTrue: [^ (real = anObject real) & (imaginary = anObject imaginary)]
		ifFalse: [^ anObject adaptToComplex: self andSend: #=]! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
abs
	"Answer the distance of the receiver from zero (0 + 0 i)."

	^ ((real * real) + (imaginary * imaginary)) sqrt! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
absSecure
	"Answer the distance of the receiver from zero (0 + 0 i).
	Try avoiding overflow and/or underflow"

	| scale |
	scale := real abs max: imaginary abs.
	^scale isZero 
		ifTrue: [scale]
		ifFalse: [(self class real: real / scale imaginary: imaginary / scale) squaredNorm sqrt * scale]! !

!Complex methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToCollection: rcvr andSend: selector
	"If I am involved in arithmetic with a Collection, return a Collection of
	the results of each element combined with me in that expression."

	^ rcvr collect: [:element | element perform: selector with: self]! !

!Complex methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToFloat: rcvr andSend: selector
	"If I am involved in arithmetic with a Float, convert it to a Complex number."
	^ rcvr asComplex perform: selector with: self! !

!Complex methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToFraction: rcvr andSend: selector
	"If I am involved in arithmetic with a Fraction, convert it to a Complex number."
	^ rcvr asComplex perform: selector with: self! !

!Complex methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToInteger: rcvr andSend: selector
	"If I am involved in arithmetic with an Integer, convert it to a Complex number."
	^ rcvr asComplex perform: selector with: self! !

!Complex methodsFor: '*Math-Quaternion-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToQuaternion: rcvr andSend: selector 
	"If I am involved in arithmetic with a Quaternion, convert me to a Quaternion."
	^ rcvr perform: selector with: self asQuaternion! !

!Complex methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/11/2012 18:06'!
addPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99
		
		added to Complex 11 May 2012 Daniel Uber
		 "
	^aPolynomial addNumber: self! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arCosh
	"Answer receiver's area hyperbolic cosine.
	That is the inverse function of cosh.
	Some possible implementations:
	^imaginary > 0 
		ifTrue: [(self + (self * self - 1) sqrt) ln]
		ifFalse: [(self + (self * self - 1) sqrt) ln negated]
	^self arcCos i
	This implementation provides an answer with a positive real part.
	It also avoids creating intermediate Complex."
	
	| x y tmp sh2x shx delta ch2x chx |
	imaginary = 0 ifTrue: [real abs > 1
			ifTrue: 
				[y := real < 0
					ifTrue: [Float pi]
					ifFalse: [0].
				x := real abs arCosh.
				^self class real: x imaginary: y]
			ifFalse: [^self class real: 0 imaginary: real arcCos]].
	tmp := self squaredNorm - 1 / 2.
	delta := tmp squared + imaginary squared.
	sh2x := tmp + delta sqrt.
	shx := sh2x sqrt.
	ch2x := 1 + sh2x.
	chx := ch2x sqrt.
	x := shx arSinh.
	y := imaginary copySignTo: (real / chx) arcCos.
	^self class real: x imaginary: y! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arSinh
	"Answer receiver's area hyperbolic sine.
	That is the inverse function of sinh."

	"Some possible implementation:

	^imaginary * real < 0 
		ifTrue: [(self + (self * self + 1) sqrt) ln]
		ifFalse: [(self - (self * self + 1) sqrt) ln]"

	^self i arcSin i negated! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arTanh
	"Answer receiver's area hyperbolic tangent.
	That is the inverse function of tanh."

	"Some other possible implementation:

	^((1 + self) / (1 - self)) ln / 2"

	^self i arcTan i negated! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arcCos
	"Answer the arc cosine of the receiver.
	This is the inverse function of cos."

	| x y tmp sh2y shy delta ch2y chy |
	imaginary = 0 ifTrue: [real abs > 1
			ifTrue: 
				[x := real < 0
					ifTrue: [Float pi]
					ifFalse: [0].
				y := real copySignTo: real abs arCosh.
				^self class real: x imaginary: y]
			ifFalse: [^self class real: real arcCos imaginary: 0]].
	tmp := self squaredNorm - 1 / 2.
	delta := tmp squared + imaginary squared.
	sh2y := tmp + delta sqrt.
	shy := sh2y sqrt.
	ch2y := 1 + sh2y.
	chy := ch2y sqrt.
	y := imaginary copySignTo: shy arSinh.
	x := (real / chy) arcCos.
	^self class real: x imaginary: y negated! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arcSin
	"Answer the arc sine of the receiver.
	This is the inverse function of sin."

	| x y tmp delta sh2y shy ch2y chy |
	imaginary = 0 
		ifTrue: 
			[real abs > 1 
				ifTrue: 
					[x := Float pi / 2 * real sign.
					y := (real copySignTo: real abs arCosh) negated.
					^self class real: x imaginary: y]
				ifFalse: [^self class real: real arcSin imaginary: 0]].
	tmp := (self squaredNorm - 1) / 2.
	delta := tmp squared + imaginary squared.
	sh2y := tmp + delta sqrt.
	shy := sh2y sqrt.
	ch2y := 1 + sh2y.
	chy := ch2y sqrt.
	y := imaginary copySignTo: shy arSinh.
	x := (real / chy) arcSin.
	^self class real: x imaginary: y! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arcTan
	"Answer the arc tangent of the receiver.
	This is the inverse function of tan."

	| r2 |
	r2 := self squaredNorm.
	^self class
		real: (real * 2 arcTan: 1 - r2) / 2
		imaginary: ((r2 + (imaginary * 2) + 1) / (r2 - (imaginary * 2) + 1)) ln / 4! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arcTan: denominator 
	"Answer the  four quadrants arc tangent of receiver over denominator."

	^denominator isZero 
		ifTrue: 
			[self isZero 
				ifTrue: 
					["shouldn't it be an error ? ^DomainError signal: '0 arcTan: 0'"
					^self class real: 0 imaginary: 0]
				ifFalse: 
					[self class
						real: Float pi / (real copySignTo: 2)
						imaginary: 0]]
		ifFalse: 
			[| res |
			res := (self / denominator) arcTan.
			denominator real < 0 ifTrue: [res := res + Float pi].
			res real > Float pi 
				ifTrue: [res := res - (Float pi * 2)].
			res]! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
arg
	"Answer the argument of the receiver."

	self isZero ifTrue: [self error: 'zero has no argument.'].
	^imaginary arcTan: real! !

!Complex methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asComplex
	^self! !

!Complex methodsFor: '*Math-Quaternion-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asQuaternion
	^ Quaternion
		qr: real
		qi: imaginary
		qj: 0
		qk: 0! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
conjugated
	"Return the complex conjugate of this complex number."

	^self class real: real imaginary: imaginary negated! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
cos
	"Answer receiver's cosine."

	^self i cosh! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
cosh
	"Answer receiver's hyperbolic cosine.
	Hyperbolic cosine is defined by same power serie expansion as for real numbers, that is in term of exponential:
	^ (self exp + self negated exp) / 2.
	This implementation avoids creating intermediate objects."
	
	^self class
		real: real cosh * imaginary cos
		imaginary: real sinh * imaginary sin! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
divideFastAndSecureBy: anObject
	"Answer the result of dividing receiver by aNumber"
	" Both operands are scaled to avoid arithmetic overflow. 
	  This algorithm works for a wide range of values, and it needs only three divisions.
	  Note: #reciprocal uses #/ for devision "
	 
	| r d newReal newImaginary |
	anObject isComplex ifTrue:
		[anObject real abs > anObject imaginary abs
		  ifTrue:
		    [r := anObject imaginary / anObject real.
			d := r*anObject imaginary + anObject real.
			newReal := r*imaginary + real/d.
			newImaginary := r negated * real + imaginary/d.
		    ]
		  ifFalse:
		    [r := anObject real / anObject imaginary.
			d := r*anObject real + anObject imaginary.
			newReal := r*real + imaginary/d.
			newImaginary := r*imaginary - real/d.
		    ].
		
		^ Complex real: newReal imaginary: newImaginary].
	^ anObject adaptToComplex: self andSend: #/.! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
divideSecureBy: anObject
	"Answer the result of dividing receiver by aNumber"
	" Both operands are scaled to avoid arithmetic overflow. This algorithm 
	  works for a wide range of values, but it requires six divisions.  
	  #divideFastAndSecureBy:  is also quite good, but it uses only 3 divisions.
	   Note: #reciprocal uses #/ for devision"
	 
	| s ars ais brs bis newReal newImaginary |
	anObject isComplex ifTrue:
		[s := anObject real abs + anObject imaginary abs.
		 ars := self real / s.
		 ais := self imaginary / s.
		 brs := anObject real / s.
		 bis := anObject imaginary / s.
		 s := brs squared + bis squared.
		
		newReal := ars*brs + (ais*bis) /s.
		newImaginary := ais*brs - (ars*bis)/s.
		^ Complex real: newReal imaginary: newImaginary].
	^ anObject adaptToComplex: self andSend: #/.! !

!Complex methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/11/2012 18:11'!
dividingPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: (1 / self)! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
exp
	"Answer the exponential of the receiver."

	^ real exp * (imaginary cos + imaginary sin i)! !

!Complex methodsFor: 'comparing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
hash
	"Hash is reimplemented because = is implemented."
	
	^ real hash bitXor: imaginary hash.! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
i
	"Answer the result of multiplying the receiver with pure imaginary.
		^self * 1 i
	This is an obvious extension of method i implemented in Number."

	^self class real: imaginary negated imaginary: real! !

!Complex methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
imaginary
	^ imaginary! !

!Complex methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isComplex
	^ true! !

!Complex methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isNumber
	^ true! !

!Complex methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isZero
	^ real isZero and: [imaginary isZero]! !

!Complex methodsFor: '*Math-Quaternion-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
j
	"same as self * 1 j"

	^Quaternion 
		qr: 0
		qi: 0
		qj: real
		qk: imaginary! !

!Complex methodsFor: '*Math-Quaternion-converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
k
	"same as self * 1 k"

	^Quaternion 
		qr: 0
		qi: 0
		qj: imaginary negated
		qk: real! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
ln
	"Answer the natural log of the receiver."

	^ self abs ln + self arg i! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
log: aNumber 
	"Answer the log base aNumber of the receiver."

	^self ln / aNumber ln! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
negated
	"Answer a Number that is the negation of the receiver."

	^self class real: real negated imaginary: imaginary negated! !

!Complex methodsFor: 'printing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
	real printOn: aStream.
	aStream nextPut: Character space.
	0 <= imaginary
		ifTrue: [aStream nextPut: $+]
		ifFalse: [aStream nextPut: $-].
	aStream nextPut: Character space.
	imaginary abs printOn: aStream.
	aStream nextPut: Character space.
	aStream nextPut: $i
! !

!Complex methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/11/2012 18:12'!
productWithVector: aVector
		"Answers a new vector product of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aVector collect: [ :each | each * self]! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
raisedTo: aNumber 
	"Answer the receiver raised to aNumber."

	aNumber isInteger ifTrue:
		["Do the special case of integer power"
		^ self raisedToInteger: aNumber].
	
	0 = aNumber ifTrue: [^ self class one].	"Special case of exponent=0"
	1 = aNumber ifTrue: [^ self].	"Special case of exponent=1"
	0 = self ifTrue: [				"Special case of self = 0"
		aNumber < 0
			ifTrue: [^ (ZeroDivide dividend: self) signal]
			ifFalse: [^ self]].
	^ (aNumber * self ln) exp		"Otherwise use logarithms"! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
raisedToInteger: operand 
	"Answer the receiver raised to the power operand, an Integer."

	"implementation note: this code is copied from Number.
	This suggest that both Number and Complex should have an
	ArithmeticValue common superclass like in Visualworks.
	Or maybe should it be a Traits (a property of fields ?)"

	| count result |
	operand isInteger ifFalse: [^ ArithmeticError signal: 'parameter is not an Integer'].
	operand = 0 ifTrue: [^ self class one].
	operand = 1 ifTrue: [^ self].
	operand < 0 ifTrue: [^ (self raisedToInteger: operand negated) reciprocal].
	count := 1 bitShift: (operand-1) highBit.
	result := self class one.
	[count > 0]
		whileTrue: 
			[result := result squared.
			(operand bitAnd: count)
				= 0 ifFalse: [result := result * self].
			count := count bitShift: -1].
	^ result! !

!Complex methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
real
	^ real! !

!Complex methodsFor: 'private' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
real: aNumber1 imaginary: aNumber2
	"Private - initialize the real and imaginary parts of a Complex"
	real := aNumber1.
	imaginary := aNumber2.! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reciprocal
	"Answer 1 divided by the receiver. Create an error notification if the 
	receiver is 0."

	self = 0
		ifTrue: [^ (ZeroDivide dividend: self) signal]
		ifFalse: [^1 / self]
		! !

!Complex methodsFor: 'testing' stamp: 'DanielUber 5/12/2012 21:57'!
sign
	^ real sign! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sin
	"Answer receiver's sine."

	^self i sinh i negated! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sinh
	"Answer receiver's hyperbolic sine.
	Hyperbolic sine is defined by same power serie expansion as for real numbers, that is in term of exponential:
	^ (self exp - self negated exp) / 2.
	This implementation avoids creating intermediate objects."
	
	^self class
		real: real sinh * imaginary cos
		imaginary: real cosh * imaginary sin! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sqrt
	"Return the square root of the receiver with a positive imaginary part."

	| u v |
	(imaginary = 0 and: [real >= 0])
		ifTrue:	[^self class real: real sqrt imaginary: 0].
	v := (self abs - real / 2) sqrt.
	u := imaginary / 2 / v.
	^self class real: u imaginary: v! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
squared
	"Answer the receiver multipled by itself."

	^self * self! !

!Complex methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
squaredNorm
	"Answer the square of receiver norm."

	^real * real + (imaginary * imaginary)! !

!Complex methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/11/2012 18:09'!
subtractToPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self negated! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
tan
	"Answer receivers tangent."

	^ self sin / self cos! !

!Complex methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
tanh
	"Answer receiver's hyperbolic tangent."

	"Some possible implementation are:

	^self sinh / self cosh

	| tr ti |
	tr := real tanh.
	ti := imaginary tan i.
	^(tr + ti) / (tr * ti + 1)"

	^self i tan i negated! !

!Complex methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/11/2012 18:00'!
timesPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 
		added to Complex class 12 May 2012, Daniel Uber
		"
	^aPolynomial timesNumber: self! !

Object subclass: #DhbAbstractDataServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbAbstractDataServer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
atEnd
		"Answers true if there is no more data element.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
close
		"Close the data stream (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
		"Answers the next element on the stream.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
open
		"Open the data stream (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset
		"Reset the position of the data stream to the beginning.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

DhbAbstractDataServer subclass: #DhbMemoryBasedDataServer
	instanceVariableNames: 'data position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMemoryBasedDataServer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
atEnd
		"Answers true if there is no more data element.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	^data size < position! !

!DhbMemoryBasedDataServer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
data: anOrderedCollection
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	data := anOrderedCollection.
	self reset.! !

!DhbMemoryBasedDataServer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
dimension
		"Answers the dimension of the vectors catered by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	^data first size! !

!DhbMemoryBasedDataServer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
		"Answers the next element on the stream.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	| answer |
	answer := data at: position.
	position := position + 1.
	^answer! !

!DhbMemoryBasedDataServer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
open
		"Open the data stream (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self reset! !

!DhbMemoryBasedDataServer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset
		"Reset the position of the data stream to the beginning.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	position := 1.! !

Object subclass: #DhbChromosomeManager
	instanceVariableNames: 'population populationSize rateOfMutation rateOfCrossover'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbChromosomeManager class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: anInteger mutation: aNumber1 crossover: aNumber2
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^self new populationSize: anInteger; rateOfMutation: aNumber1; rateOfCrossover: aNumber2; yourself! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
clone: aChromosome
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^aChromosome copy! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
crossover: aChromosome1 and: aChromosome2
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^self subclassResponsibility! !

!DhbChromosomeManager methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isFullyPopulated
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^population size >= populationSize! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
mutate: aChromosome
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^self subclassResponsibility! !

!DhbChromosomeManager methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
population
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^population! !

!DhbChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
populationSize: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	populationSize := anInteger.! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
process: aChromosome1 and: aChromosome2 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	| roll |
	roll := Number random.
	roll < rateOfCrossover 
		ifTrue: [population addAll: (self crossover: aChromosome1 and: aChromosome2)]
		ifFalse: 
			[roll < (rateOfCrossover + rateOfMutation) 
				ifTrue: 
					[population
						add: (self mutate: aChromosome1);
						add: (self mutate: aChromosome2)]
				ifFalse: 
					[population
						add: (self clone: aChromosome1);
						add: (self clone: aChromosome2)]]! !

!DhbChromosomeManager methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
randomChromosome
	self subclassResponsibility! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
randomizePopulation
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	self reset.
	[self isFullyPopulated] 
		whileFalse: [population add: self randomChromosome]! !

!DhbChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
rateOfCrossover: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	(aNumber between: 0 and: 1) 
		ifFalse: [self error: 'Illegal rate of cross-over'].
	rateOfCrossover := aNumber! !

!DhbChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
rateOfMutation: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	(aNumber between: 0 and: 1) 
		ifFalse: [self error: 'Illegal rate of mutation'].
	rateOfMutation := aNumber! !

!DhbChromosomeManager methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	population := OrderedCollection new: populationSize.! !

DhbChromosomeManager subclass: #DhbVectorChromosomeManager
	instanceVariableNames: 'origin range'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbVectorChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
crossover: aChromosome1 and: aChromosome2
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| index new1 new2|
	index := ( aChromosome1 size - 1) random + 2.
	new1 := self clone: aChromosome1.
	new1 replaceFrom: index to: new1 size with: aChromosome2 startingAt: index.
	new2 := self clone: aChromosome2.
	new2 replaceFrom: index to: new2 size with: aChromosome1 startingAt: index.
	^Array with: new1 with: new2! !

!DhbVectorChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
mutate: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| index |
	index := aVector size random + 1.
	^( aVector copy)
			at: index put: ( self randomComponent: index);
			yourself! !

!DhbVectorChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
origin: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	origin := aVector.! !

!DhbVectorChromosomeManager methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
randomChromosome
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^( ( 1 to: origin size) collect: [ :n | self randomComponent: n]) asVector! !

!DhbVectorChromosomeManager methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
randomComponent: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^( range at: anInteger) asFloat random + ( origin at: anInteger)! !

!DhbVectorChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
range: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	range := aVector.! !

Object subclass: #DhbCluster
	instanceVariableNames: 'accumulator previousSampleSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCluster class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^super new initialize! !

!DhbCluster methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator accumulate: aVector.! !

!DhbCluster methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
centerOn: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	self subclassResponsibility! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changes
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	^(self sampleSize - previousSampleSize) abs! !

!DhbCluster methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
collectAccumulatorResults
	self subclassResponsibility! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self subclassResponsibility! !

!DhbCluster methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	previousSampleSize := 0.
	^self! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isInsignificantIn: aClusterFinder
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/17/00 "
	^self sampleSize <= aClusterFinder minimumClusterSize! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isUndefined
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self subclassResponsibility! !

!DhbCluster methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	previousSampleSize := self sampleSize.
	self collectAccumulatorResults.
	accumulator reset! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sampleSize
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator count! !

DhbCluster subclass: #DhbCovarianceCluster
	instanceVariableNames: 'center'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCovarianceCluster methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
centerOn: aVector 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved. 
	Initial code: 2/16/00"

	accumulator := aVector isNil
				ifTrue: [nil]
				ifFalse: [DhbMahalanobisCenter onVector: aVector]! !

!DhbCovarianceCluster methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
collectAccumulatorResults
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator computeParameters.! !

!DhbCovarianceCluster methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator distanceTo: aVector! !

!DhbCovarianceCluster methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isUndefined
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator isNil! !

!DhbCovarianceCluster methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator printOn: aStream.! !

DhbCluster subclass: #DhbEuclideanCluster
	instanceVariableNames: 'center'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbEuclideanCluster methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
centerOn: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
		"minimal change so that it works in pharo 1.2.1" 
	center := aVector.
	accumulator := DhbVectorAccumulator new: (aVector ifNil:[0]ifNotNil:[aVector size]).! !

!DhbEuclideanCluster methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
collectAccumulatorResults
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	center := accumulator average copy.! !

!DhbEuclideanCluster methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^( aVector - center) norm! !

!DhbEuclideanCluster methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isUndefined
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^center isNil! !

!DhbEuclideanCluster methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator count printOn: aStream.
	aStream nextPutAll: ': '.
	center printOn: aStream.! !

Object subclass: #DhbCongruentialRandomNumberGenerator
	instanceVariableNames: 'constant modulus multiplicator seed'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
constant: aNumber1 multiplicator: aNumber2 modulus: aNumber3 
	"Create a new instance of the receiver with given constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new 
		initialize: aNumber1
		multiplicator: aNumber2
		modulus: aNumber3! !

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Create a new instance of the receiver with D. Knuth's constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new initialize.
					 UniqueInstance setSeed: 1.
				   ].
	^UniqueInstance! !

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
seed: aNumber
		"Create a new instance of the receiver with given seed
		  using D. Knuth's constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( super new) initialize; setSeed: aNumber; yourself! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
floatValue
		"Answer the next pseudo-random value between 0 and 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self value asFloat / modulus! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
		"Private - Initializes the constants of the receiver with D. Knuth's constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	self initialize: 2718281829.0 multiplicator: 3141592653.0 modulus: 4294967296.0.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 multiplicator: aNumber2 modulus: aNumber3
		"Private - Initializes the constants needed by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	constant := aNumber1.
	modulus := aNumber2.
	multiplicator := aNumber3.
	self setSeed: 1.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
integerValue: anInteger
		"Answer a random integer between 0 and the anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( self value  \\ ( anInteger * 1000)) // 1000! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setSeed: aNumber
		"Set the seed of the receiver to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	seed := aNumber.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value
		"Answer the next pseudo-random value.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	seed := ( seed * multiplicator + constant) \\ modulus.
	^seed! !

Object subclass: #DhbDecimalFloatingNumber
	instanceVariableNames: 'mantissa exponent'
	classVariableNames: 'Digits'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbDecimalFloatingNumber class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultDigits
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^15! !

!DhbDecimalFloatingNumber class methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultDigits: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	Digits := anInteger.! !

!DhbDecimalFloatingNumber class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
digits
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	Digits isNil
		ifTrue: [ Digits := self defaultDigits].
	^Digits! !

!DhbDecimalFloatingNumber class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self new normalize: aNumber! !

!DhbDecimalFloatingNumber class methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
resetDigits
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	Digits := nil.! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
* aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value * aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
+ aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value + aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
- aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value - aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
/ aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value / aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
normalize: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	exponent := (self class digits - (aNumber log: 10)) floor.
	mantissa := ( aNumber * ( 10 raisedToInteger: exponent)) truncated.
	^self! !

!DhbDecimalFloatingNumber methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	mantissa printOn: aStream.
	aStream nextPutAll: 'xE'.
	exponent negated printOn: aStream.! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sqrt
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new:  self value sqrt! !

!DhbDecimalFloatingNumber methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^mantissa / ( 10 raisedToInteger: exponent)! !

Object subclass: #DhbErfApproximation
	instanceVariableNames: 'constant series norm'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbErfApproximation class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Answer a unique instance. Create it if it does not exist.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new.
					 UniqueInstance initialize.
					].
	^UniqueInstance! !

!DhbErfApproximation methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
		"Private - Initialize constants needed to evaluate the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "
	constant := 0.2316419.
	norm := 1 / ( Float pi * 2) sqrt.
	series := DhbPolynomial coefficients: #( 0.31938153 -0.356563782 1.781477937 -1.821255978 1.330274429).! !

!DhbErfApproximation methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
normal: aNumber
		"Computes the value of the Normal distribution for aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "

	^[ ( aNumber squared * -0.5) exp * norm]
			on: Error do: [ :signal | signal return: 0]! !

!DhbErfApproximation methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answer erf( aNumber) using an approximation from Abramovitz and Stegun, Handbook of Mathematical Functions.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "
	| t |
	aNumber = 0
		ifTrue: [ ^0.5].
	aNumber > 0
		ifTrue: [ ^1- ( self value: aNumber negated)].
	aNumber < -20
		ifTrue: [ ^0].
	t := 1 / (1 - (constant * aNumber)).
	^( series value: t) * t * (self normal: aNumber)! !

Object subclass: #DhbFloatingPointMachine
	instanceVariableNames: 'defaultNumericalPrecision radix machinePrecision negativeMachinePrecision smallestNumber largestNumber smallNumber largestExponentArgument'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFloatingPointMachine commentStamp: '<historical>' prior: 0!
A DhbFloatingPointMachine represents the numerical precision of this system.

Instance Variables

	defaultNumericalPrecision:		The relative 
numerical precision that can be expected for a general numerical computation. One should consider to numbers a and b equal if the relative difference between them is less than the default machine precision.

	largestExponentArgument:		natural logarithm of largest number

	largestNumber:		The largest positive number that can be represented in the machine

	machinePrecision:		r^{-(n+1)}, with the largest n such that (1 + r^-n) - 1 !!= 0

	negativeMachinePrecision:		r^{-(n+1)}, with the largest n such that (1 - r^-n) - 1 !!= 0

	radix:		The radix of the floating point representation. This is often 2.

	smallNumber:		A number that can be added to some value without noticeably changing the result of the computation

	smallestNumber:		The smallest positive number different from 0.


largestExponentArgument
	- xxxxx

This class is detailed in Object Oriented Implementation of Numerical Methods, Section 1.4.1 and 1.4.2!

!DhbFloatingPointMachine class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	UniqueInstance = nil
		ifTrue: [ UniqueInstance := super new].
	^UniqueInstance! !

!DhbFloatingPointMachine class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	UniqueInstance := nil.! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeLargestNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/6/99 "
	| one floatingRadix fullMantissaNumber |

	one := 1.0.
	floatingRadix := self radix asFloat.
	fullMantissaNumber := one - ( floatingRadix * self negativeMachinePrecision).
	largestNumber := fullMantissaNumber.
	[ [ fullMantissaNumber := fullMantissaNumber * floatingRadix.
	    fullMantissaNumber isInfinite ifTrue: [^nil].
	    largestNumber := fullMantissaNumber.
		true] whileTrue: [ ].
		] on: Error do: [ :signal | signal return: nil].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeMachinePrecision
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	| one zero inverseRadix tmp |
	one := 1.0.
	zero := 0.0.
	inverseRadix := one / self radix asFloat.
	machinePrecision := one.
	[ tmp := one + machinePrecision.
	  tmp - one = zero]
		whileFalse:[  machinePrecision := machinePrecision * inverseRadix].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeNegativeMachinePrecision
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	| one zero floatingRadix inverseRadix tmp |
	one := 1.0.
	zero := 0.0.
	floatingRadix := self radix asFloat.
	inverseRadix := one / floatingRadix.
	negativeMachinePrecision := one.
	[ tmp := one - negativeMachinePrecision.
	  tmp - one = zero]
		whileFalse:[ negativeMachinePrecision := negativeMachinePrecision * inverseRadix].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeRadix
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	| one zero a b tmp1 tmp2|
	one := 1.0.
	zero := 0.0.
	a := one.
	[ a := a + a.
	  tmp1 := a + one.
	  tmp2 := tmp1 - a.
	  tmp2 - one = zero] whileTrue:[].
	b := one.
	[ b := b + b.
	  tmp1 := a + b.
	  radix := ( tmp1 - a) truncated.
	  radix = 0 ] whileTrue: [].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeSmallestNumber
	"Private - 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 11/6/99"

	| one floatingRadix inverseRadix fullMantissaNumber |
	one := 1 asFloat.
	floatingRadix := self radix asFloat.
	inverseRadix := one / floatingRadix.
	fullMantissaNumber := one - (floatingRadix * self negativeMachinePrecision).
	smallestNumber := fullMantissaNumber.
	[[fullMantissaNumber := fullMantissaNumber * inverseRadix.
	fullMantissaNumber = 0.0 ifTrue: [Error raise].
	smallestNumber := fullMantissaNumber.
	true]
		whileTrue: []]
		on: Error do: [:signal | signal return: nil]! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultNumericalPrecision
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	defaultNumericalPrecision isNil
		ifTrue: [ defaultNumericalPrecision := self machinePrecision sqrt].
	^defaultNumericalPrecision! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
largestExponentArgument
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/18/00 "
	largestExponentArgument isNil
		ifTrue: [ largestExponentArgument := self largestNumber ln].
	^largestExponentArgument! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
largestNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/5/99 "
	largestNumber isNil
		ifTrue: [ self computeLargestNumber].
	^largestNumber! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
machinePrecision
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	machinePrecision isNil
		ifTrue: [ self computeMachinePrecision].
	^machinePrecision! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
negativeMachinePrecision
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	negativeMachinePrecision isNil
		ifTrue: [ self computeNegativeMachinePrecision].
	^negativeMachinePrecision! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
radix
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	radix isNil
		ifTrue: [ self computeRadix].
	^radix! !

!DhbFloatingPointMachine methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
showParameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/6/99 "
		"added 'Transcript flush' to make sure that the transcript window gets updated"
	Transcript cr; cr;
			nextPutAll: 'Floating-point machine parameters'; cr;
			nextPutAll: '---------------------------------';cr;
			nextPutAll: 'Radix: '.
	self radix printOn: Transcript.
	Transcript cr; nextPutAll: 'Machine precision: '.
	self machinePrecision printOn: Transcript.
	Transcript cr; nextPutAll: 'Negative machine precision: '.
	self negativeMachinePrecision printOn: Transcript.
	Transcript cr; nextPutAll: 'Smallest number: '.
	self smallestNumber printOn: Transcript.
	Transcript cr; nextPutAll: 'Largest number: '.
	self largestNumber printOn: Transcript.
	Transcript flush.! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
smallNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/5/99 "
	smallNumber isNil
		ifTrue: [ smallNumber := self smallestNumber sqrt].
	^smallNumber! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
smallestNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/5/99 "
	smallestNumber isNil
		ifTrue: [ self computeSmallestNumber].
	^smallestNumber! !

Object subclass: #DhbHistogram
	instanceVariableNames: 'minimum binWidth overflow underflow moments contents freeExtent cacheSize desiredNumberOfBins'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultCacheSize
		"Private - Answer the default cache size.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^100! !

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultNumberOfBins
		"Private - Defines the default number of bins for instances of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^50! !

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
integerScales
	"Private - Scales for strict integers"
	^#( 2 4 5 8 10)! !

!DhbHistogram class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Create a standard new instance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize! !

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
scales
	"Private - Scales for any number"
	^#( 1.25 2 2.5 4 5 7.5 8 10)! !

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
semiIntegerScales
	"Private - Scales for large integers"
	^#( 2 2.5 4 5 7.5 8 10)! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aNumber
		"Accumulate aNumber into the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| bin |
	"(contents includes: nil) ifTrue: [self halt]."
	self isCached
		ifTrue: [ ^self accumulateInCache: aNumber].

	bin := self binIndex: aNumber.
	( bin between: 1 and: contents size)
		ifTrue: [ contents at: bin put: ( contents at: bin) + 1.
					 moments accumulate: aNumber.
				   ]
		ifFalse:[ self processOverflows: bin for: aNumber].
! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulateInCache: aNumber
		"Private - Accumulate aNumber inside a cache
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	contents add: aNumber.
	contents size > cacheSize
		ifTrue: [ self flushCache].! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adjustDimensionUpTo: aNumber
		"Private - Compute an adequate bin width and adjust the minimum and number of bins accordingly.
		 aNumber is the maximum value to accumulate. The minimum value has already been assigned.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	| maximum |
	binWidth := self roundToScale: ( aNumber - minimum) / desiredNumberOfBins.
	minimum := ( minimum / binWidth) floor * binWidth.
	maximum := ( aNumber / binWidth) ceiling * binWidth.
	contents := Array new: ( ( maximum - minimum) / binWidth) ceiling.
	contents atAllPut: 0.! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^moments average! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
binIndex: aNumber
		"Answers the index of the bin corresponding to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^( ( aNumber - minimum) / binWidth) floor + 1! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
binWidth
		"Answer the bin width for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifTrue: [ self flushCache].
	^binWidth! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
chi2Against: aScaledDistribution
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/30/00 "
	| chi2 |
	chi2 := 0.
	self pointsAndErrorsDo:
		[ :each | chi2 := ( each chi2Contribution: aScaledDistribution) + chi2].
	^chi2! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
chi2ConfidenceLevelAgainst: aScaledDistribution
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/30/00 "
	^( DhbChiSquareDistribution degreeOfFreedom: ( contents size - aScaledDistribution parameters size))
			confidenceLevel: ( self chi2Against: aScaledDistribution)! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
collectIntegralPoints: aBlock
		"Collects the points needed to display the receiver as an integral.
		 Needed to use polymorphic behavior when plotting the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	| answer bin lastContents integral norm x |
	self isCached
		ifTrue: [ self flushCache].
	answer := OrderedCollection new: ( contents size * 2 + 1).
	bin := self minimum.
	answer add: ( aBlock value: bin @ 0).
	integral := self underflow.
	norm := self totalCount.
	contents do:
		[ :each |
		  integral := integral + each.
		  x := integral / norm.
		  answer add: ( aBlock value: bin @ x).
		  bin := bin + binWidth.
		  answer add: ( aBlock value: bin @ x).
		].
	answer add: ( aBlock value: bin @ 0).
	^answer asArray! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
collectPoints: aBlock
		"Collects the points needed to display the receiver.
		 Needed to use polymorphic behavior when plotting the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	| answer bin lastContents |
	self isCached
		ifTrue: [ self flushCache].
	answer := OrderedCollection new: ( contents size * 2 + 1).
	bin := self minimum.
	answer add: ( aBlock value: bin @ 0).
	contents do:
		[ :each |
		  answer add: ( aBlock value: bin @ each).
		  bin := bin + binWidth.
		  answer add: ( aBlock value: bin @ each).
		].
	answer add: ( aBlock value: bin @ 0).
	^answer asArray! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
count
		"Answer the count of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^moments count! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
countAt: aNumber
		"Answer the count in the bin corresponding to aNumber or 0 if outside the limits.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	| n |
	n := self binIndex: aNumber.
	^( n between: 1 and: contents size)
			ifTrue: [ contents at: n]
			ifFalse:[ 0]! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
countOverflows: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	anInteger > 0
		ifTrue: [ overflow := overflow + 1]
		ifFalse:[ underflow := underflow + 1].! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
countsBetween: aNumber1 and: aNumber2
		"Computes the events located between aNumber1 and aNumber2.
		 NOTE: This method assumes the two numbers are within the limits
			   of the receiver and that the receiver is not cached.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	| n1 n2 answer |
	n1 := self binIndex: aNumber1.
	n2 := self binIndex: aNumber2.
	answer := ( contents at: n1) * ( ( binWidth * n1 + minimum) - aNumber1) / binWidth.
	n2 > contents size
		ifTrue: [ n2 := contents size + 1]
		ifFalse:[ answer := answer + ( ( contents at: n2) * ( aNumber2 - ( binWidth * ( n2 - 1) + self maximum)) / binWidth)].
	( n1 + 1) to: ( n2 - 1) do: [ :n | answer := answer + ( contents at: n)].
	^answer! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
countsUpTo: aNumber
		"Computes the events located up to aNumber.
		 NOTE: This method assumes aNumber is within the limits
					 of the receiver and that the receiver is not cached.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	| n answer |
	n := self binIndex: aNumber.
	n > contents size
		ifTrue: [ ^self count].
	answer := ( contents at: n) * ( aNumber - ( binWidth * ( n - 1) + self minimum)) / binWidth.
	1 to: ( n - 1) do: [ :m | answer := answer + ( contents at: m)].
	^answer + underflow! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
errorOnAverage
		"Answer the error on the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^moments errorOnAverage! !

!DhbHistogram methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fConfidenceLevel: aStatisticalMomentsOrHistogram
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	^moments fConfidenceLevel: aStatisticalMomentsOrHistogram! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
flushCache
	"Private - 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 15/2/99"

	| maximum values |
	minimum isNil ifTrue: [minimum := contents isEmpty
					ifTrue: [0]
					ifFalse: [contents first]].
	maximum := minimum.
	contents do: [:each | each < minimum
			ifTrue: [minimum := each]
			ifFalse: [each > maximum ifTrue: [maximum := each]]].
	maximum = minimum ifTrue: [maximum := minimum + desiredNumberOfBins].
	values := contents.
	self adjustDimensionUpTo: maximum.
	values do: [:each | self accumulate: each]! !

!DhbHistogram methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
freeExtent: aBoolean
		"Defines the range of the receiver to be freely adjustable.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	( underflow = 0 and: [ overflow = 0])
		ifFalse: [ self error: 'Histogram extent cannot be redefined'].
	freeExtent := aBoolean.! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
growContents: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	anInteger > 0
		ifTrue: [ self growPositiveContents: anInteger]
		ifFalse:[ self growNegativeContents: anInteger].! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
growNegativeContents: anInteger 
	"Private - 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 15/2/99"

	| n newSize newContents |
	n := 1 - anInteger.
	newSize := contents size + n.
	newContents := Array new: newSize.
	newContents at: 1 put: 1.
	2 to: n do: [:i | newContents at: i put: 0].
	newContents
		replaceFrom: n + 1
		to: newSize
		with: contents
		startingAt: 1.
	contents := newContents.
	minimum := anInteger - 1 * binWidth + minimum! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
growPositiveContents: anInteger 
	"Private - 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 15/2/99"

	| n newContents |
	n := contents size.
	newContents := Array new: anInteger.
	newContents
		replaceFrom: 1
		to: n
		with: contents
		startingAt: 1.
	n + 1 to: anInteger - 1 do: [:i | newContents at: i put: 0].
	newContents at: anInteger put: 1.
	contents := newContents! !

!DhbHistogram methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
		"Private - initializes the receiver with standard settings.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	freeExtent := false.
	cacheSize := self class defaultCacheSize.
	desiredNumberOfBins := self class defaultNumberOfBins.
	contents := OrderedCollection new: cacheSize.
	moments := DhbFixedStatisticalMoments new.
	overflow := 0.
	underflow := 0.
	^self! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inverseDistributionValue: aNumber
		"Private - Compute the value which corresponds to a integrated count of aNumber.
		 NOTE: aNumber is assumed to be between 0 and 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	| count x integral |
	count := self count * aNumber.
	x := self minimum.
	integral := 0.
	contents do:
		[ :each | | delta |
		  delta := count - integral.
		  each > delta
			ifTrue: [ ^self binWidth * delta / each + x].
		  integral := integral + each.
		  x := self binWidth + x.
		].
	^self maximum! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isCached
		"Private - Answer true if the content of the receiver is cached.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^binWidth isNil! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isEmpty
		"Always false. 
		 Needed to use polymorphic behavior when plotting the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^false! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^moments kurtosis! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
lowBinLimitAt: anInteger
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^( anInteger - 1) * binWidth + minimum! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
maximum
		"Answer the minimum for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifTrue: [ self flushCache].
	^contents size * binWidth + minimum! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
maximumCount
		"Answer the maximum count of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	self isCached
		ifTrue: [ self flushCache].
	^contents inject: ( contents isEmpty ifTrue: [ 1] ifFalse:[ contents at: 1])
					into: [ :max :each | max max: each]! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
minimum
		"Answer the minimum for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifTrue: [ self flushCache].
	^minimum! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
overflow
		"Answer the overflow of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^overflow! !

!DhbHistogram methodsFor: 'iterators' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
pointsAndErrorsDo: aBlock
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	| x |
	x := self minimum - ( self binWidth / 2).
	contents do:
		[ :each |
		  x := x + self binWidth.
		  aBlock value: ( DhbWeightedPoint point: x count: each).
		].! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
processOverflows: anInteger for: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	freeExtent
		ifTrue: [ self growContents: anInteger.
					 moments accumulate: aNumber
				   ]
		ifFalse:[ self countOverflows: anInteger].! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
roundToScale: aNumber
	"Private - Adjust aNumber of the lowest upper scale"
	| orderOfMagnitude norm scales rValue |
	orderOfMagnitude := ( aNumber log: 10) floor.
	scales := self class scales.
	aNumber isInteger
		ifTrue: [ orderOfMagnitude < 1
						ifTrue: [ orderOfMagnitude := 1].
					  orderOfMagnitude = 1
						ifTrue: [ scales := self class integerScales].
					  orderOfMagnitude = 2
						ifTrue: [ scales := self class semiIntegerScales].
				    ].
	norm := 10 raisedToInteger: orderOfMagnitude.
	rValue := aNumber / norm.
	^( scales detect: [ :each | rValue <= each]) * norm! !

!DhbHistogram methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setDesiredNumberOfBins: anInteger
		"Defines the desired number of bins. It may be adjusted to a few units later on.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	anInteger > 0
		ifFalse:[ self error: 'Desired number of bins must be positive'].
	desiredNumberOfBins := anInteger.! !

!DhbHistogram methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setRangeFrom: aNumber1 to: aNumber2 bins: anInteger
		"Defines the range of the receiver by specifying the minimum, maximum and number of bins.
		 Values are adjusted to correspond to a reasonable value for the bin width and the limits.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifFalse: [ self error: 'Histogram limits cannot be redefined'].
	minimum := aNumber1.
	self setDesiredNumberOfBins: anInteger;
		   adjustDimensionUpTo: aNumber2.! !

!DhbHistogram methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setWidth: aNumber1 from: aNumber2 bins: anInteger
		"Defines the range of the receiver by specifying the minimum, bin width and number of bins.
		 Values are adjusted to correspond to a reasonable value for the bin width and the limits.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifFalse: [ self error: 'Histogram limits cannot be redefined'].
	minimum := aNumber2.
	self setDesiredNumberOfBins: anInteger;
		   adjustDimensionUpTo: ( aNumber1 * anInteger + aNumber2).! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^moments skewness! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
standardDeviation
		"Answer the standardDeviation of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^moments standardDeviation! !

!DhbHistogram methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
tConfidenceLevel: aStatisticalMomentsOrHistogram
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	^moments tConfidenceLevel: aStatisticalMomentsOrHistogram! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
totalCount
		"Answer the count of the recevier, inclusing underflow and overflow
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^moments count + underflow + overflow! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
underflow
		"Answer the underflow of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^underflow! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
unnormalizedVariance
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	^moments unnormalizedVariance! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^moments variance! !

Object subclass: #DhbIncompleteBetaFunction
	instanceVariableNames: 'alpha1 alpha2 fraction inverseFraction logNorm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteBetaFunction class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
shape: aNumber1 shape: aNumber2 
	"Create an instance of the receiver with given shape parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 shape: aNumber2! !

!DhbIncompleteBetaFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateFraction: aNumber 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	fraction isNil 
		ifTrue: 
			[fraction := DhbIncompleteBetaFractionTermServer new.
			fraction setParameter: alpha1 second: alpha2].
	fraction setArgument: aNumber.
	^(DhbContinuedFraction server: fraction)
		desiredPrecision: DhbFloatingPointMachine new defaultNumericalPrecision;
		evaluate! !

!DhbIncompleteBetaFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateInverseFraction: aNumber 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	inverseFraction isNil 
		ifTrue: 
			[inverseFraction := DhbIncompleteBetaFractionTermServer new.
			inverseFraction setParameter: alpha2 second: alpha1].
	inverseFraction setArgument: (1 - aNumber).
	^(DhbContinuedFraction server: inverseFraction)
		desiredPrecision: DhbFloatingPointMachine new defaultNumericalPrecision;
		evaluate! !

!DhbIncompleteBetaFunction methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 shape: aNumber2 
	"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	alpha1 := aNumber1.
	alpha2 := aNumber2.
	logNorm := ( alpha1 + alpha2) logGamma - alpha1 logGamma - alpha2 logGamma.
	^self! !

!DhbIncompleteBetaFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Compute the value of the receiver for argument aNumber.
		 Note: aNumber must be between 0 and 1 (otherwise an exception will occur)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	| norm |
	aNumber = 0
		ifTrue: [ ^0].
	aNumber = 1
		ifTrue: [ ^1].
	norm :=  ( aNumber ln * alpha1 + ( ( 1 - aNumber) ln * alpha2) + logNorm) exp.
	^( alpha1 + alpha2 + 2) * aNumber < ( alpha1 + 1)
		ifTrue: [ norm / ( ( self evaluateFraction: aNumber) * alpha1)]
		ifFalse:[ 1 - ( norm / ( ( self evaluateInverseFraction: aNumber) * alpha2))]! !

Object subclass: #DhbIncompleteGammaFunction
	instanceVariableNames: 'alpha alphaLogGamma series fraction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteGammaFunction class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
shape: aNumber
		"Defines a new instance of the receiver with paramater aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	^super new initialize: aNumber! !

!DhbIncompleteGammaFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateFraction: aNumber 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	fraction isNil 
		ifTrue: 
			[fraction := DhbIncompleteGammaFractionTermServer new.
			fraction setParameter: alpha].
	fraction setArgument: aNumber.
	^(DhbContinuedFraction server: fraction)
		desiredPrecision: DhbFloatingPointMachine new defaultNumericalPrecision;
		evaluate! !

!DhbIncompleteGammaFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateSeries: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	series isNil
		ifTrue: [ series := DhbIncompleteGammaSeriesTermServer new.
				  series setParameter: alpha.
				].
	series setArgument: aNumber.
	^(DhbInfiniteSeries server: series)
		desiredPrecision: DhbFloatingPointMachine new defaultNumericalPrecision;
		evaluate! !

!DhbIncompleteGammaFunction methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber
		"Private - Defines the parameter alpha of the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	alpha := aNumber asFloat.
	alphaLogGamma := alpha logGamma.
	^self! !

!DhbIncompleteGammaFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Compute the value of the receiver for argument aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	| x norm |
	aNumber = 0
		ifTrue: [ ^0].
	x := aNumber asFloat.
	norm := [ ( x ln * alpha - x - alphaLogGamma) exp] on: Error do: [ :signal | signal return: nil].
	norm isNil
		ifTrue: [ ^1].
	^x - 1 < alpha
		ifTrue: [ ( self evaluateSeries: x) * norm]
		ifFalse:[ 1 - ( norm / ( self evaluateFraction: x))]! !

Object subclass: #DhbIterativeProcess
	instanceVariableNames: 'precision desiredPrecision maximumIterations result iterations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbIterativeProcess subclass: #DhbClusterFinder
	instanceVariableNames: 'dataServer dataSetSize minimumRelativeClusterSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbClusterFinder class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: anInteger server: aClusterDataServer type: aClusterClass
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	^super new initialize: anInteger server: aClusterDataServer type: aClusterClass! !

!DhbClusterFinder methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aVector
		"Private - Accumulate aVector into the nearest cluster.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	( result at: ( self indexOfNearestCluster: aVector)) accumulate: aVector.! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
clusters: aCollectionOfClusters
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/18/00 "
	result := aCollectionOfClusters.! !

!DhbClusterFinder methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
collectChangesAndResetClusters
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	| hasEmptyClusters changes |
	changes := 0.
	hasEmptyClusters := false.
	result do: 
			[:each | 
			changes := each changes + changes.
			( each isInsignificantIn: self)
				ifTrue: 
					[each centerOn: nil.
					hasEmptyClusters := true]
				ifFalse: [each reset].
			].
	hasEmptyClusters 
		ifTrue: [result := result reject: [:each | each isUndefined]].
	^changes / 2! !

!DhbClusterFinder methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
dataServer: aClusterDataServer
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/18/00 "
	dataServer := aClusterDataServer.! !

!DhbClusterFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"Perform an accumulation of the data from the server.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	dataServer reset.
	dataSetSize := 0.
	[ dataServer atEnd]
		whileFalse:[ self accumulate: dataServer next.
					 dataSetSize := dataSetSize + 1.
					].
	^self collectChangesAndResetClusters! !

!DhbClusterFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
finalizeIterations
	"Close the data server.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	dataServer close! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
indexOfNearestCluster: aVector
		"Private - Answers the index of the cluster nearest to aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	| distance index |
	index := 1.
	distance := ( result at: 1) distanceTo: aVector.
	2 to: result size do:
		[ :n | | x |
		  x := ( result at: n) distanceTo: aVector.
		  x < distance
			ifTrue: [ distance := x.
					  index := n.
					].
		].
	^index! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anInteger server: aClusterDataServer type: aClusterClass
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self dataServer: aClusterDataServer.
	self clusters: ( (1 to: anInteger) collect: [ :n | aClusterClass new]).
	minimumRelativeClusterSize := 0.
	^self! !

!DhbClusterFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	dataServer open.
	result 
		do: [:each | each isUndefined ifTrue: [each centerOn: dataServer next]]! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
minimumClusterSize
	^(minimumRelativeClusterSize * dataSetSize) rounded! !

!DhbClusterFinder methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
minimumRelativeClusterSize: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	minimumRelativeClusterSize := aNumber max: 0.! !

!DhbClusterFinder methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	aStream nextPutAll: 'Iterations: '.
	iterations printOn: aStream.
	result do: [ :each | aStream cr. each printOn: aStream].! !

DhbIterativeProcess subclass: #DhbFunctionalIterator
	instanceVariableNames: 'functionBlock relativePrecision'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbFunctionalIterator subclass: #DhbBisectionZeroFinder
	instanceVariableNames: 'positiveX negativeX'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeInitialValues
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	positiveX isNil
		ifTrue: [ self error: 'No positive value supplied'].
	negativeX isNil
		ifTrue: [ self error: 'No negative value supplied'].! !

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"Perform one step of bisection.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	result := ( positiveX + negativeX) * 0.5.
	( functionBlock value: result) > 0
		ifTrue: [ positiveX := result]
		ifFalse:[ negativeX := result].
	^self relativePrecision: ( positiveX - negativeX) abs! !

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
findNegativeXFrom: aNumber1 range: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	| n |
	n := 0.
	[ negativeX := Number random * aNumber2 + aNumber1.
	  ( functionBlock value: negativeX) < 0
		] whileFalse: [ n := n + 0.1.
						n > maximumIterations
							ifTrue: [ self error: 'Unable to find a negative function value'].
					  ].! !

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
findPositiveXFrom: aNumber1 range: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	| n |
	n := 0.
	[ positiveX := Number random * aNumber2 + aNumber1.
	  ( functionBlock value: positiveX) > 0
		] whileFalse: [ n := n + 1.
						n > maximumIterations
							ifTrue: [ self error: 'Unable to find a positive function value'].
					  ].! !

!DhbBisectionZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setNegativeX: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	( functionBlock value: aNumber) < 0
		ifFalse:[ self error: 'Function is not negative at x = ', aNumber printString].
	negativeX := aNumber.! !

!DhbBisectionZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setPositiveX: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	( functionBlock value: aNumber) > 0
		ifFalse:[ self error: 'Function is not positive at x = ', aNumber printString].
	positiveX := aNumber.! !

DhbFunctionalIterator subclass: #DhbFunctionOptimizer
	instanceVariableNames: 'optimizingPointClass bestPoints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFunctionOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^super defaultPrecision * 100! !

!DhbFunctionOptimizer class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
forOptimizer: aFunctionOptimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^self new initializeForOptimizer: aFunctionOptimizer! !

!DhbFunctionOptimizer class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
maximizingFunction: aFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new initializeAsMaximizer; setFunction: aFunction! !

!DhbFunctionOptimizer class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
minimizingFunction: aFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new initializeAsMinimizer; setFunction: aFunction! !

!DhbFunctionOptimizer methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
addPointAt: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	bestPoints 
		add: (optimizingPointClass vector: aNumber function: functionBlock)! !

!DhbFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
bestPoints
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^bestPoints! !

!DhbFunctionOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	result := bestPoints first position.! !

!DhbFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
functionBlock
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^functionBlock! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialValue: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	result := aVector copy.! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	bestPoints := SortedCollection sortBlock: [ :a :b | a betterThan: b].
	^super initialize! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeAsMaximizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	optimizingPointClass := DhbMaximizingPoint.
	^self initialize! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeAsMinimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	optimizingPointClass := DhbMinimizingPoint.
	^self! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeForOptimizer: aFunctionOptimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	optimizingPointClass := aFunctionOptimizer pointClass.
	functionBlock := aFunctionOptimizer functionBlock.
	^self initialize! !

!DhbFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
pointClass
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^optimizingPointClass! !

!DhbFunctionOptimizer methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	super printOn: aStream.
	bestPoints do: [ :each | aStream cr. each printOn: aStream].! !

DhbFunctionOptimizer subclass: #DhbGeneticOptimizer
	instanceVariableNames: 'chromosomeManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbGeneticOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^500! !

!DhbGeneticOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^0! !

!DhbGeneticOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
chromosomeManager: aChromosomeManager
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	chromosomeManager := aChromosomeManager.
	^self! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
collectPoints
	"Private - 
	(c) Copyrights Didier BESSET, 2000, all rights reserved. 
	Initial code: 23-Feb-00"

	| bestPoint |
	bestPoints isEmpty not ifTrue: [bestPoint := bestPoints removeFirst].
	bestPoints removeAll: bestPoints asArray.
	chromosomeManager population do: [:each | self addPointAt: each].
	bestPoint notNil ifTrue: [bestPoints add: bestPoint].
	result := bestPoints first position! !

!DhbGeneticOptimizer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computePrecision
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^1! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| randomScale |
	randomScale := self randomScale.
	chromosomeManager reset.
	[ chromosomeManager isFullyPopulated]
		whileFalse: [ self processRandomParents: randomScale].
	self collectPoints.
	^self computePrecision! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeIterations
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	chromosomeManager randomizePopulation.
	self collectPoints! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
processRandomParents: aNumberArray
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	chromosomeManager process: ( bestPoints at: ( self randomIndex: aNumberArray)) position
						and:  ( bestPoints at: ( self randomIndex: aNumberArray)) position.! !

!DhbGeneticOptimizer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
randomIndex: aNumberArray
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| x n |
	x := Number random.
	n := 1.
	aNumberArray do: 
		[ :each |
		  x < each
			ifTrue: [ ^n].
		  n := n + 1.
		].
	^aNumberArray size	"Never reached unless an error occurs"! !

!DhbGeneticOptimizer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
randomScale
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| norm fBest fWorst answer|
	fBest := bestPoints first value.
	fWorst := bestPoints last value.
	norm := 1 / ( fBest - fWorst).
	answer := bestPoints collect: [ :each | (each value - fWorst) * norm].
	norm := 1 / ( answer inject: 0 into: [ :sum :each | each + sum]).
	fBest := 0.
	^answer collect: [ :each | fBest := each * norm + fBest. fBest]! !

DhbFunctionOptimizer subclass: #DhbHillClimbingOptimizer
	instanceVariableNames: 'unidimensionalFinder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbHillClimbingOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeInitialValues
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 05-Jan-00 "
	unidimensionalFinder := DhbOneVariableFunctionOptimizer forOptimizer: self.
	unidimensionalFinder desiredPrecision: desiredPrecision.
	bestPoints := ( 1 to: result size)
							collect: [ :n |
									   ( DhbVectorProjectedFunction function: functionBlock)
											direction: ( ( DhbVector new: result size)
															atAllPut: 0;
															at: n put: 1;
															yourself);
											yourself
									 ].! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| oldResult |
	precision := 1.0.
	bestPoints inject: result
				 into: [ :prev :each | ( self minimizeDirection: each from: prev)].
	self shiftDirections.
	self minimizeDirection: bestPoints last.
	oldResult := result.
	result := bestPoints last origin.
	precision := 0.0.
	result with: oldResult do:
		[ :x0 :x1 |
		  precision := ( self precisionOf: (x0 - x1) abs relativeTo: x0 abs) max: precision.
		].
	^precision! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
minimizeDirection: aVectorFunction 
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^unidimensionalFinder
		reset;
		setFunction: aVectorFunction;
		addPointAt: 0;
		addPointAt: precision;
		addPointAt: precision negated;
		evaluate! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
minimizeDirection: aVectorFunction from: aVector 
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	^aVectorFunction
		origin: aVector;
		argumentWith: ( self minimizeDirection: aVectorFunction)! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
shiftDirections
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| position delta firstDirection |
	firstDirection := bestPoints first direction.
	bestPoints inject: nil
					into: [ :prev :each |
							position isNil
								ifTrue: [ position := each origin]
								ifFalse:[ prev direction: each direction].
							each
							].
	position := bestPoints last origin - position.
	delta := position norm.
	delta > desiredPrecision
		ifTrue: [ bestPoints last direction: (position scaleBy: (1 / delta))]
		ifFalse:[ bestPoints last direction: firstDirection].! !

DhbFunctionOptimizer subclass: #DhbMultiVariableGeneralOptimizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeInitialValues
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	self range notNil
		ifTrue: [ self performGeneticOptimization].
	self performSimplexOptimization.! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| optimizer |
	optimizer := DhbHillClimbingOptimizer forOptimizer: self.
	optimizer desiredPrecision: desiredPrecision;
			  maximumIterations: maximumIterations;
			  initialValue: result.
	result := optimizer evaluate.
	^optimizer precision! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
origin
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	^result! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
origin: anArrayOrVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	result := anArrayOrVector.! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
performGeneticOptimization
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	| optimizer manager |
	optimizer := DhbGeneticOptimizer forOptimizer: self.
	manager := DhbVectorChromosomeManager new: 100 mutation: 0.1 crossover: 0.1.
	manager origin: self origin asVector; range: self range asVector.
	optimizer chromosomeManager: manager.
	result := optimizer evaluate.! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
performSimplexOptimization
	"Private - 
	(c) Copyrights Didier BESSET, 2000, all rights reserved. 
	Initial code: 2/29/00"

	| optimizer |
	optimizer := DhbSimplexOptimizer forOptimizer: self.
	optimizer desiredPrecision: desiredPrecision sqrt; maximumIterations: maximumIterations; initialValue: result asVector.
	result := optimizer evaluate! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
range
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	^self bestPoints! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
range: anArrayOrVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	bestPoints := anArrayOrVector.! !

DhbFunctionOptimizer subclass: #DhbOneVariableFunctionOptimizer
	instanceVariableNames: ''
	classVariableNames: 'GoldenSection'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbOneVariableFunctionOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^DhbFloatingPointMachine new defaultNumericalPrecision * 10! !

!DhbOneVariableFunctionOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
goldenSection
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	GoldenSection isNil ifTrue: [GoldenSection := (3 - 5 sqrt) / 2].
	^GoldenSection! !

!DhbOneVariableFunctionOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeInitialValues
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	[ bestPoints size  > 3] whileTrue: [ bestPoints removeLast].
	bestPoints size = 3
		ifTrue: [ self hasBracketingPoints
					ifFalse:[ bestPoints removeLast].
				].
	bestPoints size < 3
		ifTrue: [ ( DhbOptimizingBracketFinder forOptimizer: self) evaluate].! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computePrecision
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^self precisionOf: ( ( bestPoints at: 2) position - ( bestPoints at: 3) position) abs
		   relativeTo: ( bestPoints at: 1) position abs! !

!DhbOneVariableFunctionOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	self addPointAt: self nextXValue.
	bestPoints removeAtIndex: self indexOfOuterPoint.
	^self computePrecision! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
hasBracketingPoints
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| x1 |
	x1 := ( bestPoints at: 1) position.
	^( ( bestPoints at: 2) position - x1) * (( bestPoints at: 3) position - x1) < 0! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
indexOfOuterPoint
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| inferior superior x |
	inferior := false.
	superior := false.
	x := bestPoints first position.
	2 to: 4 do:
		[ :n |
		  ( bestPoints at: n) position < x
				ifTrue: [ inferior
							ifTrue: [ ^n].
						  inferior := true.
						]
				ifFalse:[ superior
							ifTrue: [ ^n].
						  superior := true.
						].
		].! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextXValue
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| d3 d2 x1 |
	x1 := ( bestPoints at: 1) position.
	d2 := ( bestPoints at: 2) position - x1.
	d3 := ( bestPoints at: 3) position - x1.
	^( d3 abs > d2 abs ifTrue: [ d3]
					   ifFalse:[ d2]) * self class goldenSection + x1! !

!DhbOneVariableFunctionOptimizer methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	[ bestPoints isEmpty] whileFalse: [ bestPoints removeLast].! !

DhbOneVariableFunctionOptimizer subclass: #DhbOptimizingBracketFinder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbOptimizingBracketFinder class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialPoints: aSortedCollection function: aFunction
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new setInitialPoints: aSortedCollection; setFunction: aFunction! !

!DhbOptimizingBracketFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeInitialValues
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	[bestPoints size < 2] whileTrue: [self addPointAt: Number random]! !

!DhbOptimizingBracketFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| x1 x2 |
	x1 := ( bestPoints at: 1) position.
	x2 := ( bestPoints at: 2) position.
	self addPointAt: ( x1 * 3 - ( x2 * 2)).
	precision := ( x2 - x1) * ( ( bestPoints at: 3) position - x1).
	self hasConverged
		ifFalse:[ bestPoints removeLast].
	^precision! !

!DhbOptimizingBracketFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	result := bestPoints.! !

!DhbOptimizingBracketFinder methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeForOptimizer: aFunctionOptimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	super initializeForOptimizer: aFunctionOptimizer.
	bestPoints := aFunctionOptimizer bestPoints.
	^self! !

!DhbOptimizingBracketFinder methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setInitialPoints: aSortedCollection
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	bestPoints := aSortedCollection.! !

DhbFunctionOptimizer subclass: #DhbSimplexOptimizer
	instanceVariableNames: 'worstVector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbSimplexOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^DhbFloatingPointMachine new defaultNumericalPrecision * 1000! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
buildInitialSimplex
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	| projectedFunction finder partialResult |
	projectedFunction := DhbProjectedOneVariableFunction 
				function: functionBlock.
	finder := DhbOneVariableFunctionOptimizer forOptimizer: self.
	finder setFunction: projectedFunction.
	[bestPoints size < (result size + 1)] whileTrue: 
			[projectedFunction
				setArgument: result;
				bumpIndex.
			partialResult := finder
						reset;
						evaluate.
			bestPoints add: (optimizingPointClass 
						vector: (projectedFunction argumentWith: partialResult)
						function: functionBlock)]! !

!DhbSimplexOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeInitialValues
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	bestPoints 
		add: (optimizingPointClass vector: result function: functionBlock).
	self buildInitialSimplex.
	worstVector := bestPoints removeLast position! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computePrecision
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/16/00 "
	| functionValues bestFunctionValue |
	functionValues := bestPoints collect: [ :each | each value].
	bestFunctionValue := functionValues removeFirst.
	^functionValues inject: 0
					into: [ :max :each | ( self precisionOf: ( each - bestFunctionValue) abs relativeTo: bestFunctionValue abs) max: max]! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
contract
	"Private - Contract the Simplex around the best Vector.
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/16/00 "

	| bestVector oldVectors |
	bestVector := bestPoints first position.
	oldVectors := OrderedCollection with: worstVector.
	[bestPoints size > 1] whileTrue: [oldVectors add: bestPoints removeLast position].
	oldVectors do: [:each | self contract: each around: bestVector].
	worstVector := bestPoints removeLast position.
	^self computePrecision! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
contract: aVector around: bestVector 
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/16/00 "

	bestPoints 
		add: (optimizingPointClass vector: bestVector * 0.5 + (aVector * 0.5)
				function: functionBlock)! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	| centerOfGravity newPoint nextPoint |
	centerOfGravity := (bestPoints inject: ((worstVector copy)
						atAllPut: 0;
						yourself)
				into: [:sum :each | each position + sum]) * (1 / bestPoints size).
	newPoint := optimizingPointClass vector: 2 * centerOfGravity - worstVector
				function: functionBlock.
	(newPoint betterThan: bestPoints first) 
		ifTrue: 
			[nextPoint := optimizingPointClass 
						vector: newPoint position * 2 - centerOfGravity
						function: functionBlock.
			(nextPoint betterThan: newPoint) ifTrue: [newPoint := nextPoint]]
		ifFalse: 
			[newPoint := optimizingPointClass 
						vector: centerOfGravity * 0.666667 + (worstVector * 0.333333)
						function: functionBlock.
			(newPoint betterThan: bestPoints first) ifFalse: [^self contract]].
	worstVector := bestPoints removeLast position.
	bestPoints add: newPoint.
	result := bestPoints first position.
	^self computePrecision! !

!DhbSimplexOptimizer methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	super printOn: aStream.
	aStream cr. 
	worstVector printOn: aStream.! !

!DhbFunctionalIterator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
function: aBlock
		"Convenience method to create a instance with given function block.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^self new setFunction: aBlock; yourself! !

!DhbFunctionalIterator methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeInitialValues
	self subclassResponsibility! !

!DhbFunctionalIterator methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeIterations
	"If no initial value has been defined, take 0 as the starting point (for lack of anything better).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	functionBlock isNil ifTrue: [self error: 'No function supplied'].
	self computeInitialValues! !

!DhbFunctionalIterator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
relativePrecision: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/4/99 "
	^self precisionOf: aNumber relativeTo: result abs! !

!DhbFunctionalIterator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setFunction: aBlock
		"Defines the function for which zeroes will be found.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	( aBlock respondsTo: #value:)
		ifFalse:[ self error: 'Function block must implement the method value:'].
	functionBlock := aBlock.! !

DhbFunctionalIterator subclass: #DhbNewtonZeroFinder
	instanceVariableNames: 'derivativeBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbNewtonZeroFinder class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
function: aBlock1 derivative: aBlock2
		"Convenience method to create a instance with given function block.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^(self new) setFunction: aBlock1; setDerivative: aBlock2; yourself! !

!DhbNewtonZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeInitialValues
		"Private - If no derivative has been defined, take an ad-hoc definition.
		 If no initial value has been defined, take 0 as the starting point (for lack of anything better).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| n |
	result isNil
		ifTrue: [ result := 0].
	derivativeBlock isNil
		ifTrue: [ derivativeBlock := self defaultDerivativeBlock].
	n := 0.
	[ (derivativeBlock value: result) equalsTo: 0]
		whileTrue: [ n := n + 1.
					 n > maximumIterations
						ifTrue: [ self error: 'Function''s derivative seems to be zero everywhere'].
					 result := Number random + result].! !

!DhbNewtonZeroFinder methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultDerivativeBlock
		"Private - Answers a block computing the function's derivative by approximation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^[ :x | 5000 * ( ( functionBlock value: (x + 0.0001)) - ( functionBlock value: (x - 0.0001)))]! !

!DhbNewtonZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"Compute one step of Newton's zero finding method. Answers the estimated precision.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| delta |
	delta := ( functionBlock value: result) / ( derivativeBlock value: result).
	result := result - delta.
	^self relativePrecision: delta abs! !

!DhbNewtonZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialValue: aNumber
		"Define the initial value for the iterations.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	result := aNumber.! !

!DhbNewtonZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setDerivative: aBlock 
	"Defines the derivative of the function for which zeroes will be found. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 7/1/99"

	| x |
	(aBlock respondsTo: #value:)
		ifFalse: [self error: 'Derivative block must implement the method value:'].
	x := result isNil
				ifTrue: [Number random]
				ifFalse: [result + Number random].
	((aBlock value: x)
		relativelyEqualsTo: (self defaultDerivativeBlock value: x)
		upTo: 1.0e-4)
		ifFalse: [self error: 'Supplied derivative is not correct'].
	derivativeBlock := aBlock! !

!DhbNewtonZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setFunction: aBlock
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	super setFunction: aBlock.
	derivativeBlock := nil.! !

DhbFunctionalIterator subclass: #DhbTrapezeIntegrator
	instanceVariableNames: 'from to sum step'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbTrapezeIntegrator subclass: #DhbRombergIntegrator
	instanceVariableNames: 'order points interpolator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbRombergIntegrator class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultOrder
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	^5! !

!DhbRombergIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeInitialValues
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	super computeInitialValues.
	points := OrderedCollection new: order.
	interpolator := DhbNevilleInterpolator points: points.
	points add: 1 @ sum.! !

!DhbRombergIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	| interpolation |
	points addLast: (points last x * 0.25) @ self higherOrderSum.
	points size < order
		ifTrue: [ ^1].
	interpolation := interpolator valueAndError: 0.
	points removeFirst.
	result := interpolation at: 1.
	^self relativePrecision: ( interpolation at: 2) abs! !

!DhbRombergIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
		"Private - initialize the parameters of the receiver with default values.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	order := self class defaultOrder.
	^super initialize! !

!DhbRombergIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
order: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	anInteger < 2
		ifTrue: [ self error: 'Order for Romberg integration must be larger than 1'].
	order := anInteger.! !

DhbTrapezeIntegrator subclass: #DhbSimpsonIntegrator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbSimpsonIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/4/99 "
	| oldResult oldSum |
	iterations < 2
		ifTrue: [ self higherOrderSum.
				  ^1
				].
	oldResult := result.
	oldSum := sum.
	result := (self higherOrderSum * 4 - oldSum) / 3.
	^self relativePrecision: ( result - oldResult) abs! !

!DhbTrapezeIntegrator class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^13! !

!DhbTrapezeIntegrator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
function: aBlock from: aNumber1 to: aNumber2
		"Create an new instance with given parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^super new initialize: aBlock from: aNumber1 to: aNumber2! !

!DhbTrapezeIntegrator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Private - Block the constructor method for this class.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^self error: 'Method new:from:to: must be used'! !

!DhbTrapezeIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeInitialValues
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	step := to - from.
	sum := ( ( functionBlock value: from) + ( functionBlock value: to)) * step /2.
	result := sum.! !

!DhbTrapezeIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	| oldResult |
	oldResult := result.
	result := self higherOrderSum.
	^self relativePrecision: ( result - oldResult) abs! !

!DhbTrapezeIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
from: aNumber1 to: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	from := aNumber1.
	to := aNumber2.! !

!DhbTrapezeIntegrator methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
higherOrderSum
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	| x newSum |
	x := step / 2 + from.
	newSum := 0.
	[ x < to ]
		whileTrue: [ newSum := ( functionBlock value: x) + newSum.
					 x := x + step.
				   ].
	sum := ( step * newSum + sum) / 2.
	step := step / 2.
	^sum! !

!DhbTrapezeIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aBlock from: aNumber1 to: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	functionBlock := aBlock.
	self from: aNumber1 to: aNumber2.
	^self! !

DhbIterativeProcess subclass: #DhbIncompleteBetaFunctionFraction
	instanceVariableNames: 'x q1 q2 q3 numerator denominator alpha1 alpha2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteBetaFunctionFraction class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
shape: aNumber1 shape: aNumber2 
	"Create an instance of the receiver with given shape parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 shape: aNumber2! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
	"Compute and add the next term of the fraction.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	| m m2 temp |
	m := iterations + 1.
	m2 := m * 2.
	temp := m * (alpha2 - m) * x 
				/ ((q3 + m2) * (alpha1 + m2)).
	denominator := self limitedSmallValue: ( denominator * temp + 1).
	numerator := self limitedSmallValue: ( temp / numerator + 1).
	denominator := 1 / denominator.
	result := result * numerator * denominator.
	temp := (alpha1 + m) negated * (q1 + m) * x 
				/ ((q2 + m2) * (alpha1 + m2)).
	denominator := self limitedSmallValue: ( denominator * temp + 1).
	numerator := self limitedSmallValue: ( temp / numerator + 1).
	denominator := 1 / denominator.
	temp := numerator * denominator.
	result := result * temp.
	^(temp - 1) abs! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 shape: aNumber2 
	"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	alpha1 := aNumber1.
	alpha2 := aNumber2.
	q1 := alpha1 + alpha2.
	q2 := alpha1 + 1.
	q3 := alpha1 - 1.
	^self! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeIterations
	"Initialize the iterations (subclasses must write their own method and call this one last).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	numerator := 1.
	denominator := 1 / (self limitedSmallValue: 1 - (q1 * x / q2)).
	result := denominator! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setArgument: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	x := aNumber.! !

DhbIterativeProcess subclass: #DhbInfiniteSeries
	instanceVariableNames: 'termServer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbInfiniteSeries subclass: #DhbContinuedFraction
	instanceVariableNames: 'numerator denominator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbContinuedFraction methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"Perform one iteration.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	| terms delta |
	terms := termServer termsAt: iterations.
	denominator := 1 / ( self limitedSmallValue: ( (terms at: 1) * denominator + (terms at: 2))).
	numerator := self limitedSmallValue: ( (terms at: 1) / numerator + (terms at: 2)).
	delta := numerator * denominator.
	result := result * delta.
	^( delta - 1) abs! !

!DhbContinuedFraction methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeIterations
	"Initialize the series.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	numerator := self limitedSmallValue: termServer initialTerm.
	denominator := 0.
	result := numerator! !

!DhbInfiniteSeries class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
server: aTermServer
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	^self new initialize: aTermServer! !

!DhbInfiniteSeries methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"Perform one iteration.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	| delta |
	delta := termServer termAt: iterations.
	result := result + delta.
	^self precisionOf: delta abs relativeTo: result abs! !

!DhbInfiniteSeries methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aTermServer
		"Private - Assigns the object responsible to compute each term.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	termServer := aTermServer.
	^self! !

!DhbInfiniteSeries methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeIterations
	"Initialize the series.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	result := termServer initialTerm! !

!DhbIterativeProcess class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^50! !

!DhbIterativeProcess class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^DhbFloatingPointMachine new defaultNumericalPrecision! !

!DhbIterativeProcess class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Create an instance of the class.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^super new initialize! !

!DhbIterativeProcess methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
desiredPrecision: aNumber
		"Defines the desired precision for the result.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	aNumber > 0
		ifFalse: [ ^self error: 'Illegal precision: ', aNumber printString].
	desiredPrecision := aNumber.! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluate
	"Perform the iteration until either the desired precision is attained or the number of iterations exceeds the maximum.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	iterations := 0.
	self initializeIterations.
	
	[iterations := iterations + 1.
	precision := self evaluateIteration.
	self hasConverged or: [iterations >= maximumIterations]] 
			whileFalse: [].
	self finalizeIterations.
	^self result! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"Dummy method (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^self subclassResponsibility! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
finalizeIterations
	"Perform cleanup operation if needed (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
hasConverged
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/4/99 "
	^precision <= desiredPrecision! !

!DhbIterativeProcess methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
		"Private - initialize the parameters of the receiver with default values.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	desiredPrecision := self class defaultPrecision.
	maximumIterations := self class defaultMaximumIterations.
	^self! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeIterations
	"Initialize the iterations (must be implemented by subclass when needed).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
iterations
		"Answers the number of iterations performed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^iterations! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
limitedSmallValue: aNumber
		"Private - prevent aNumber from being smaller in absolute value than a small number.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	^aNumber abs < DhbFloatingPointMachine new smallNumber
			ifTrue: [ DhbFloatingPointMachine new smallNumber]
			ifFalse:[ aNumber]! !

!DhbIterativeProcess methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
maximumIterations: anInteger
		"Defines the maximum number of iterations.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	( anInteger isInteger and: [ anInteger > 1])
		ifFalse: [ ^self error: 'Invalid maximum number of iteration: ', anInteger printString].
	maximumIterations := anInteger.! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
precision
		"Answer the attained precision for the result.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^precision! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
precisionOf: aNumber1 relativeTo: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/5/99 "
	^aNumber2 > DhbFloatingPointMachine new defaultNumericalPrecision
		ifTrue: [ aNumber1 / aNumber2]
		ifFalse:[ aNumber1]! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
result
		"Answer the result of the iterations (if any)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^result! !

DhbIterativeProcess subclass: #DhbJacobiTransformation
	instanceVariableNames: 'lowerRows transform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbJacobiTransformation class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
matrix: aSymmetricMatrix
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	^super new initialize: aSymmetricMatrix! !

!DhbJacobiTransformation class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Prevent using this message to create instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbJacobiTransformation methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| indices |
	indices := self largestOffDiagonalIndices.
	self transformAt: ( indices at: 1) and: ( indices at: 2).
	^precision! !

!DhbJacobiTransformation methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
exchangeAt: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| temp n |
	n := anInteger + 1.
	temp := result at: n.
	result at: n put: ( result at: anInteger).
	result at: anInteger put: temp.
	transform do:
		[ :each |
		  temp := each at: n.
		  each at: n put: ( each at: anInteger).
		  each at: anInteger put: temp.
		].! !

!DhbJacobiTransformation methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
finalizeIterations
	"Transfer the eigenValues into a vector and set this as the result.
		 eigen values and transform matrix are sorted using a bubble sort.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "

	| n |
	n := 0.
	result := lowerRows collect: 
					[:each | 
					n := n + 1.
					each at: n].
	self sortEigenValues! !

!DhbJacobiTransformation methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aSymmetricMatrix
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| n m |
	n := aSymmetricMatrix numberOfRows.
	lowerRows := Array new: n.
	transform := Array new: n.
	1 to: n do:
		[ :k |
		  lowerRows at: k put: ( ( aSymmetricMatrix rowAt: k) copyFrom: 1 to: k).
		  transform at: k put: ( ( Array new: n) atAllPut: 0; at: k put: 1; yourself).
		].
	^self! !

!DhbJacobiTransformation methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
largestOffDiagonalIndices
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| n m abs |
	n := 2.
	m := 1.
	precision := ( ( lowerRows at: n) at: m) abs.
	1 to: lowerRows size do:
		[ :i |
		  1 to: ( i - 1) do:
			[ :j |
			  abs := ( ( lowerRows at: i) at: j) abs.
			  abs > precision
				ifTrue: [ n := i.
						  m := j.
						  precision := abs.
						]. 
			].
		].
	^Array with: m with: n! !

!DhbJacobiTransformation methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"Append to the argument aStream, a sequence of characters that describes the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| first |
	first := true.
	lowerRows do: 
		[ :each |
		  first ifTrue: [ first := false]
				 ifFalse:[ aStream cr].
		  each printOn: aStream.
		].! !

!DhbJacobiTransformation methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sortEigenValues
		"Private - Use a bubble sort.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| n bound m |
	n := lowerRows size.
	bound := n.
	[ bound = 0 ]
		whileFalse: [ m := 0.
					  1 to: bound - 1 do:
						[ :j |
						  ( result at: j) abs > ( result at: j + 1) abs
							ifFalse:[ self exchangeAt: j.
									  m := j.
									].
						].
					  bound := m.
					].! !

!DhbJacobiTransformation methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
transform
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	^DhbMatrix rows: transform! !

!DhbJacobiTransformation methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
transformAt: anInteger1 and: anInteger2
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| d t s c tau apq app aqq arp arq |
	apq := ( lowerRows at: anInteger2) at: anInteger1.
	apq = 0
		ifTrue: [ ^nil].
	app := ( lowerRows at: anInteger1) at: anInteger1.
	aqq := ( lowerRows at: anInteger2) at: anInteger2.
	d := aqq - app.
	arp := d * 0.5 / apq.
	t := arp > 0 ifTrue: [ 1 / ( ( arp squared + 1) sqrt + arp)]
				 ifFalse:[ 1 / ( arp - ( arp squared + 1) sqrt)].
	c := 1 / ( t squared + 1) sqrt.
	s := t * c.
	tau := s / ( 1 + c).
	1 to: ( anInteger1 - 1)
		do: [ :r |
			  arp := ( lowerRows at: anInteger1) at: r.
			  arq := ( lowerRows at: anInteger2) at: r.
			  ( lowerRows at: anInteger1) at: r put: ( arp - ( s * (tau * arp + arq))).
			  ( lowerRows at: anInteger2) at: r put: ( arq + ( s * (arp - (tau * arq)))).
			].
	( anInteger1 + 1) to: ( anInteger2 - 1)
		do: [ :r |
			  arp := ( lowerRows at: r) at: anInteger1.
			  arq := ( lowerRows at: anInteger2) at: r.
			  ( lowerRows at: r) at: anInteger1 put: ( arp - ( s * (tau * arp + arq))).
			  ( lowerRows at: anInteger2) at: r put: ( arq + ( s * (arp - (tau * arq)))).
			].
	( anInteger2 + 1) to: lowerRows size
		do: [ :r |
			  arp := ( lowerRows at: r) at: anInteger1.
			  arq := ( lowerRows at: r) at: anInteger2.
			  ( lowerRows at: r) at: anInteger1 put: ( arp - ( s * (tau * arp + arq))).
			  ( lowerRows at: r) at: anInteger2 put: ( arq + ( s * (arp - (tau * arq)))).
			].
	1 to: lowerRows size
		do: [ :r |
			  arp := ( transform at: r) at: anInteger1.
			  arq := ( transform at: r) at: anInteger2.
			  ( transform at: r) at: anInteger1 put: ( arp - ( s * (tau * arp + arq))).
			  ( transform at: r) at: anInteger2 put: ( arq + ( s * (arp - (tau * arq)))).
			].
	( lowerRows at: anInteger1) at: anInteger1 put: ( app - (t * apq)).
	( lowerRows at: anInteger2) at: anInteger2 put: ( aqq + (t * apq)).
	( lowerRows at: anInteger2) at: anInteger1 put: 0.! !

DhbIterativeProcess subclass: #DhbLargestEigenValueFinder
	instanceVariableNames: 'matrix eigenvector transposeEigenvector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLargestEigenValueFinder class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^100! !

!DhbLargestEigenValueFinder class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
matrix: aMatrix
		"Create a new instance of the receiver for a given matrix and default precision.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^( self new) initialize: aMatrix; yourself! !

!DhbLargestEigenValueFinder class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
matrix: aMatrix precision: aNumber
		"Create a new instance of the receiver for a given matrix and desired precision.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^( self new) initialize: aMatrix; desiredPrecision: aNumber; yourself! !

!DhbLargestEigenValueFinder methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
eigenvalue
	"Answer the eigen value found by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^result! !

!DhbLargestEigenValueFinder methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
eigenvector
	"Answer the normalized eigen vector found by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^eigenvector * (1 / eigenvector norm)! !

!DhbLargestEigenValueFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
	"Iterate the product of the matrix of the eigen vector and the transpose.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	| oldEigenvalue |
	oldEigenvalue := result.
	transposeEigenvector := transposeEigenvector * matrix.
	transposeEigenvector := transposeEigenvector 
				* (1 / (transposeEigenvector at: 1)).
	eigenvector := matrix * eigenvector.
	result := eigenvector at: 1.
	eigenvector := eigenvector * (1 / result).
	^oldEigenvalue isNil 
		ifTrue: [2 * desiredPrecision]
		ifFalse: [(result - oldEigenvalue) abs]! !

!DhbLargestEigenValueFinder methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aMatrix
		"Defines the matrix for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	matrix := aMatrix.! !

!DhbLargestEigenValueFinder methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeIterations
	"Initialize the iterations (subclasses must write their own method and call this one last).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	eigenvector := DhbVector new: matrix numberOfRows.
	eigenvector atAllPut: 1.0.
	transposeEigenvector := DhbVector new: eigenvector size.
	transposeEigenvector atAllPut: 1.0! !

!DhbLargestEigenValueFinder methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextLargestEigenValueFinder
	"Return an eigen value finder for the same eigen values of the receiver except the one found.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	| norm |
	norm := 1 / (eigenvector * transposeEigenvector).
	^self class 
		matrix: matrix * ((DhbSymmetricMatrix identity: eigenvector size) 
						- (eigenvector * norm tensorProduct: transposeEigenvector))
		precision: desiredPrecision! !

DhbIterativeProcess subclass: #DhbLeastSquareFit
	instanceVariableNames: 'dataHolder errorMatrix chiSquare equations constants degreeOfFreedom'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLeastSquareFit class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
histogram: aHistogram distributionClass: aProbabilityDensityFunctionClass 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "

	^self points: aHistogram
		function: (DhbScaledProbabilityDensityFunction histogram: aHistogram
				distributionClass: aProbabilityDensityFunctionClass)! !

!DhbLeastSquareFit class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
points: aDataHolder function: aParametricFunction 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 11/3/99"

	^aParametricFunction isNil
		ifTrue: [nil]
		ifFalse: [super new initialize: aDataHolder data: aParametricFunction]! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aWeightedPoint
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	| f g |
	f := result valueAndGradient: aWeightedPoint xValue.
	g := f last.
	f := f first.
	constants accumulate: g * ( ( aWeightedPoint yValue - f) * aWeightedPoint weight).
	1 to: g size do:
		[ :k |
		  ( equations at: k) accumulate: g * ( ( g at: k) * aWeightedPoint weight).
		].! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulateEquationSystem
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	dataHolder pointsAndErrorsDo: [ :each | self accumulate: each].! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
chiSquare
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	chiSquare isNil
		ifTrue: [ self computeChiSquare].
	^chiSquare! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeChanges
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	errorMatrix := DhbLUPDecomposition direct: equations.
	^errorMatrix solve: constants! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeChiSquare
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	chiSquare := 0.
	degreeOfFreedom := self numberOfFreeParameters negated.
	dataHolder pointsAndErrorsDo:
		[ :each |
		  chiSquare := ( each chi2Contribution: result) + chiSquare.
		  degreeOfFreedom := degreeOfFreedom + 1.
		].! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeEquationSystem
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	constants atAllPut: 0.
	equations do: [ :each | each atAllPut: 0].
	self accumulateEquationSystem.! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
confidenceLevel
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/6/00 "
	^( DhbChiSquareDistribution degreeOfFreedom: self degreeOfFreedom) confidenceLevel: self chiSquare! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
degreeOfFreedom
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	degreeOfFreedom isNil
		ifTrue: [ self computeChiSquare].
	^degreeOfFreedom! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
errorMatrix
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 14/4/99 "
	^DhbSymmetricMatrix rows: errorMatrix inverseMatrixComponents! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluateIteration
		"Dummy method (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| changes maxChange|
	self computeEquationSystem.
	changes := self computeChanges.
	result changeParametersBy: changes.
	maxChange := 0.
	result parameters with: changes do: 
		[ :r :d | maxChange := ( d / r) abs max: maxChange].
	^maxChange! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
finalizeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	equations := nil.
	constants := nil.
	degreeOfFreedom := nil.
	chiSquare := nil! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fitType
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^'Least square fit'! !

!DhbLeastSquareFit methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aDataHolder data: aParametricFunction
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	dataHolder := aDataHolder.
	result := aParametricFunction.
	^self! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	| n |
	n := self numberOfParameters.
	constants := (DhbVector new: n)
				atAllPut: 0;
				yourself.
	equations := (1 to: n) collect: 
					[:k | 
					(DhbVector new: n)
						atAllPut: 0;
						yourself]! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
numberOfFreeParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^self numberOfParameters! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
numberOfParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^result parameters size! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/8/00 "
	^result value: aNumber! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
valueAndError: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/8/00 "
	| valueGradient |
	valueGradient := result valueAndGradient: aNumber.
	^Array with: valueGradient first
		   with: ( valueGradient last * ( self errorMatrix * valueGradient last)) sqrt! !

DhbLeastSquareFit subclass: #DhbMaximumLikekihoodHistogramFit
	instanceVariableNames: 'count countVariance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMaximumLikekihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aWeightedPoint
		"Private - 
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| f g temp inverseProbability|
	f := result valueAndGradient: aWeightedPoint xValue.
	g := f last copyFrom: 1 to: ( f last size - 1).
	f := f first.
	f = 0 ifTrue: [ ^nil].
	inverseProbability := 1 / f.
	temp := aWeightedPoint yValue * inverseProbability.
	constants accumulate: g * temp.
	temp := temp * inverseProbability.
	1 to: g size do:
		[ :k |
		  ( equations at: k) accumulate: g * ( ( g at: k) * temp).
		].! !

!DhbMaximumLikekihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeChanges
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^super computeChanges copyWith: 0! !

!DhbMaximumLikekihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeNormalization
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "

	| numerator denominator temp |
	numerator := 0.
	denominator := 0.
	dataHolder pointsAndErrorsDo: 
			[:each | 
			temp := result value: each xValue.
			temp = 0 
				ifFalse: 
					[numerator := numerator + (each yValue squared / temp).
					denominator := denominator + temp]].
	count := ( numerator / denominator) sqrt.
	countVariance := numerator / ( 4 * count).! !

!DhbMaximumLikekihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
finalizeIterations
	"Compute the normalization factor.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	self computeNormalization.
	result setCount: count.
	super finalizeIterations! !

!DhbMaximumLikekihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fitType
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^'Maximum likelihood fit'! !

!DhbMaximumLikekihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "

	result setCount: 1.
	count := dataHolder totalCount.
	super initializeIterations! !

!DhbMaximumLikekihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
normalization
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/12/00 "
	^count! !

!DhbMaximumLikekihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
normalizationError
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/12/00 "
	^countVariance sqrt! !

!DhbMaximumLikekihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
numberOfFreeParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^super numberOfParameters! !

!DhbMaximumLikekihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
numberOfParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^super numberOfParameters - 1! !

!DhbMaximumLikekihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
valueAndError: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/8/00 "

	| valueGradient gradient gVar |
	valueGradient := result valueAndGradient: aNumber.
	gradient := valueGradient last copyFrom: 1 to: valueGradient last size - 1.
	gVar := gradient * (self errorMatrix * gradient) / count.
	^Array with: valueGradient first
		with: ((valueGradient first / count) squared * countVariance + gVar) sqrt! !

Object subclass: #DhbLUPDecomposition
	instanceVariableNames: 'rows permutation parity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLUPDecomposition class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
direct: anArrayOfArrays
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	^self new initialize: anArrayOfArrays.! !

!DhbLUPDecomposition class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
equations: anArrayOfArrays
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
		^self new initialize: ( anArrayOfArrays collect: [ :each | each copy]).! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
backwardSubstitution: anArray
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| n sum answer|
	n := rows size.
	answer := DhbVector new: n.
	n to: 1 by: -1 do:
		[ :i |
		  sum := anArray at: i.
		  ( i + 1) to: n do: [ :j | sum := sum - ( ( ( rows at: i) at: j) * ( answer at: j))].
		  answer at: i put: sum / ( ( rows at: i) at: i).
		].
	^answer! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
decompose
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| n |
	n := rows size.
	permutation := (1 to: n) asArray.
	1 to: ( n - 1) do:
		[ :k |
		  self swapRow: k withRow: ( self largestPivotFrom: k);
			   pivotAt: k.
		].! !

!DhbLUPDecomposition methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
determinant
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/5/99 "
	| n |
	permutation isNil
		ifTrue: [ self protectedDecomposition].
	permutation = 0
		ifTrue: [ ^0].	"Singular matrix has 0 determinant"
	n := 0.
	^rows inject: parity
		  into: [ :det :each | n := n + 1. ( each at: n) * det]! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
forwardSubstitution: anArray
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| n sum answer|
	answer := permutation collect: [ :each | anArray at: each].
	n := rows size.
	2 to: n do:
		[ :i |
		  sum := answer at: i.
		  1 to: ( i - 1) do: [ :j | sum := sum - ( ( ( rows at: i) at: j) * ( answer at: j))].
		  answer at: i put: sum.
		].
	^answer! !

!DhbLUPDecomposition methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anArrayOfArrays
		"Private - A copy of the original array is made.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	rows := anArrayOfArrays.
	parity := 1.
	^self! !

!DhbLUPDecomposition methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inverseMatrixComponents
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| n inverseRows column |
	permutation isNil
		ifTrue: [ self protectedDecomposition].
	permutation = 0
		ifTrue: [ ^nil].	"Singular matrix has no inverse"
	n := rows size.
	inverseRows :=( 1 to: n) asVector collect: [ :j | DhbVector new: n].
	1 to: n do:
		[ :j |
		  column := self solve: ( ( Array new: rows size) atAllPut: 0; at: j put: 1; yourself).
		  1 to: n do: [ :i | ( inverseRows at: i) at: j put: ( column at: i)].
		].
	^inverseRows! !

!DhbLUPDecomposition methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
largestPivotFrom: anInteger
		"Private - Answers the largest pivot element in column anInteger, from position anInteger upward.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| valueOfMaximum indexOfMaximum value |
	valueOfMaximum := ( ( rows at: anInteger) at: anInteger) abs.
	indexOfMaximum := anInteger.
	( anInteger + 1) to: rows size do:
		[ :n |
		  value := ( ( rows at: n) at: anInteger) abs.
		  value > valueOfMaximum
				ifTrue: [ valueOfMaximum := value.
						  indexOfMaximum := n.
						].
		].
	^indexOfMaximum! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
pivotAt: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| inversePivot size k |
	inversePivot := 1 / ( ( rows at: anInteger) at: anInteger).
	size := rows size.
	k := anInteger + 1.
	k to: size
		do: [ :i |
			  ( rows at: i) at: anInteger put: (( rows at: i) at: anInteger) * inversePivot.
			  k to: size
				do: [ :j |
					  ( rows at: i) at: j put: ( ( rows at: i) at: j) - ( (( rows at: i) at: anInteger) * (( rows at: anInteger) at: j)).
					]
			].! !

!DhbLUPDecomposition methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| first delimitingString n k |
	n := rows size.
	first := true.
	rows do:
		[ :row |
		  first ifTrue: [ first := false]
				 ifFalse:[ aStream cr].
		  delimitingString := '('.
		  row do:
			[ :each |
				aStream nextPutAll: delimitingString.
				each printOn: aStream.
				delimitingString := ' '.
			].
		  aStream nextPut: $).
		].! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
protectedDecomposition
		"Private - If decomposition fails, set permutation to 0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	[ self decompose] on: Error do: [ :signal | permutation := 0. signal return: nil].! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
solve: anArrayOrVector
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	permutation isNil
		ifTrue: [ self protectedDecomposition].
	^permutation = 0
		ifTrue: [ nil]
		ifFalse:[ self backwardSubstitution: ( self forwardSubstitution: anArrayOrVector)]! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
swapRow: anInteger1 withRow: anInteger2
		"Private - Swap the rows indexed by the given integers.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	anInteger1 = anInteger2
		ifFalse:[ | swappedRow |
				  swappedRow := rows at: anInteger1.
				  rows at: anInteger1 put: ( rows at: anInteger2).
				  rows at: anInteger2 put: swappedRow.
				  swappedRow := permutation at: anInteger1.
				  permutation at: anInteger1 put: ( permutation at: anInteger2).
				  permutation at: anInteger2 put: swappedRow.
				  parity := parity negated.
				].! !

Object subclass: #DhbLagrangeInterpolator
	instanceVariableNames: 'pointCollection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLagrangeInterpolator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Create a new instance of the receiver without points. Points must be added with add:
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^super new initialize! !

!DhbLagrangeInterpolator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
points: aCollectionOfPoints
		"Create a new instance of the receiver with given points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self new initialize: aCollectionOfPoints! !

!DhbLagrangeInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
add: aPoint
		"Add a point to the collection of points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^pointCollection add: aPoint! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultSamplePoints
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^OrderedCollection new! !

!DhbLagrangeInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
		"Private - Create an empty point collection for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self initialize: self defaultSamplePoints! !

!DhbLagrangeInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aCollectionOfPoints
		"Private - Defines the collection of points for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	pointCollection := aCollectionOfPoints.
	^self! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
size
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 3/12/00 "
	^pointCollection size! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Compute the value of the Lagrange interpolation polynomial on the receiver's points at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| norm dx products answer size |
	norm := 1.
	size := pointCollection size.
	products := Array new: size.
	products atAllPut: 1.
	1 to: size
		do: [ :n |
			  dx := aNumber - ( self xPointAt: n).
			  dx = 0
				ifTrue: [ ^( self yPointAt: n)].
			  norm := norm * dx.
			  1 to: size
				do: [ :m |
					  m = n
						ifFalse:[ products at: m put: ( (( self xPointAt: m) - ( self xPointAt: n)) * ( products at: m))].
					].
			].
	answer := 0.
	1 to: size do:
		[ :n | answer := ( self yPointAt: n) / ( ( products at: n) * ( aNumber - ( self xPointAt: n))) + answer].
	^norm * answer! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
xPointAt: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^( pointCollection at: anInteger) x! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
yPointAt: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^( pointCollection at: anInteger) y! !

DhbLagrangeInterpolator subclass: #DhbNevilleInterpolator
	instanceVariableNames: 'leftErrors rightErrors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbNevilleInterpolator subclass: #DhbBulirschStoerInterpolator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbBulirschStoerInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeDifference: aNumber at: anInteger1 order: anInteger2
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 28/5/99 "
	| diff ratio |
	ratio := ( ( self xPointAt: anInteger1) - aNumber) * ( rightErrors at: anInteger1)
							/ ( (  self xPointAt: ( anInteger1 + anInteger2)) - aNumber).
	diff := ( ( leftErrors at: ( anInteger1 + 1)) - ( rightErrors at: anInteger1))
							/ ( ratio - ( leftErrors at: ( anInteger1 + 1))).
	rightErrors at: anInteger1 put: ( leftErrors at: ( anInteger1 + 1)) * diff.	
	leftErrors at: anInteger1 put: ratio * diff.! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeDifference: aNumber at: anInteger1 order: anInteger2
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 28/5/99 "
	| leftDist rightDist ratio |
	leftDist := ( self xPointAt: anInteger1) - aNumber.
	rightDist := (  self xPointAt: ( anInteger1 + anInteger2)) - aNumber.
	ratio := ( ( leftErrors at: ( anInteger1 + 1)) - ( rightErrors at: anInteger1)) / ( leftDist - rightDist).
	leftErrors at: anInteger1 put: ratio * leftDist.
	rightErrors at: anInteger1 put: ratio * rightDist.! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultSamplePoints
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^SortedCollection sortBlock: [ :a :b | a x < b x]! !

!DhbNevilleInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
	
	super initialize.
	leftErrors := Array new.
	rightErrors := Array new! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeDifferences: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 28/5/99 "
	| size nearestIndex dist minDist |
	size := pointCollection size.
	leftErrors size = size
		ifFalse:[ leftErrors := Array new: size.
				  rightErrors := Array new: size.
				].
	minDist := ( ( self xPointAt: 1) - aNumber) abs.
	nearestIndex := 1.
	leftErrors at: 1 put: ( self yPointAt: 1).
	rightErrors at: 1 put: leftErrors first.
	2 to: size do:
		[ :n |
		  dist := ( ( self xPointAt: n) - aNumber) abs.
		  dist < minDist
			ifTrue: [ dist = 0
						ifTrue: [ ^n negated].
					  nearestIndex := n.
					  minDist := dist.
					].
		 leftErrors at: n put: ( self yPointAt: n).
		 rightErrors at: n put: ( leftErrors at: n).
		].
	^nearestIndex! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Compute the value of the Lagrange interpolation polynomial on the receiver's points at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	^(self valueAndError: aNumber) first! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
valueAndError: aNumber
		"Compute and return the interpolated value of the interpolation Lagranage polynomial
		 and its estimated error.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	| size nearestIndex answer error |
	nearestIndex := self initializeDifferences: aNumber.
	nearestIndex < 0
		ifTrue: [ ^Array with: ( self yPointAt: nearestIndex negated) with: 0].
	answer := leftErrors at: nearestIndex.
	nearestIndex := nearestIndex - 1.
	size := pointCollection size.
	1 to: ( size - 1) do:
		[ :m |
		  1 to: ( size - m) do:
			[ :n | self computeDifference: aNumber at: n order: m].
		  size - m > ( 2 * nearestIndex)
				ifTrue: [ error := leftErrors at: ( nearestIndex + 1) ]
				ifFalse:[ error := rightErrors at: ( nearestIndex).
							  nearestIndex := nearestIndex - 1.
							].
		  answer := answer + error.
		].
	^Array with: answer with: error abs! !

DhbLagrangeInterpolator subclass: #DhbNewtonInterpolator
	instanceVariableNames: 'coefficients'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbNewtonInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
add: aPoint
		"Add a point to the collection of points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	self resetCoefficients.
	^super add: aPoint! !

!DhbNewtonInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeCoefficients
		"Private - Computes the coefficients for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| size k1 kn|
	size := pointCollection size.
	coefficients := ( 1 to: size) collect: [ :n | self yPointAt: n].
	1 to: (size - 1)
		do: [ :n |
			  size to: ( n + 1)  by: -1
				do: [ :k |
					  k1 := k - 1.
					  kn := k - n.
					  coefficients at: k put: ( (( coefficients at: k) - ( coefficients at: k1)) 
											/ ((self xPointAt: k) - (self xPointAt: kn))).
					].
			].! !

!DhbNewtonInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
resetCoefficients
		"Private - Reset the coefficients of the receiver to force a new computation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	coefficients := nil.! !

!DhbNewtonInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Compute the value of the Lagrange interpolation polynomial on the receiver's points at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| answer size |
	coefficients isNil
		ifTrue: [ self computeCoefficients].
	size := coefficients size.
	answer := coefficients at: size.
	(size - 1) to: 1 by: -1
		do: [ :n | answer := answer * ( aNumber - (self xPointAt:  n)) + ( coefficients at: n)].
	^answer! !

DhbNewtonInterpolator subclass: #DhbSplineInterpolator
	instanceVariableNames: 'startPointDerivative endPointDerivative'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbSplineInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeSecondDerivatives
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "

	| size u w s dx inv2dx |
	size := pointCollection size.
	coefficients := Array new: size.
	u := Array new: size - 1.
	startPointDerivative isNil 
		ifTrue: 
			[coefficients at: 1 put: 0.
			u at: 1 put: 0]
		ifFalse: 
			[coefficients at: 1 put: -1 / 2.
			s := 1 / (( self xPointAt: 2) x - ( self xPointAt: 1) x).
			u at: 1
				put: 3 * s 
						* (s * (( self yPointAt: size) - ( self yPointAt: size - 1)) 
								- startPointDerivative)].
	2 to: size - 1
		do: 
			[:n | 
			dx := (self xPointAt: n) - (self xPointAt: ( n - 1)).
			inv2dx := 1 / (( self xPointAt: n + 1) - (self xPointAt: n - 1)).
			s := dx * inv2dx.
			w := 1 / (s * (coefficients at: n - 1) + 2).
			coefficients at: n put: (s - 1) * w.
			u at: n
				put: (((( self yPointAt: n + 1) - ( self yPointAt: n)) 
						/ (( self xPointAt: n + 1) - ( self xPointAt: n)) 
							- ((( self yPointAt: n) - ( self yPointAt: n - 1)) / dx)) * 6 
						* inv2dx - ((u at: n - 1) * s)) 
						* w].
	endPointDerivative isNil 
		ifTrue: [coefficients at: size put: 0]
		ifFalse: 
			[w := 1 / 2.
			s := 1 / ((self xPointAt:  size) - (self xPointAt: ( size - 1))).
			u at: 1
				put: 3 * s * (endPointDerivative 
								- (s * (self yPointAt: size) - (self yPointAt: size - 1))).
			coefficients at: size
				put: s - (w * (u at: size - 1) / ((coefficients at: size - 1) * w + 1))].
	size - 1 to: 1
		by: -1
		do: 
			[:n | 
			coefficients at: n
				put: (coefficients at: n) * (coefficients at: n + 1) + (u at: n)]! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultSamplePoints
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^SortedCollection sortBlock: [ :a :b | a x < b x]! !

!DhbSplineInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
endPointDerivative: aNumber
		"Defines the end point derivatives.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	endPointDerivative := aNumber.
	self resetCoefficients.! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
resetEndPointDerivatives
		"Set the end point derivatives to undefined.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	self setEndPointDerivatives: ( Array new: 2).! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setEndPointDerivatives: anArray
		"Defines the end point derivatives.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	startPointDerivative := anArray at: 1.
	endPointDerivative := anArray at: 2.
	self resetCoefficients.! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
startPointDerivative: aNumber
		"Defines the end point derivatives.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	startPointDerivative := aNumber.
	self resetCoefficients.! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber 
	"Computes the value of a cubic spline interpolation over the points of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "

	| n1 n2 n step a b |
	coefficients isNil ifTrue: [self computeSecondDerivatives].
	n2 := pointCollection size.
	n1 := 1.
	[n2 - n1 > 1] whileTrue: 
			[n := (n1 + n2) // 2.
			(self xPointAt:  n) > aNumber ifTrue: [n2 := n] ifFalse: [n1 := n]].
	step := (self xPointAt: n2) - (self xPointAt: n1).
	a := ((self xPointAt: n2) - aNumber) / step.
	b := (aNumber - (self xPointAt: n1)) / step.
	^a * (self yPointAt: n1) + (b * (self yPointAt: n2)) 
		+ ((a * (a squared - 1) * (coefficients at: n1) 
				+ (b * (b squared - 1) * (coefficients at: n2))) * step squared 
				/ 6)! !

Object subclass: #DhbLanczosFormula
	instanceVariableNames: 'coefficients sqrt2Pi'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLanczosFormula class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Answer a unique instance. Create it if it does not exist.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new.
					 UniqueInstance initialize.
					].
	^UniqueInstance! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
gamma: aNumber
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self leadingFactor: aNumber) exp * ( self series: aNumber) * sqrt2Pi / aNumber! !

!DhbLanczosFormula methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
		"Private - Initialize the coefficients of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	sqrt2Pi := ( Float pi * 2) sqrt.
	coefficients := #( 76.18009172947146 -86.50532032941677 24.01409824083091 -1.231739572450155 0.1208650973866179e-2 -0.5395239384953e-5).
	^self! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
leadingFactor: aNumber
		"Private - Answers the log of the leading factor in Lanczos' formula.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	| temp |
	temp := aNumber + 5.5.
	^( temp ln * ( aNumber + 0.5) - temp)! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
logGamma: aNumber
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self leadingFactor: aNumber) + ( ( self series: aNumber) * sqrt2Pi / aNumber) ln! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
series: aNumber
		"Private - Answer the value of the series of Lanczos' formula.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	| term |
	term := aNumber.
	^coefficients inject: 1.000000000190015
						into: [ :sum :each | term := term + 1. each / term + sum]! !

Object subclass: #DhbLinearEquationSystem
	instanceVariableNames: 'rows solutions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLinearEquationSystem class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
equations: anArrayOfArrays constant: anArray 
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^self new initialize: anArrayOfArrays constants: (Array with: anArray)! !

!DhbLinearEquationSystem class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
equations: anArrayOfArrays constants: anArrayOfConstantArrays 
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^self new initialize: anArrayOfArrays constants: anArrayOfConstantArrays! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
backSubstitutionAt: anInteger
		"Private - Perform the back-substitution step corresponding to the anInteger-th constant array.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| size answer accumulator |
	size := rows size.
	answer := Array new: size.
	size to: 1 by: -1 do:
		[ :n |
		  accumulator := (rows at: n) at: (anInteger + size).
		  ( n + 1) to: size
			do: [ :m | accumulator := accumulator - ( ( answer at: m) * ( ( rows at: n) at: m))].
		  answer at: n put: ( accumulator / ( ( rows at: n) at: n)).
		].
	solutions at: anInteger put: answer.! !

!DhbLinearEquationSystem methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anArrayOfArrays constants: anArrayOfConstantArrays
		"Private - Initialize the receiver with system's matrix in anArrayOfArrays and several constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	rows := anArrayOfArrays collect: [ :each | n := n + 1. each, ( anArrayOfConstantArrays collect: [ :c | c at: n])].
	^self! !

!DhbLinearEquationSystem methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
largestPivotFrom: anInteger
		"Private - Answers the largest pivot element in column anInteger, from position anInteger upward.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| valueOfMaximum indexOfMaximum |
	valueOfMaximum := ( rows at: anInteger) at: anInteger.
	indexOfMaximum := anInteger.
	( anInteger + 2) to: rows size do:
		[ :n |
		  ( ( rows at: n) at: anInteger) > valueOfMaximum
				ifTrue: [ valueOfMaximum := ( rows at: n) at: anInteger.
							 indexOfMaximum := n.
						   ].
		].
	^indexOfMaximum! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
pivotAt: anInteger
		"Private - Performs pivot operation with pivot element at anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| inversePivot rowPivotValue row pivotRow |
	pivotRow := rows at: anInteger.
	inversePivot := 1 / ( pivotRow at: anInteger).
	( anInteger + 1) to: rows size do:
		[ :n |
		  row := rows at: n.
		  rowPivotValue := ( row at: anInteger) * inversePivot.
		  anInteger to: row size do:
			[ :m |
			  row at: m put: ( ( row at: m) - (( pivotRow at: m) * rowPivotValue)).
			].
		].! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
pivotStepAt: anInteger
		"Private - Performs an optimum pivot operation at anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	self swapRow: anInteger withRow: ( self largestPivotFrom: anInteger);
		   pivotAt: anInteger.! !

!DhbLinearEquationSystem methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"Append to the argument aStream, a sequence of characters that describes the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| first delimitingString n k |
	n := rows size.
	first := true.
	rows do:
		[ :row |
		  first ifTrue: [ first := false]
				 ifFalse:[ aStream cr].
		  delimitingString := '('.
		  k := 0.
		  row do:
			[ :each |
				aStream nextPutAll: delimitingString.
				each printOn: aStream.
				k := k + 1.
				delimitingString := k < n ifTrue: [ ' '] ifFalse: [ ' : '].
			].
		  aStream nextPut: $).
		].! !

!DhbLinearEquationSystem methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
solution
		"Answers the solution corresponding to the first constant array.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	^self solutionAt: 1! !

!DhbLinearEquationSystem methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
solutionAt: anInteger
		"Answer the solution corresponding to the anInteger-th constant array.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	"solutions isNil
		ifTrue: [ [self solve] when: Error do: [ :signal |solutions := 0. signal return: nil.] ].
	solutions = 0
		ifTrue: [ ^nil].
	( solutions at: anInteger) isNil
		ifTrue: [ self backSubstitutionAt: anInteger].
	^solutions at: anInteger"

	solutions isNil
		ifTrue: [ [self solve] on: Error do: [ :signal |solutions := 0. signal return: nil.] ].
	solutions = 0
		ifTrue: [ ^nil].
	( solutions at: anInteger) isNil
		ifTrue: [ self backSubstitutionAt: anInteger].
	^solutions at: anInteger! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
solve
		"Private - Perform LU decomposition of the system.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	1 to: rows size do: [ :n | self pivotStepAt: n].
	solutions := Array new: ( (rows at: 1) size - rows size).! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
swapRow: anInteger1 withRow: anInteger2
		"Private - Swap the rows indexed by the given integers.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| swappedRow |
	anInteger1 = anInteger2
		ifFalse:[ swappedRow := rows at: anInteger1.
					  rows at: anInteger1 put: ( rows at: anInteger2).
					  rows at: anInteger2 put: swappedRow.
					].! !

Object subclass: #DhbLinearRegression
	instanceVariableNames: 'sum1 sumX sumY sumXX sumYY sumXY slope intercept correlationCoefficient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLinearRegression class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Create a new instance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	^( super new) reset; yourself! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
add: aPoint
		"Accumulate aPoint into of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	self add: aPoint weight: 1.! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
add: aPoint weight: aNumber 
	"Accumulate aPoint into of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	sum1 := sum1 + aNumber.
	sumX := sumX + (aPoint x * aNumber).
	sumY := sumY + (aPoint y * aNumber).
	sumXX := sumXX + (aPoint x squared * aNumber).
	sumYY := sumYY + (aPoint y squared * aNumber).
	sumXY := sumXY + (aPoint x * aPoint y * aNumber).
	self resetResults! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asEstimatedPolynomial
		"Answer the resulting linear dependence found by the receiver in the form of a polynomial
		 with embedded error matrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^( DhbEstimatedPolynomial coefficients: self coefficients)
			errorMatrix: self errorMatrix;
			yourself! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asPolynomial
		"Answer the resulting linear dependence found by the receiver in the form of a polynomial.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	^DhbPolynomial coefficients: self coefficients! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
coefficients
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^Array with: self intercept with: self slope! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeResults
	"Private - Compute the results of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	| xNorm xyNorm |
	xNorm := sumXX * sum1 - (sumX * sumX).
	xyNorm := sumXY * sum1 - (sumX * sumY).
	slope := xyNorm / xNorm.
	intercept := (sumXX * sumY - (sumXY * sumX)) / xNorm.
	correlationCoefficient := xyNorm 
				/ (xNorm * (sumYY * sum1 - (sumY * sumY))) sqrt! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
correlationCoefficient
		"Answers the correlation coefficient of the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	correlationCoefficient isNil
		ifTrue: [ self computeResults].
	^correlationCoefficient! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
errorMatrix
	"Answer the resulting linear dependence found by the receiver in the form of a polynomial
		 with embedded error matrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "

	| c1 cx cxx |
	c1 := 1.0 / (sumXX * sum1 - sumX squared).
	cx := sumX negated * c1.
	cxx := sumXX * c1.
	c1 := sum1 * c1.
	^DhbSymmetricMatrix rows: (Array with: (Array with: cxx with: cx)
				with: (Array with: cx with: c1))! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
errorOnIntercept
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/5/99 "

	^(sumXX / (sumXX * sum1 - sumX squared)) sqrt! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
errorOnSlope
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/5/99 "

	^(sum1 / (sumXX * sum1 - sumX squared)) sqrt! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
intercept
		"Answers the intercept of the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	intercept isNil
		ifTrue: [ self computeResults].
	^intercept! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
remove: aPoint 
	"Remove aPoint which was accumulated into of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	sum1 := sum1 - 1.
	sumX := sumX - aPoint x.
	sumY := sumY - aPoint y.
	sumXX := sumXX - aPoint x squared.
	sumYY := sumYY - aPoint y squared.
	sumXY := sumXY - (aPoint x * aPoint y).
	self resetResults! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset
	"Set all accumulators of the receiver to zero and reset its results.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	sum1 := 0.
	sumX := 0.
	sumY := 0.
	sumXX := 0.
	sumYY := 0.
	sumXY := 0.
	self resetResults! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
resetResults
		"Private - Reset the results of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	slope := nil.
	intercept := nil.
	correlationCoefficient := nil.! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
slope
		"Answers the slope of the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	slope isNil
		ifTrue: [ self computeResults].
	^slope! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answer the value interpolated at aNumber by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	^aNumber * self slope + self intercept! !

Object subclass: #DhbMahalanobisCenter
	instanceVariableNames: 'center inverseCovariance accumulator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMahalanobisCenter class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self new initialize: anInteger! !

!DhbMahalanobisCenter class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
onVector: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self new center: aVector! !

!DhbMahalanobisCenter methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator accumulate: aVector.! !

!DhbMahalanobisCenter methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
center: aVector
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator := DhbCovarianceAccumulator new: aVector size.
	center := aVector.
	inverseCovariance := DhbSymmetricMatrix identity: aVector size.
	^self! !

!DhbMahalanobisCenter methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeParameters
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	center := accumulator average copy.
	inverseCovariance := accumulator covarianceMatrix inverse.! !

!DhbMahalanobisCenter methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
count
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator count! !

!DhbMahalanobisCenter methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	| delta |
	delta := aVector - center.
	^delta * inverseCovariance * delta! !

!DhbMahalanobisCenter methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator := DhbCovarianceAccumulator new: anInteger.
	^self! !

!DhbMahalanobisCenter methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator count printOn: aStream.
	aStream nextPutAll: ': '.
	center printOn: aStream.! !

!DhbMahalanobisCenter methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset
		"Leave center and inverse covariant matrix untouched
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator reset.! !

Object subclass: #DhbMatrix
	instanceVariableNames: 'rows lupDecomposition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
join: anArrayOfMatrices
		"Inverse of the split operation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/5/99 "
	| rows n row rowSize n1 n2 |
	rows := OrderedCollection new.
	n1 := ( anArrayOfMatrices at: 1) numberOfColumns.
	n2 := n1 + 1.
	rowSize := n1 + ( anArrayOfMatrices at: 2) numberOfColumns.
	n := 0.
	( anArrayOfMatrices at: 1) rowsDo:
		[ :each |
		  n := n + 1.
		  row := DhbVector new: rowSize.
		  row replaceFrom: 1 to: n1 with: each startingAt: 1;
			  replaceFrom: n2 to: rowSize with: ( ( anArrayOfMatrices at: 2) rowAt: n) startingAt: 1.
		  rows add: row.
		].
	n := 0.
	( anArrayOfMatrices at: 3) rowsDo:
		[ :each |
		  n := n + 1.
		  row := DhbVector new: rowSize.
		  row replaceFrom: 1 to: n1 with: each startingAt: 1;
			  replaceFrom: n2 to: rowSize with: ( ( anArrayOfMatrices at: 4) rowAt: n) startingAt: 1.
		  rows add: row.
		].
	^self rows: rows! !

!DhbMatrix class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
lupCRLCriticalDimension
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/5/99 "
	^40! !

!DhbMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: anInteger
		"Create an empty square matrix of dimension anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self new initialize: anInteger! !

!DhbMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
rows: anArrayOrVector
		"Create a new matrix with given components.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self new initializeRows: anArrayOrVector! !

!DhbMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/11/2012 13:55'!
rows: rowsInteger columns: columnsInteger

	^ self new initializeRows: rowsInteger columns: columnsInteger! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
* aNumberOrMatrixOrVector
		"Answers the product of the receiver with the argument.
		 The argument can be a number, matrix or vector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aNumberOrMatrixOrVector productWithMatrix: self! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
+ aMatrix
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix addWithRegularMatrix: self! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
- aMatrix
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix subtractWithRegularMatrix: self! !

!DhbMatrix methodsFor: 'comparing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
= aNumberOrMatrix 
	aNumberOrMatrix isNil ifTrue: [^false].
	aNumberOrMatrix isNumber ifTrue: [^(self numberOfRows = 1 and: [self numberOfColumns = 1])
			and: [(self rowAt: 1 columnAt: 1)
					= aNumberOrMatrix]].
	aNumberOrMatrix class = self class ifFalse: [^false].
	^self rows = aNumberOrMatrix rows! !

!DhbMatrix methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aMatrix
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	self rowsCollect: [ :each | n := n + 1. each accumulate: ( aMatrix rowAt: n)]! !

!DhbMatrix methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulateNegated: aMatrix
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	self rowsCollect: [ :each | n := n + 1. each accumulateNegated: ( aMatrix rowAt: n)]! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
addWithMatrix: aMatrix class: aMatrixClass
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	^aMatrixClass rows: ( self rowsCollect: [ :each | n := n + 1. each + ( aMatrix rowAt: n)])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
addWithRegularMatrix: aMatrix
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix addWithMatrix: self class: aMatrix class! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
addWithSymmetricMatrix: aMatrix
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/28/99 "
	^aMatrix addWithMatrix: self class: self class! !

!DhbMatrix methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asSymmetricMatrix
		"Convert the receiver to a symmetric matrix (no check is made).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^DhbSymmetricMatrix rows: rows! !

!DhbMatrix methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
at: aRowIndex at: aColumnIndex
		"Answers the aRowIndex-th, aColumnIndex-th entry in the receiver.
		 (c) Copyrights Joseph WHITESELL, 2001, all rights reserved.
		 Initial code: 08/17/2001 "
	^ self rowAt: aRowIndex columnAt: aColumnIndex! !

!DhbMatrix methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
at: rowIndex at: columnIndex put: value

	self rowAt: rowIndex columnAt: columnIndex put: value
	
! !

!DhbMatrix methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
atColumn: anInteger

	^ self columnAt: anInteger! !

!DhbMatrix methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
atColumn: aColumnIndex put: aCollection

	aCollection withIndexDo: [: value : rowIndex |
		self rowAt: rowIndex columnAt: aColumnIndex put: value ]
	
! !

!DhbMatrix methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
atColumn: aColumnNumber put: aCollection startingAt: rowNumber
	" Fill the receiver with aCollection at aColumnNumber beggining
	at rowNumber. "
	
	aCollection withIndexDo: [: value : rowIndex |
		self rowAt: rowIndex + rowNumber
			columnAt: aColumnNumber 
			put: value ]
	
! !

!DhbMatrix methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
atRow: rowIndex put: aCollection

	aCollection withIndexDo: [: value : columnIndex |
		self rowAt: rowIndex columnAt: columnIndex put: value ]
	
! !

!DhbMatrix methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
atRow: rowIndex put: aCollection startingAt: startColumnNumber 
	" Fill the receiver with aCollection at rowIndex beggining at startColumnNumber. "
		
	aCollection withIndexDo: [: value : columnIndex |
		self 
			rowAt: rowIndex 
			columnAt: columnIndex + startColumnNumber
			put: value ]
	
! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
columnAt: anInteger
		"Answers the anInteger-th column of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows collect: [ :each | each at: anInteger]! !

!DhbMatrix methodsFor: 'iterators' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
columnsCollect: aBlock
		"Perform the collect: operation on the rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	^rows last collect: [ :each | n := n + 1. aBlock value: ( self columnAt: n)]! !

!DhbMatrix methodsFor: 'iterators' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
columnsDo: aBlock
		"Perform the collect: operation on the rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	^rows last do: [ :each | n := n + 1. aBlock value: ( self columnAt: n)]! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
determinant
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/5/99 "
	^self lupDecomposition determinant! !

!DhbMatrix methodsFor: 'comparing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
hash
	^rows hash! !

!DhbMatrix methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anInteger
		"Build empty components for a square matrix.
		  No check is made: components are assumed to be orgainized in rows.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	rows := ( 1 to: anInteger) asVector collect: [ :each | DhbVector new: anInteger].! !

!DhbMatrix methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeRows: anArrayOrVector
		"Defines the components of the recevier.
		  No check is made: components are assumed to be orgainized in rows.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	rows := anArrayOrVector asVector collect: [ :each | each asVector].! !

!DhbMatrix methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeRows: rowsInteger columns: columnsInteger
	"Build empty components for a matrix."
		
	rows := ( 1 to: rowsInteger ) asVector collect: [ :each | DhbVector new: columnsInteger ].! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inverse
	"Answer the inverse of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^self isSquare 
		ifTrue: [self lupInverse]
		ifFalse: [self squared inverse * self transpose]! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inversePureCRL
		"Answer the inverse of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/5/99 "
	^self squared inversePureCRL * self transpose! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isSquare
		"Answers true if the number of rows is equal to the number of columns.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows size = rows last size! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isSymmetric
		"Answers false because the receiver is not a symmetric matrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^false! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
largestPowerOf2SmallerThan: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved
		 Initial code: 21/3/99 "
	| m m2|
	m := 2.
	[ m2 := m * 2.
	  m2 < anInteger] whileTrue:[ m := m2].
	^m! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
lupDecomposition
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	lupDecomposition isNil
		ifTrue: [ lupDecomposition :=DhbLUPDecomposition equations: rows].
	^lupDecomposition! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
lupInverse
	^self class rows: self lupDecomposition inverseMatrixComponents! !

!DhbMatrix methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
negate
		"Inverse the sign of all components of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	rows do: [ :each |each negate].! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
numberOfColumns
		"Answer the number of rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows last size! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
numberOfRows
		"Answer the number of rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows size! !

!DhbMatrix methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"Append to the argument aStream, a sequence of characters that describes the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| first |
	first := true.
	rows do: 
		[ :each |
		  first ifTrue: [ first := false]
				 ifFalse:[ aStream cr].
		  each printOn: aStream.
		].! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromDouble: aDouble 
	^self * aDouble! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromFixedPoint: aFixedPoint 
	^self * aFixedPoint! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromFraction: aFraction 
	^self * aFraction! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromInteger: anInteger 
	^self * anInteger! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productWithMatrix: aMatrix
		"Answers the product of aMatrix with the receiver (in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self productWithMatrixFinal: aMatrix! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productWithMatrixFinal: aMatrix
		"Answers the product of aMatrix with the receiver (in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self class rows: ( aMatrix rowsCollect: [ :row | self columnsCollect: [ :col | row * col]])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productWithSymmetricMatrix: aSymmetricMatrix
		"Answers the product of the receiver with aSymmetricMatrix (in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self class rows: ( self rowsCollect: [ :row | aSymmetricMatrix columnsCollect: [ :col | row * col]])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productWithTransposeMatrix: aMatrix
		"Answers the product of the receiver with the transpose of aMatrix(in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self class rows: ( self rowsCollect: [ :row | aMatrix rowsCollect: [ :col | row * col]])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productWithVector: aVector
		"Answers the product of the receiver with aVector
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self columnsCollect: [ :each | each * aVector]! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
rowAt: anInteger
		"Answers the anInteger-th row of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows at: anInteger! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
rowAt: aRowIndex columnAt: aColumnIndex
		"Answers the aRowIndex-th, aColumnIndex-th entry in the receiver.
		 (c) Copyrights Joseph WHITESELL, 2001, all rights reserved.
		 Initial code: 08/17/2001 "
	^(rows at: aRowIndex) at: aColumnIndex! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
rowAt: aRowIndex columnAt: aColumnIndex put: aValue
	
	^(rows at: aRowIndex) at: aColumnIndex put: aValue! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
rows
	^rows! !

!DhbMatrix methodsFor: 'iterators' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
rowsCollect: aBlock
		"Perform the collect: operation on the rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows collect: aBlock! !

!DhbMatrix methodsFor: 'iterators' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
rowsDo: aBlock
		"Perform the collect: operation on the rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows do: aBlock! !

!DhbMatrix methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
scaleBy: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/11/00 "
	rows do: [ :each | each scaleBy: aNumber].! !

!DhbMatrix methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
split
		"Private - Answers an array of 4 matrices split from the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/5/99 "
	| n m n1 m1 |
	n := self numberOfRows.
	m := self numberOfColumns.
	n1 := self largestPowerOf2SmallerThan: n.
	m1 := self largestPowerOf2SmallerThan: m.
	^Array
		with: ( self class rows: ( ( 1 to: n1) asVector collect: [ :k | ( rows at: k) copyFrom: 1 to: m1]))
		with:( self class rows: ( ( 1 to: n1) asVector collect: [ :k | ( rows at: k) copyFrom: (m1 + 1) to: m]))
		with: ( self class rows: ( ( (n1 + 1) to: n) asVector collect: [ :k | ( rows at: k) copyFrom: 1 to: m1]))
		with:( self class rows: ( ( (n1 + 1) to: n) asVector collect: [ :k | ( rows at: k) copyFrom: (m1 + 1) to: m]))! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
squared
		"Answers the product of the transpose of the receiver with the receiver (in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^DhbSymmetricMatrix rows: ( self columnsCollect: [ :col | self columnsCollect: [ :colT | col * colT]])! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
strassenProductWithMatrix: aMatrix
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/5/99 "
	| matrixSplit selfSplit p1 p2 p3 p4 p5 p6 p7 |
	( self numberOfRows > 2 and: [ self numberOfColumns > 2])
		ifFalse:[ ^self class rows: ( aMatrix rowsCollect: [ :row | self columnsCollect: [ :col | row * col]])].
	selfSplit := self split.
	matrixSplit := aMatrix split.
	p1 := ( ( selfSplit at: 2) - ( selfSplit at: 4)) strassenProductWithMatrix: ( matrixSplit at: 1).
	p2 := ( selfSplit at: 4) strassenProductWithMatrix: ( ( matrixSplit at: 1) + ( matrixSplit at: 2)).
	p3 := ( selfSplit at: 1) strassenProductWithMatrix: ( ( matrixSplit at: 3) + ( matrixSplit at: 4)).
	p4 := ( ( selfSplit at: 3) - ( selfSplit at: 1)) strassenProductWithMatrix: ( matrixSplit at: 4).
	p5 := ( ( selfSplit at: 1) + ( selfSplit at: 4)) strassenProductWithMatrix: ( ( matrixSplit at: 1) + ( matrixSplit at: 4)).
	p6 := ( ( selfSplit at: 3) + ( selfSplit at: 4)) strassenProductWithMatrix: ( ( matrixSplit at: 2) - ( matrixSplit at: 4)).
	p7 := ( ( selfSplit at: 1) + ( selfSplit at: 2)) strassenProductWithMatrix: ( ( matrixSplit at: 1) - ( matrixSplit at: 3)).
	^self class join: ( Array
							with: ( p5 + p4 + p6 - p2)
							with: (p1 + p2)
							with: ( p3 + p4)
							with: ( p5 + p1 - p3 - p7)
							)! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
subtractWithMatrix: aMatrix class: aMatrixClass
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	^aMatrixClass rows: ( self rowsCollect: [ :each | n := n + 1. each - ( aMatrix rowAt: n)])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
subtractWithRegularMatrix: aMatrix
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix subtractWithMatrix: self class: aMatrix class! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
subtractWithSymmetricMatrix: aMatrix
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/28/99 "
	^aMatrix subtractWithMatrix: self class: self class! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
transpose
		"Answer a new matrix, transpose of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self class rows: ( self columnsCollect: [ :each | each])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
transposeProductWithMatrix: aMatrix
		"Answers the product of the transpose of the receiver with aMatrix (in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self class rows: ( self columnsCollect: [ :row | aMatrix columnsCollect: [ :col | row * col]])! !

DhbMatrix subclass: #DhbSymmetricMatrix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbSymmetricMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
identity: anInteger
		"Create an identity matrix of dimension anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self new initializeIdentity: anInteger! !

!DhbSymmetricMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
join: anArrayOfMatrices
		"Inverse of the split operation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| rows n |
	rows := OrderedCollection new.
	n := 0.
	( anArrayOfMatrices at: 1) rowsDo:
		[ :each |
		  n := n + 1.
		  rows add: each, ( ( anArrayOfMatrices at: 3) columnAt: n).
		].
	n := 0.
	( anArrayOfMatrices at: 2) rowsDo:
		[ :each |
		  n := n + 1.
		  rows add: ( ( anArrayOfMatrices at: 3) rowAt: n), each.
		].
	^self rows: rows! !

!DhbSymmetricMatrix class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
lupCRLCriticalDimension
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/5/99 "
	^36! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
+ aMatrix
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix addWithSymmetricMatrix: self! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
- aMatrix
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix subtractWithSymmetricMatrix: self! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
addWithSymmetricMatrix: aMatrix
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix addWithMatrix: self class: self class! !

!DhbSymmetricMatrix methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
clear
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/11/00 "
	rows do: [ :each | each atAllPut: 0].! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
crlInverse
	| matrices b1 cb1ct cb1 |
	matrices := self split.
	b1 := (matrices at: 1) inverse.
	cb1 := (matrices at: 3) * b1.
	cb1ct := (cb1 productWithTransposeMatrix: (matrices at: 3)) 
				asSymmetricMatrix.
	matrices at: 3 put: (matrices at: 2) * cb1.
	matrices at: 2 put: ((matrices at: 2) accumulateNegated: cb1ct) inverse.
	matrices at: 1 put: ( b1 accumulate: (cb1 transposeProductWithMatrix: (matrices at: 3))).
	(matrices at: 3) negate.
	^self class join: matrices! !

!DhbSymmetricMatrix methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeIdentity: anInteger
		"Build components for an identity matrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	rows := ( 1 to: anInteger) asVector collect: [ :n | (DhbVector new: anInteger) atAllPut: 0; at: n put: 1; yourself].! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inverse
	"Answer the inverse of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^(rows size < self class lupCRLCriticalDimension or: [lupDecomposition notNil]) 
			ifTrue: [self lupInverse]
			ifFalse: [self crlInverse]! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inverse1By1
		"Private - Answer the inverse of the receiver when it is a 1x1 matrix (no check is made).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/5/99 "
	^self class rows: ( DhbVector with: ( DhbVector with: ( 1/ ((rows at: 1) at: 1))))! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inverse2By2
		"Private - Answer the inverse of the receiver when it is a 2x2 matrix (no check is made).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/5/99 "
	| line1 line2 |
	line1 := DhbVector with: ((rows at: 2) at: 2) with: ((rows at: 1) at: 2) negated.
	line2 := DhbVector with: ((rows at: 1) at: 2) negated with: ((rows at: 1) at: 1).
	^self class rows: ( DhbVector with: line1 with: line2) * (1 / (((rows at: 1) at: 1) * ((rows at: 2) at: 2) - ((rows at: 1) at: 2) squared))! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inversePureCRL
		"Answer the inverse of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| matrices b1 cb1 cb1ct |
	rows size = 1
		ifTrue: [ ^self inverse1By1].
	rows size = 2
		ifTrue: [ ^self inverse2By2].
	matrices := self split.
	b1 := ( matrices at: 1) inversePureCRL.
	cb1 := ( matrices at: 3) * b1.
	cb1ct := ( cb1 productWithTransposeMatrix: ( matrices at: 3)) asSymmetricMatrix.
	matrices at: 2 put: ( ( matrices at: 2) accumulateNegated: cb1ct) inversePureCRL.
	matrices at: 3 put: ( matrices at: 2) * cb1.
	matrices at: 1 put: ( b1 accumulate: ( cb1 transposeProductWithMatrix: ( matrices at: 3))).
	( matrices at: 3) negate.
	^self class join: matrices! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inversePureLUP
		"Answer the inverse of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	lupDecomposition := nil.
	^self class rows: lupDecomposition inverseMatrixComponents! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isSquare
		"Answers true because a symmetric matrix is square.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^true! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isSymmetric
		"Answers true because the receiver is a symmetric matrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^true! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productWithMatrix: aMatrix
		"Answers the product of aMatrix with the receiver (in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix productWithSymmetricMatrix: self! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productWithSymmetricMatrix: aSymmetricMatrix
		"Answers the product of aMatrix with the receiver (in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aSymmetricMatrix productWithMatrixFinal: self! !

!DhbSymmetricMatrix methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
split
	"Private - 
	Answers an array of 3 matrices split from the receiver. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 11/2/99"

	| n |
	n := self largestPowerOf2SmallerThan: rows size.
	^Array
		with: (self class rows: ((1 to: n) asVector collect: [:k | (rows at: k)
						copyFrom: 1 to: n]))
		with: (self class rows: ((n + 1 to: rows size) asVector collect: [:k | (rows at: k)
						copyFrom: n + 1 to: rows size]))
		with: (self class superclass rows: ((n + 1 to: rows size) asVector collect: [:k | (rows at: k)
						copyFrom: 1 to: n]))! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
subtractWithSymmetricMatrix: aMatrix
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix subtractWithMatrix: self class: self class! !

Object subclass: #DhbMinimizingPoint
	instanceVariableNames: 'value position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbMinimizingPoint subclass: #DhbMaximizingPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMaximizingPoint methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
betterThan: anOptimizingPoint
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^value > anOptimizingPoint value! !

!DhbMinimizingPoint class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: aVector value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^self new vector: aVector; value: aNumber; yourself! !

!DhbMinimizingPoint class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
vector: aVector function: aFunction 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "

	^self new: aVector value: (aFunction value: aVector)! !

!DhbMinimizingPoint methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
betterThan: anOptimizingPoint
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^value < anOptimizingPoint value! !

!DhbMinimizingPoint methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
position
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "

	^position! !

!DhbMinimizingPoint methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 05-Jan-00 "

	position printOn: aStream.
	aStream
		nextPut: $:;
		space.
	value printOn: aStream! !

!DhbMinimizingPoint methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^value! !

!DhbMinimizingPoint methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	value := aNumber.! !

!DhbMinimizingPoint methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
vector: aVector 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "

	position := aVector! !

Object subclass: #DhbMitchellMooreGenerator
	instanceVariableNames: 'randoms lowIndex highIndex'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
constants: anArray lowIndex: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "
	^super new initialize: anArray lowIndex: anInteger! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
default
	"Private- 
	 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "

	| congruentialGenerator |
	congruentialGenerator := DhbCongruentialRandomNumberGenerator new.
	^self generateSeeds: congruentialGenerator! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
generateSeeds: congruentialGenerator
	"Private- "
	^self constants: ((1 to: 55) collect: [:n | congruentialGenerator floatValue])
		lowIndex: 24! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := self default].
	^UniqueInstance! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset: anInteger
	"Reset the unique instance used for the default series"
	UniqueInstance := self seed: anInteger.! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
seed: anInteger
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/18/00 "

	| congruentialGenerator |
	congruentialGenerator := DhbCongruentialRandomNumberGenerator seed: anInteger.
	^self generateSeeds: congruentialGenerator! !

!DhbMitchellMooreGenerator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
floatValue
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "

	| x |
	x := (randoms at: lowIndex) + (randoms at: highIndex).
	x < 1.0 ifFalse: [x := x - 1.0].
	randoms at: highIndex put: x.
	highIndex := highIndex + 1.
	highIndex > randoms size ifTrue: [highIndex := 1].
	lowIndex := lowIndex + 1.
	lowIndex > randoms size ifTrue: [lowIndex := 1].
	^x! !

!DhbMitchellMooreGenerator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anArray lowIndex: anInteger
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "

	randoms := anArray.
	lowIndex := anInteger.
	highIndex := randoms size.
	^self! !

!DhbMitchellMooreGenerator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
integerValue: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "
	^( self floatValue * anInteger) truncated! !

Object subclass: #DhbPolynomial
	instanceVariableNames: 'coefficients'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbPolynomial commentStamp: '<historical>' prior: 0!
A DhbPolynomial represents a single variable polynomial function as an array of coefficients.

The constructor coefficients: anArray expects coefficients in increasing degree, that is constant term first. So, for example, to represent X^2 + 2X, we want to create:

DhbPolynomial coefficients: #(0 2 1)

The printOn: method similarly displays the polynomial from lowest nonzero degree to highest. 

DhbPolynomial coefficients: #(0 2 1)   2 X +  X^2 

!

DhbPolynomial subclass: #DhbEstimatedPolynomial
	instanceVariableNames: 'errorMatrix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbEstimatedPolynomial methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
error: aNumber
		"Compute the error on the value of the receiver for argument aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| errorVector term nextTerm |
	nextTerm := 1.
	errorVector := ( coefficients collect: [ :each | term := nextTerm. nextTerm := aNumber * nextTerm. term]) asVector.
	^( errorVector * errorMatrix * errorVector) sqrt! !

!DhbEstimatedPolynomial methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
errorMatrix
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/5/99 "
	^errorMatrix! !

!DhbEstimatedPolynomial methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
errorMatrix: aMatrix
		"Defines the error matrix of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	errorMatrix := aMatrix.! !

!DhbEstimatedPolynomial methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
valueAndError: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/5/99 "
	^Array with: ( self value: aNumber) with: ( self error: aNumber)! !

!DhbPolynomial class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
coefficients: anArray
		" Creates a new instance with given coefficients
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	^self new initialize: anArray reverse! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
* aNumberOrPolynomial
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aNumberOrPolynomial timesPolynomial: self! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
+ aNumberOrPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aNumberOrPolynomial addPolynomial: self! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
- aNumberOrPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aNumberOrPolynomial subtractToPolynomial: self! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
/ aNumberOrPolynomial
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aNumberOrPolynomial dividingPolynomial: self! !

!DhbPolynomial methodsFor: 'comparing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
= aNumberOrPolynomial 
	aNumberOrPolynomial isNil ifTrue: [^false].
	aNumberOrPolynomial isNumber ifTrue: [^coefficients size = 1 and: [coefficients first = aNumberOrPolynomial]].
	aNumberOrPolynomial class = self class ifFalse: [^false].
	^self coefficients = aNumberOrPolynomial coefficients! !

!DhbPolynomial methodsFor: '*Math-Complex-Extensions' stamp: 'DanielUber 5/12/2012 21:53'!
adaptToComplex: rcvr andSend: selector
	^(self class coefficients: (Array with: rcvr) ) perform: selector with: self! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToNumber: rcvr andSend: selector
	^(self class coefficients: (Array with: rcvr) ) perform: selector with: self! !

!DhbPolynomial methodsFor: '*Math-Quaternion-Extensions' stamp: 'DanielUber 5/12/2012 21:53'!
adaptToQuaternion: rcvr andSend: selector
	^(self class coefficients: (Array with: rcvr) ) perform: selector with: self! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
addNumber: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	| newCoefficients |
	newCoefficients := coefficients reverse.
	newCoefficients at: 1 put: newCoefficients first + aNumber.
	^self class coefficients: newCoefficients! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
addPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^self class coefficients: ( ( 0 to: (self degree max: aPolynomial degree)) collect: [ :n | ( aPolynomial at: n) + ( self at: n)])! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
at: anInteger
		"Answers the coefficient of order anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^anInteger < coefficients size
		ifTrue: [ coefficients at: ( coefficients size - anInteger)]
		ifFalse:[ 0]! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
coefficients
	"(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 21/5/99"

	^coefficients reverse! !

!DhbPolynomial methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
deflatedAt: aNumber
		"Answers a new polynomial quotient of the receiver with polynomial (X-aNumber)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/4/99 "
	| remainder next newCoefficients|
	remainder := 0.
	newCoefficients := coefficients collect:
						[ :each |
						  next := remainder. 
						  remainder := remainder * aNumber + each.
						  next].
	^self class coefficients: ( newCoefficients copyFrom: 2 to: newCoefficients size) reverse! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
degree
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^coefficients size - 1! !

!DhbPolynomial methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
derivative
		"Answer a new polynomial, derivative of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	| n |
	n := coefficients size.
	^self class coefficients: ( ( coefficients collect: [ :each | n := n - 1. each * n]) reverse copyFrom: 2 to: coefficients size)! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
differenceFromDouble: aDouble 
	^self subtractFrom: aDouble! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
differenceFromFixedPoint: aFixedPoint 
	^self subtractFrom: aFixedPoint! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
differenceFromFraction: aFraction 
	^self subtractFrom: aFraction! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
differenceFromInteger: anInteger 
	^self subtractFrom: anInteger! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
dividingPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^( self dividingPolynomialWithRemainder: aPolynomial) first! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
dividingPolynomialWithRemainder: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	| remainderCoefficients quotientCoefficients n m norm quotientDegree |
	n := self degree.
	m := aPolynomial degree.
	quotientDegree := m - n.
	quotientDegree < 0
		ifTrue: [ ^Array with: ( self class new: #(0)) with: aPolynomial].
	quotientCoefficients := Array new: quotientDegree + 1.
	remainderCoefficients := ( 0 to: m) collect: [ :k | aPolynomial at: k].
	norm := 1 / coefficients first.
	quotientDegree to: 0 by: -1
		do: [ :k | | x |
			  x := ( remainderCoefficients at: n + k + 1) * norm.
			  quotientCoefficients at: (quotientDegree + 1 - k) put: x.
			  (n + k - 1) to: k by: -1
				do: [ :j | 
				remainderCoefficients at: j + 1 put: 
							( ( remainderCoefficients at: j + 1) - ( x * (self at: j - k)))
				].
			].
	^Array with: ( self class coefficients: quotientCoefficients reverse)
		   with: ( self class coefficients: ( remainderCoefficients copyFrom: 1 to: n))! !

!DhbPolynomial methodsFor: 'comparing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
hash
	^coefficients hash! !

!DhbPolynomial methodsFor: 'initialization' stamp: 'DanielUber 5/11/2012 17:23'!
initialize
	"initialize an empty polynomial as the constant zero"
	super initialize.
	coefficients := #( 0 ).
	^ self! !

!DhbPolynomial methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anArray
		"Private - Initialize the coefficients of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	coefficients := anArray.
	^self! !

!DhbPolynomial methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
integral
		"Answer a new polynomial, integral of the receiver with value 0 at x=0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	^self integral: 0! !

!DhbPolynomial methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
integral: aValue
		"Answer a new polynomial, integral of the receiver with given value at x=0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	| n |
	n := coefficients size + 1.
	^self class coefficients: ( ( coefficients collect: [ :each | n := n - 1. each / n]) copyWith: aValue) reverse! !

!DhbPolynomial methodsFor: 'display' stamp: 'DanielUber 5/11/2012 17:52'!
printOn: aStream 
	"Append to aStream a written representation of the receiver. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 4/1/99"

	| n firstNonZeroCoefficientPrinted |
	n := 0.
	firstNonZeroCoefficientPrinted := false.
	coefficients
		reverseDo: 
			[:each | 
			each = 0
				ifFalse: 
					[firstNonZeroCoefficientPrinted
						ifTrue: 
							[aStream space.
							((each respondsTo: #< ) and: [each < 0]) ifFalse: [aStream nextPut: $+].
							aStream space]
						ifFalse: [firstNonZeroCoefficientPrinted := true].
					(each = 1 and: [n > 0])
						ifFalse: [each printOn: aStream].
					n > 0
						ifTrue: 
							[aStream nextPutAll: ' X'.
							n > 1
								ifTrue: 
									[aStream nextPut: $^.
									n printOn: aStream]]].
			n := n + 1].
		
	"add a quick check, print a zero if the polynomial has all zero coefficients
	Daniel Uber 11 May 2012"
	firstNonZeroCoefficientPrinted ifFalse: [0 printOn: aStream].! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromDouble: aDouble 
	^self * aDouble! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromFixedPoint: aFixedPoint 
	^self * aFixedPoint! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromFraction: aFraction 
	^self * aFraction! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
productFromInteger: anInteger 
	^self * anInteger! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
roots
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/4/99 "
	^self roots: DhbFloatingPointMachine new defaultNumericalPrecision! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
roots: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/4/99 "
	| pol roots x rootFinder |
	rootFinder := DhbNewtonZeroFinder new.
	rootFinder desiredPrecision: aNumber.
	pol := self class coefficients: ( coefficients reverse collect: [ :each | each asFloat]).
	roots := OrderedCollection new: self degree.
	[ rootFinder setFunction: pol; setDerivative: pol derivative.
	  x := rootFinder evaluate.
	  rootFinder hasConverged
		] whileTrue: [ roots add: x. 
					   pol := pol deflatedAt: x. 
					   pol degree > 0
						 ifFalse: [ ^roots].
					 ].
	^roots! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
subtractFrom: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	| newCoefficients |
	newCoefficients := ( coefficients collect: [ :c | c negated]) reverse.
	newCoefficients at: 1 put: newCoefficients first + aNumber.
	^self class coefficients: newCoefficients! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
subtractToPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^self class coefficients: ( ( 0 to: (self degree max: aPolynomial degree)) collect: [ :n | ( aPolynomial at: n) - ( self at: n)])! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sumFromDouble: aDouble 
	^self + aDouble! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sumFromFixedPoint: aFixedPoint 
	^self + aFixedPoint! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sumFromFraction: aFraction 
	^self + aFraction! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sumFromInteger: anInteger 
	^self + anInteger! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
timesNumber: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^self class coefficients: ( coefficients reverse collect: [ :each | each * aNumber])! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
timesPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	| productCoefficients degree|
	degree := aPolynomial degree + self degree.
	productCoefficients := ( degree to: 0 by: -1)
			collect:[ :n | | sum |
					  sum := 0.
					  0 to: (degree - n)
						do: [ :k | sum := (self at: k) * (aPolynomial at: ( degree - n - k)) + sum].
					  sum
					].
	^self class coefficients: productCoefficients! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answer the value of the polynomial for the specified variable value.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	^coefficients inject: 0 into: [ :sum :each | sum * aNumber + each]! !

Object subclass: #DhbPolynomialLeastSquareFit
	instanceVariableNames: 'pointCollection degreePlusOne'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbPolynomialLeastSquareFit class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: anInteger
		"Create a new instance of the receiver with given degree.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^super new initialize: anInteger! !

!DhbPolynomialLeastSquareFit class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: anInteger on: aCollectionOfPoints
		"Create a new instance of the receiver with given degree and points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^super new initialize: anInteger on: aCollectionOfPoints! !

!DhbPolynomialLeastSquareFit methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aWeightedPoint into: aVectorOfVectors and: aVector
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| t p powers |
	p := 1.0.
	powers := aVector collect: [ :each | t := p. p := p * aWeightedPoint xValue. t].
	aVector accumulate: powers * ( aWeightedPoint yValue * aWeightedPoint weight).
	1 to: aVector size do:
		[ :k |
		  ( aVectorOfVectors at: k) accumulate: powers * ( ( powers at: k) * aWeightedPoint weight).
		].! !

!DhbPolynomialLeastSquareFit methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
add: aWeightedPoint
		"Add a point to the collection of points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^pointCollection add: aWeightedPoint! !

!DhbPolynomialLeastSquareFit methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeEquations
		"Private - Answer a pair Matrix/Vector defining the system of equations
		 to solve the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| rows vector |
	vector := ( DhbVector new: degreePlusOne) atAllPut: 0 ; yourself.
	rows := ( 1 to: degreePlusOne) collect: [ :k | ( DhbVector new: degreePlusOne) atAllPut: 0 ; yourself].
	pointCollection do:
		[ :each | self accumulate: each into: rows and: vector].
	^Array with: ( DhbSymmetricMatrix rows: rows) with: vector! !

!DhbPolynomialLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
evaluate
		"Perform the least square fit and answers the fitted polynomial.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| system errorMatrix |
	system := self computeEquations.
	errorMatrix := ( system at: 1) inverse.
	^( DhbEstimatedPolynomial coefficients: errorMatrix * (system at: 2))
			errorMatrix: errorMatrix;
			yourself! !

!DhbPolynomialLeastSquareFit methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anInteger
		"Private - Create an empty point collection for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^self initialize: anInteger on: OrderedCollection new! !

!DhbPolynomialLeastSquareFit methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anInteger on: aCollectionOfPoints
		"Private - Defines the collection of points for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	pointCollection := aCollectionOfPoints.
	degreePlusOne := anInteger + 1.
	^self! !

Object subclass: #DhbProbabilityDensity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbProbabilityDensity subclass: #DhbAsymptoticChiSquareDistribution
	instanceVariableNames: 'degreeOfFreedom reducedDOF normalDistribution'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbAsymptoticChiSquareDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
degreeOfFreedom: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	^super new initialize: anInteger! !

!DhbAsymptoticChiSquareDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	^'Chi square distribution'! !

!DhbAsymptoticChiSquareDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^degreeOfFreedom! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	normalDistribution changeParametersBy: aVector.! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
confidenceLevel: aNumber
		"Answer the probability in percent of finding a chi square value
		 distributed according to the receiver larger than aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	^( 1 - ( self distributionValue: aNumber)) *100! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	| x |
	^aNumber > 0
		ifTrue: [ x := (aNumber * 2) sqrt.
				  ( DhbErfApproximation new value: (x - reducedDOF))
				]
		ifFalse:[ 0]! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	degreeOfFreedom := anInteger.
	reducedDOF := ( degreeOfFreedom * 2 - 1) sqrt.
	^self! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	^12 / degreeOfFreedom! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"Returns an Array containing the parameters of the distribution.
		 It is used to print out the distribution and for fitting.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: degreeOfFreedom! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbNormalDistribution random + reducedDOF) squared / 2! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	^( 2 / degreeOfFreedom) sqrt * 2! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	| x |
	^aNumber > 0
		ifTrue: [ x := (aNumber * 2) sqrt.
				  ( DhbErfApproximation new normal: (x - reducedDOF)) / x
				]
		ifFalse:[ 0]! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^degreeOfFreedom * 2! !

DhbProbabilityDensity subclass: #DhbBetaDistribution
	instanceVariableNames: 'alpha1 alpha2 gamma1 gamma2 logNorm incompleteBetaFunction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbBetaDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Beta distribution'! !

!DhbBetaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the 
	given histogram using best guesses. This method can be used to 
	find the initial values for a fit. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 12/3/99"

	| average variance a b |
	(aHistogram minimum < 0 or: [aHistogram maximum > 1])
		ifTrue: [^nil].
	average := aHistogram average.
	variance := aHistogram variance.
	a := 1 - average / variance - 1 * average.
	a > 0 ifFalse: [^nil].
	b := 1 / average - 1 * a.
	b > 0 ifFalse: [^nil].
	^self shape: a shape: b! !

!DhbBetaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbBetaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
shape: aNumber1 shape: aNumber2 
	"Create an instance of the receiver with given shape parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 shape: aNumber2! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^alpha1 / ( alpha1 + alpha2)! !

!DhbBetaDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	alpha1 := alpha1 + ( aVector at: 1).
	alpha2 := alpha2 + ( aVector at: 2).
	self computeNorm.
	gamma1 := nil.
	gamma2 := nil.
	incompleteBetaFunction := nil.! !

!DhbBetaDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeNorm
		"Private - Compute the norm of the receiver because its parameters have changed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	logNorm := (alpha1 + alpha2) logGamma - alpha1 logGamma - alpha2 logGamma.! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	incompleteBetaFunction isNil
		ifTrue: [ incompleteBetaFunction := DhbIncompleteBetaFunction shape: alpha1 shape: alpha2].
	^incompleteBetaFunction value: aNumber! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
firstGammaDistribution
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	gamma1 isNil
		ifTrue: [ gamma1 := DhbGammaDistribution shape: alpha1 scale: 1].
	 ^gamma1! !

!DhbBetaDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 shape: aNumber2 
	"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	(aNumber1 > 0 and: [aNumber2 > 0]) 
		ifFalse: [self error: 'Illegal distribution parameters'].
	alpha1 := aNumber1.
	alpha2 := aNumber2.
	self computeNorm.
	^self! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^3 * ( alpha1 + alpha2 + 1) * ( (alpha1 + alpha2) squared * 2 + ( ( alpha1 + alpha2 - 6) * alpha1 * alpha2)
			/ ( ( alpha1 + alpha2 + 2) * ( alpha1 + alpha2 + 3) * alpha1 * alpha2)) - 3! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha1 with: alpha2! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| r |
	r := self firstGammaDistribution random.
	^r / ( self secondGammaDistribution random + r)! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
secondGammaDistribution
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	gamma2 isNil
		ifTrue: [ gamma2 := DhbGammaDistribution shape: alpha2 scale: 1].
	 ^gamma2! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^( alpha1 + alpha2 + 1) sqrt * 2 * ( alpha2 - alpha1) / ( ( alpha1 * alpha2) sqrt * ( alpha1 + alpha2 + 2))! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber 
	"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^(aNumber > 0 and: [ aNumber < 1]) 
		ifTrue: 
			[( ( aNumber ln * (alpha1 - 1) ) + ( ( 1 - aNumber) ln * ( alpha2 - 1)) + logNorm) exp]
		ifFalse: [0]! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^alpha1 * alpha2 / ( ( alpha1 + alpha2) squared * ( alpha1 + alpha2 + 1))! !

DhbProbabilityDensity subclass: #DhbCauchyDistribution
	instanceVariableNames: 'mu beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCauchyDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Cauchy distribution'! !

!DhbCauchyDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	^self shape: aHistogram average
		scale: 4 * aHistogram variance 
				/ (Float pi * (aHistogram maximum squared + aHistogram minimum squared)) 
						sqrt! !

!DhbCauchyDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
	"Create an instance of the receiver with center 0 and scale 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^self shape: 0 scale: 1! !

!DhbCauchyDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
shape: aNumber1 scale: aNumber2 
	"Create an instance of the receiver with given center and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^mu! !

!DhbCauchyDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	mu := mu + ( aVector at: 1).
	beta := beta + ( aVector at: 2).! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 Assumes that the value of the receiver is 0 for x < 0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(( aNumber - mu) / beta) arcTan / Float pi + (1 / 2)! !

!DhbCauchyDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	mu := aNumber1.
	beta := aNumber2.
	^self! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: mu with: beta! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^( ( aNumber - (1 / 2)) * Float pi) tan * beta + mu! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
standardDeviation
		"The standard deviation of the receiver is not defined.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^nil! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^beta / ( Float pi * ( beta squared + ( aNumber - mu) squared))! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| dp denominator |
	dp := self value: aNumber.
	denominator := 1 / ( ( aNumber - mu) squared + beta squared).
	^Array with: dp
		   with: ( DhbVector with: 2 * dp * ( aNumber - mu) * denominator
							 with: dp * ( 1 / beta - ( 2 * beta * denominator)))! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"The variance of the receiver is not defined.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^nil! !

DhbProbabilityDensity subclass: #DhbExponentialDistribution
	instanceVariableNames: 'beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbExponentialDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Exponential distribution'! !

!DhbExponentialDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	| mu |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	mu := aHistogram average.
	^mu > 0 ifTrue: [ self scale: aHistogram average]
			ifFalse:[ nil]! !

!DhbExponentialDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Create a new instance of the receiver with scale parameter 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: 1! !

!DhbExponentialDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
scale: aNumber
		"Create a new instance of the receiver with given scale parameter.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^beta! !

!DhbExponentialDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	beta := beta + ( aVector at: 1).! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^[1 - ( ( aNumber / beta negated) exp)]
			on: Error do: [ :signal | signal return: 0]! !

!DhbExponentialDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber
		"Private - Set the scale parameter of the receiver to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	aNumber > 0
		ifFalse: [ self error: 'Illegal distribution parameters'].
	beta := aNumber.
	^self! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^6! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: beta! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(1 - aNumber) ln negated * beta! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^DhbMitchellMooreGenerator new floatValue ln * beta negated! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^2! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^beta! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^[ ( aNumber / beta) negated exp / beta]
			on: Error do: [ :signal | signal return: 0]! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	| dp |
	dp := self value: aNumber.
	^Array with: dp
		   with: ( DhbVector with: ( aNumber / beta - 1) * dp / beta)! !

DhbProbabilityDensity subclass: #DhbFisherSnedecorDistribution
	instanceVariableNames: 'dof1 dof2 norm chiSquareDistribution1 chiSquareDistribution2 incompleteBetaFunction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFisherSnedecorDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
degreeOfFreedom: anInteger1 degreeOfFreedom: anInteger2 
	"Create a new instance of the receiver with given degrees of freedom.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "

	^super new initialize: anInteger1 and: anInteger2! !

!DhbFisherSnedecorDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Fisher-Snedecor distribution'! !

!DhbFisherSnedecorDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "

	| n1 n2 a |
	aHistogram minimum < 0 ifTrue: [^nil].
	n2 := (2 / (1 - (1 / aHistogram average))) rounded.
	n2 > 0 ifFalse: [^nil].
	a := (n2 - 2) * (n2 - 4) * aHistogram variance / (n2 squared * 2).
	n1 := (0.7 * (n2 - 2) / (1 - a)) rounded.
	^n1 > 0 
		ifTrue: [self degreeOfFreedom: n1 degreeOfFreedom: n2]
		ifFalse: [nil]! !

!DhbFisherSnedecorDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbFisherSnedecorDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
test: aStatisticalMoment1 with: aStatisticalMoment2 
	"Perform a consistency Fisher test (or F-test) on the variances of two statistical moments ( or histograms).
		 Answer the probability of passing the test.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "

	^(self class degreeOfFreedom: aStatisticalMoment1 count
		degreeOfFreedom: aStatisticalMoment2 count) 
			distributionValue: aStatisticalMoment1 variance 
					/ aStatisticalMoment2 variance! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 Undefined if dof2 is smaller than 3.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^dof2 > 2
		ifTrue: [ dof2 / ( dof2 - 2)]
		ifFalse:[ nil]! !

!DhbFisherSnedecorDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	dof1 := ( dof1 + ( aVector at: 1)) max: 1.
	dof2 := ( dof2 + ( aVector at: 2)) max: 1.
	self computeNorm.
	chiSquareDistribution1 := nil.
	chiSquareDistribution2 := nil.
	incompleteBetaFunction := nil.! !

!DhbFisherSnedecorDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeNorm
		"Private - Compute the norm of the receiver because its parameters have changed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	norm := ( dof1 ln * ( dof1 / 2) ) + ( dof2 ln * ( dof2 / 2) )
						- ( ( dof1 / 2) logBeta: ( dof2 / 2) ).! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
confidenceLevel: aNumber
		"Answer the probability in percent of finding a value
		 distributed according to the receiver outside of the
		 interval [ 1/aNumber, aNumber].
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "
	aNumber < 0
		ifTrue: [ self error: 'Confidence level argument must be positive'].
	^( 1- ( self acceptanceBetween: aNumber reciprocal and: aNumber)) * 100! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "
	^1 - ( self incompleteBetaFunction value: ( dof2 / ( aNumber * dof1 + dof2)))! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
incompleteBetaFunction
	"Private - Answers the incomplete beta function used to compute
		 the symmetric acceptance integral of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "

	incompleteBetaFunction isNil 
		ifTrue: 
			[incompleteBetaFunction := DhbIncompleteBetaFunction shape: dof2 / 2
						shape: dof1 / 2].
	^incompleteBetaFunction! !

!DhbFisherSnedecorDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anInteger1 and: anInteger2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	dof1 := anInteger1.
	dof2 := anInteger2.
	self computeNorm.
	^self! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: dof1 with: dof2! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed according to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	chiSquareDistribution1 isNil
		ifTrue: [ chiSquareDistribution1 := DhbChiSquareDistribution degreeOfFreedom: dof1.
				  chiSquareDistribution2 := DhbChiSquareDistribution degreeOfFreedom: dof2.
				].
	^chiSquareDistribution1 random * dof2 / ( chiSquareDistribution2 random * dof1)! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > 0
		ifTrue: [ ( norm + ( aNumber ln * ( dof1 / 2 - 1) ) - ( (aNumber * dof1 + dof2) ln * ( ( dof1 + dof2) / 2))) exp]
		ifFalse:[ 0]! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the receiver.
		 Undefined if dof2 is smaller than 5.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^dof2 > 4 ifTrue: [ dof2 squared * 2 * ( dof1 + dof2 - 2) / ( ( dof2 - 2) squared * dof1 * ( dof2 - 4))]
				   ifFalse:[ nil]! !

DhbProbabilityDensity subclass: #DhbFisherTippettDistribution
	instanceVariableNames: 'alpha beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFisherTippettDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Fisher-Tippett distribution'! !

!DhbFisherTippettDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	| beta |
	beta := aHistogram standardDeviation.
	beta = 0 ifTrue: [^nil].
	beta := beta * (6 sqrt / Float pi).
	^self shape: aHistogram average - (0.5772156649 * beta) scale: beta! !

!DhbFisherTippettDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
	"Create a standard version of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^self shape: 0 scale: 1! !

!DhbFisherTippettDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
shape: aNumber1 scale: aNumber2 
	"Create an instance of the receiver with given shape and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^0.577256649 * beta + alpha! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	alpha := alpha + ( aVector at: 1).
	beta := beta + ( aVector at: 2).! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 14/4/99 "
	| arg |
	arg := ( aNumber - alpha) / beta.
	arg := arg < DhbFloatingPointMachine new largestExponentArgument negated
					ifTrue: [ ^0]
					ifFalse:[arg negated exp].
	^arg > DhbFloatingPointMachine new largestExponentArgument ifTrue: [ 1]
						ifFalse:[ arg negated exp]! !

!DhbFisherTippettDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	aNumber2 > 0
		ifFalse: [ self error: 'Illegal distribution parameters'].
	alpha := aNumber1.
	beta := aNumber2.
	^self! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
integralFrom: aNumber1 to: aNumber2
		"Private - Compute the integral of the receiver from aNumber1 to aNumber2.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 14/4/99 "
	^( DhbRombergIntegrator new: self from: aNumber1 to: aNumber2) evaluate! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
integralUpTo: aNumber
		"Private - Compute the integral of the receiver from -infinity to aNumber.
		 aNumber must be below 0 (no checking!!).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 14/4/99 "
	^( DhbRombergIntegrator new:
			[ :x | x = 0 ifTrue: [ 0] ifFalse: [ ( self value: 1 / x) / x squared] ]
			from: 1 / aNumber to: 0) evaluate! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^2.4! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha with: beta! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed according to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/4/99 "
	| t |
	[ t := DhbMitchellMooreGenerator new floatValue ln negated.
	  t > 0] whileFalse: [].
	^t ln negated * beta + alpha! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^1.3! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Float pi * beta / ( 6 sqrt)! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	| arg |
	arg := ( aNumber - alpha) / beta.
	arg := arg > DhbFloatingPointMachine new largestExponentArgument ifTrue: [ ^0]
						ifFalse:[arg negated exp + arg].
	^arg > DhbFloatingPointMachine new largestExponentArgument ifTrue: [ 0]
						ifFalse:[ arg negated exp / beta]! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| dp dy y|
	dp := self value: aNumber.
	y := ( aNumber - alpha) / beta.
	dy := ( y negated exp - 1).
	^Array with: dp
		   with: ( DhbVector with: dy * dp / beta negated
							 with: dp * ( y * dy + 1) / beta negated)! !

DhbProbabilityDensity subclass: #DhbGammaDistribution
	instanceVariableNames: 'alpha beta norm randomCoefficients incompleteGammaFunction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbGammaDistribution subclass: #DhbChiSquareDistribution
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbChiSquareDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
degreeOfFreedom: anInteger
		"Create a new instance of the receiver with given degree of freedom.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^anInteger > 40
		ifTrue: [ DhbAsymptoticChiSquareDistribution degreeOfFreedom: anInteger]
		ifFalse:[ super shape: anInteger / 2 scale: 2]! !

!DhbChiSquareDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Chi square distribution'! !

!DhbChiSquareDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	| dof |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	dof := aHistogram average rounded.
	^dof > 0 ifTrue: [ self degreeOfFreedom: aHistogram average rounded]
			 ifFalse:[ nil]! !

!DhbChiSquareDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
shape: aNumber1 scale: aNumber2
		"Create an instance of the receiver with given shape and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbChiSquareDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 13/4/99 "
	super changeParametersBy: (Array with: aVector first / 2 with: 0).! !

!DhbChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
confidenceLevel: aNumber
		"Answer the probability in percent of finding a chi square value
		 distributed according to the receiver larger than aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	^( 1 - ( self distributionValue: aNumber)) *100! !

!DhbChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha * 2! !

!DhbGammaDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Gamma distribution'! !

!DhbGammaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	| alpha beta |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	alpha := aHistogram average.
	beta := aHistogram variance / alpha.
	^[ self shape: alpha / beta scale: beta] on: Error do: [ :signal | signal return: nil]! !

!DhbGammaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbGammaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
shape: aNumber1 scale: aNumber2
		"Create an instance of the receiver with given shape and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 scale: aNumber2! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^alpha * beta! !

!DhbGammaDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	alpha := alpha + ( aVector at: 1).
	beta := beta + ( aVector at: 2).
	self computeNorm.
	incompleteGammaFunction := nil.
	randomCoefficients := nil.! !

!DhbGammaDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeNorm
		"Private - Compute the norm of the receiver because its parameters have changed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	norm := beta ln * alpha + alpha logGamma.! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self incompleteGammaFunction value: aNumber / beta! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
incompleteGammaFunction
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	incompleteGammaFunction isNil 
		ifTrue: 
			[incompleteGammaFunction := DhbIncompleteGammaFunction shape: alpha].
	^incompleteGammaFunction! !

!DhbGammaDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	( aNumber1 > 0 and: [ aNumber2 > 0])
		ifFalse: [ self error: 'Illegal distribution parameters'].
	alpha := aNumber1.
	beta := aNumber2.
	self computeNorm.
	^self! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeRandomCoefficientsForLargeAlpha
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	| a b q d |
	a := 1 / ( 2 * alpha - 1) sqrt.
	b := alpha - (4 ln).
	q := 1 / a + alpha.
	d := 4.5 ln + 1.
	^Array with: a with: b with: q with: d! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initializeRandomCoefficientsForSmallAlpha
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	| e |
	e := 1 exp.
	^( e + alpha) / e! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^6 / alpha! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha with: beta! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( alpha > 1 ifTrue: [ self randomForLargeAlpha]
						ifFalse:[ self randomForSmallAlpha]) * beta! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
randomCoefficientsForLargeAlpha
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	randomCoefficients isNil
		ifTrue: [ randomCoefficients := self initializeRandomCoefficientsForLargeAlpha].
	^randomCoefficients! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
randomCoefficientsForSmallAlpha
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	randomCoefficients isNil
		ifTrue: [ randomCoefficients := self initializeRandomCoefficientsForSmallAlpha].
	^randomCoefficients! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
randomForLargeAlpha
		"Private - Generate a random number distributed according to the receiver
						when alpha > 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	[ true] whileTrue: [
	| u1 u2 c v y z w|
	u1 := DhbMitchellMooreGenerator new floatValue.
	u2 := DhbMitchellMooreGenerator new floatValue.
	c := self randomCoefficientsForLargeAlpha.
	v := ( u1 / ( 1 - u1)) ln * (c at: 1).
	y := v exp * alpha.
	z := u1 squared * u2.
	w := ( c at: 3) * v + ( c at: 2) - y.
	( c at: 4) + w >= ( 4.5 * z) ifTrue: [ ^y].
	z ln <= w ifTrue: [ ^y].
								].! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
randomForSmallAlpha
		"Private - Generate a random number distributed according to the receiver
						when alpha < 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	[ true] whileTrue: [
	| p |
	p := DhbMitchellMooreGenerator new floatValue * self randomCoefficientsForSmallAlpha.
	p > 1
		ifTrue: [ | y |
					 y := ( ( self randomCoefficientsForSmallAlpha - p) / alpha) ln negated.
					 DhbMitchellMooreGenerator new floatValue <= ( y raisedTo: ( alpha - 1))
					 	ifTrue: [ ^y].
					]
		ifFalse: [ | y |
						y := p raisedTo: ( 1 / alpha).
					 DhbMitchellMooreGenerator new floatValue <= ( y negated exp)
					 	ifTrue: [ ^y].
					 ].
								].! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^2 / alpha sqrt! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > 0
		ifTrue: [ ( aNumber ln * (alpha - 1) - (aNumber / beta) - norm) exp]
		ifFalse:[ 0].! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^beta squared * alpha! !

DhbProbabilityDensity subclass: #DhbHistogrammedDistribution
	instanceVariableNames: 'histogram'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbHistogrammedDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Experimental distribution'! !

!DhbHistogrammedDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
histogram: aHistogram
		"Create a new instance of the receiver corresponding to a histogram.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^super new initialize: aHistogram! !

!DhbHistogrammedDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
acceptanceBetween: aNumber1 and: aNumber2
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value larger than aNumber 1 and lower than or equal to aNumber2.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( histogram countsBetween: ( aNumber1 max: histogram minimum)
						and: ( aNumber2 min: histogram maximum) ) / histogram totalCount! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram average! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber < histogram minimum
		ifTrue: [ 0]
		ifFalse:[ aNumber < histogram maximum
							ifTrue: [ ( histogram countsUpTo: aNumber) / histogram totalCount]
							ifFalse:[ 1]
					]! !

!DhbHistogrammedDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aHistogram
		"Private - Defines the histogram of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	aHistogram count = 0
		ifTrue: [ self error: 'Cannot define probability density on an empty histogram'].
	histogram := aHistogram.
	^self! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 Undefined. Must be implemented by subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^histogram kurtosis! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose distribution is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram inverseDistributionValue: aNumber! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 Undefined. Must be implemented by subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^histogram skewness! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram standardDeviation! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^( aNumber >= histogram minimum and: [ aNumber < histogram maximum])
		ifTrue: [ ( histogram countAt: aNumber) / ( histogram totalCount * histogram binWidth)]
		ifFalse:[ 0]! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram variance! !

DhbProbabilityDensity subclass: #DhbLaplaceDistribution
	instanceVariableNames: 'mu beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLaplaceDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Laplace distribution'! !

!DhbLaplaceDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	^self shape: aHistogram average scale: (aHistogram variance / 2) sqrt! !

!DhbLaplaceDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "

	^self shape: 0 scale: 1! !

!DhbLaplaceDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
shape: aNumber1 scale: aNumber2 
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^mu! !

!DhbLaplaceDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	mu := mu + ( aVector at: 1).
	beta := beta + ( aVector at: 2).! !

!DhbLaplaceDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > mu
		ifTrue: [ 1 - ( ( ( aNumber - mu) / beta) negated exp / 2)]
		ifFalse:[ ( ( ( aNumber - mu) / beta) exp / 2)]! !

!DhbLaplaceDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	mu := aNumber1.
	beta := aNumber2.
	^self! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^3! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: mu with: beta! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| r |
	r := DhbMitchellMooreGenerator new floatValue ln * beta negated.
	^DhbMitchellMooreGenerator new floatValue > 0.5
		ifTrue: [ mu + r]
		ifFalse:[ mu - r]! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^beta * ( 2 sqrt)! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^( ( aNumber - mu) / beta) abs negated exp / ( 2 * beta)! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| dp |
	dp := self value: aNumber.
	^Array	with: dp
			with: ( DhbVector with: ( aNumber - mu) sign * dp / beta
							  with: ( ( ( aNumber - mu) abs / beta - 1) * dp / beta))! !

DhbProbabilityDensity subclass: #DhbNormalDistribution
	instanceVariableNames: 'mu sigma nextRandom'
	classVariableNames: 'NextRandom'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbNormalDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Normal distribution'! !

!DhbNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	^self new: aHistogram average sigma: aHistogram standardDeviation! !

!DhbNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Create a new instance of the receiver with mu=0 and sigma=1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self new: 0 sigma: 1! !

!DhbNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: aNumber1 sigma: aNumber2
		"Create a new instance of the receiver with given mu and sigma.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 sigma: aNumber2! !

!DhbNormalDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed according to a (0,1) normal distribution.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| v1 v2 w y |
	NextRandom isNil
		ifTrue: [ [ v1 := Number random * 2 - 1.
					v2 := Number random * 2 - 1.
					w := v1 squared + v2 squared.
					w > 1 ] whileTrue: [].
				  y := ( ( w ln * 2 negated) / w) sqrt.
				v1 := y * v1.
				NextRandom := y * v2.
				]
		ifFalse:[ v1 :=NextRandom.
				  NextRandom := nil.
				].
	^v1! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^mu! !

!DhbNormalDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	mu := mu + ( aVector at: 1).
	sigma := sigma + ( aVector at: 2).! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^DhbErfApproximation new value: ( ( aNumber - mu) / sigma)! !

!DhbNormalDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 sigma: aNumber2
		"Private - Defines the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	mu := aNumber1.
	sigma := aNumber2.
	^self! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: mu with: sigma! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self class random * sigma + mu! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^sigma! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbErfApproximation new normal: (aNumber - mu) / sigma) / sigma! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	| dp y |
	y := ( aNumber - mu) / sigma.
	dp := ( DhbErfApproximation new normal: y) / sigma.
	^Array with: dp
		   with: ( DhbVector with: dp * y / sigma
							 with: dp * ( y squared - 1) / sigma)! !

!DhbProbabilityDensity class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Unknown distribution'! !

!DhbProbabilityDensity class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 Default returns nil (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^nil! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
acceptanceBetween: aNumber1 and: aNumber2
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value larger than aNumber 1 and lower than or equal to aNumber2.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( self distributionValue: aNumber2) - ( self distributionValue: aNumber1)! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
approximatedValueAndGradient: aNumber
		"Private - gradients an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 The gradient is computed by approximation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| delta parameters dp gradient n |
	parameters := self parameters.
	n := parameters size.
	dp := self value: aNumber.
	delta := Array new: n.
	delta atAllPut: 0.
	gradient := DhbVector new: n.
	1 to: n do:
		[ :k |
		  delta at: k put: ( parameters at: k) * 0.0001.
		  self changeParametersBy: delta.
		  gradient at: k put: ( ( ( self value: aNumber) - dp) / ( delta at: k)).
		  delta at: k put: ( delta at: k ) negated.
		  k > 1
			ifTrue: [ delta at: ( k - 1) put: 0].
		].
	self changeParametersBy: delta.
	^Array with: dp with: gradient! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	self subclassResponsibility.! !

!DhbProbabilityDensity methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector 
	self subclassResponsibility! !

!DhbProbabilityDensity methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionFunction
		" (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^DhbProbabilityDistributionFunction density: self! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self subclassResponsibility! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inverseDistributionValue: aNumber
		"Answer the number whose distribution value is aNumber.
		 NOTE: Subclass MUST NOT overwrite this method.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( aNumber between: 0 and: 1)
			ifTrue: [ self privateInverseDistributionValue: aNumber]
			ifFalse:[ self error: 'Illegal argument for inverse distribution value']! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 Undefined. Must be implemented by subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^nil! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
	"Returns an Array containing the parameters of the distribution. 
	It is used to print out the distribution and for fitting. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 4/3/99"

	^self subclassResponsibility! !

!DhbProbabilityDensity methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 4/3/99"

	| params |
	aStream nextPutAll: self class distributionName.
	(params := self parameters) notNil
		ifTrue: 
			[| first |
			first := true.
			aStream nextPut: $(.
			params
				do: 
					[:each | 
					first
						ifTrue: [first := false]
						ifFalse: [aStream nextPut: $,].
					aStream space.
					each printOn: aStream].
			aStream nextPut: $)]! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose distribution is aNumber.
		 NOTE: Subclass may overwrite this method for faster computation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbNewtonZeroFinder function: [ :x | ( self distributionValue: x) - aNumber] derivative: self)
		initialValue: self average / (1 - aNumber); evaluate! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed according to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self privateInverseDistributionValue: DhbMitchellMooreGenerator new floatValue! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 Undefined. Must be implemented by subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^nil! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 NOTE: At least one of the methods variance or standardDeviation must be implemented by the subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self variance sqrt! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	self subclassResponsibility.! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^self approximatedValueAndGradient: aNumber! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the receiver.
		 NOTE: At least one of the methods variance or standardDeviation must be implemented by the subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self standardDeviation squared! !

DhbProbabilityDensity subclass: #DhbProbabilityDensityWithUnknownDistribution
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbProbabilityDensityWithUnknownDistribution subclass: #DhbLogNormalDistribution
	instanceVariableNames: 'normalDistribution'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLogNormalDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Log normal distribution'! !

!DhbLogNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	| average variance sigma2 |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	average := aHistogram average.
	average > 0
		ifFalse: [ ^nil].
	variance := aHistogram variance.
	sigma2 := ( variance / average squared + 1) ln.
	sigma2 > 0
		ifFalse: [ ^nil].
	^self new: ( average ln - (sigma2 * 0.5)) sigma: sigma2 sqrt! !

!DhbLogNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Create a new instance of the receiver with mu=0 and sigma=1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self new: 0 sigma: 1! !

!DhbLogNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: aNumber1 sigma: aNumber2
		"Create a new instance of the receiver with given mu and sigma.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 sigma: aNumber2! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( normalDistribution variance*0.5 + normalDistribution average) exp! !

!DhbLogNormalDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	normalDistribution changeParametersBy: aVector.! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fourthCentralMoment
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 24/6/99 "
	| y x |
	y := normalDistribution average exp.
	x := normalDistribution variance exp.
	^( y squared squared) * ( x squared)
		* ( ( ( x squared * x - 4) * ( x squared) + 6) * x - 3)! !

!DhbLogNormalDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 sigma: aNumber2
		"Private - Defines the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	normalDistribution := DhbNormalDistribution new: aNumber1 sigma: aNumber2.
	^self! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| x |
	x := normalDistribution variance exp.
	^( ( x + 2) * x + 3) * ( x squared) - 6! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^normalDistribution parameters! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^normalDistribution random exp! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| x |
	x := normalDistribution variance exp.
	^(x - 1) sqrt * (x + 2)! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
thirdCentralMoment
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 24/6/99 "
	| y x |
	y := normalDistribution average exp.
	x := normalDistribution variance exp.
	^( y squared * y) * ( x raisedTo: (3/2))
		* ( ( x squared negated + 3) * x - 2)! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > 0
		ifTrue: [ ( normalDistribution value: aNumber ln) / aNumber]
		ifFalse:[ 0]! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( normalDistribution average * 2 + normalDistribution variance) exp * ( normalDistribution variance exp - 1)! !

!DhbProbabilityDensityWithUnknownDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
acceptanceBetween: aNumber1 and: aNumber2
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value larger than aNumber 1 and lower than or equal to aNumber2.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbRombergIntegrator new: self from: aNumber1 to: aNumber2) evaluate! !

!DhbProbabilityDensityWithUnknownDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 This general purpose routine uses numerical integration.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbRombergIntegrator new: self from: self lowestValue to: aNumber) evaluate! !

!DhbProbabilityDensityWithUnknownDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
lowestValue
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/18/00 "
	^0! !

DhbProbabilityDensity subclass: #DhbStudentDistribution
	instanceVariableNames: 'degreeOfFreedom norm chiSquareDistribution incompleteBetaFunction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbStudentDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asymptoticLimit
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/4/99 "
	^30! !

!DhbStudentDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
degreeOfFreedom: anInteger 
	"Create a new instance of the receiver with anInteger degrees of freedom.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "

	^anInteger > self asymptoticLimit 
		ifTrue: [DhbNormalDistribution new]
		ifFalse: 
			[anInteger = 1 
				ifTrue: [DhbCauchyDistribution shape: 0 scale: 1]
				ifFalse: [super new initialize: anInteger]]! !

!DhbStudentDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Student distribution'! !

!DhbStudentDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	| dof var |
	var := aHistogram variance.
	var = 0
		ifTrue: [ ^nil].
	dof :=  ( 2 / (1 - (1 / aHistogram variance))) rounded max: 1.
	^dof > self asymptoticLimit ifTrue: [ nil]
								ifFalse:[ self degreeOfFreedom: dof]! !

!DhbStudentDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbStudentDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
test: aStatisticalMoment1 with: aStatisticalMoment2
		"Preform a consistency Student test (or t-test) on the averages of  two statistical moments ( or histograms).
		 Answers the probability of failing the test.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	| t |
	t := ( aStatisticalMoment1 average - aStatisticalMoment2 average) abs.
	^1 - ( ( self class degreeOfFreedom: (  aStatisticalMoment1 count + aStatisticalMoment2 count - 2)) acceptanceBetween: t negated and: t)! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^0! !

!DhbStudentDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	degreeOfFreedom := degreeOfFreedom + ( aVector at: 1).
	self computeNorm.! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
chiSquareDistribution
		"Private - Answer the chi square distribution used to generate
		 random numbers for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	chiSquareDistribution isNil
		ifTrue: [ chiSquareDistribution := DhbChiSquareDistribution degreeOfFreedom: (degreeOfFreedom - 1)].
	^chiSquareDistribution! !

!DhbStudentDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeNorm
		"Private - Compute the norm of the receiver because its parameters have changed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	norm := ( ( degreeOfFreedom / 2 logBeta: ( 1 / 2) ) + ( degreeOfFreedom ln / 2)) negated.! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
confidenceLevel: aNumber
		"Answer the probability in percent of finding a value
		 distributed according to the receiver with an absolute value
		 larger than aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "
	^( 1 - ( self symmetricAcceptance: aNumber abs)) * 100! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "
	aNumber = 0
		ifTrue: [ ^1/2].
	^( aNumber > 0
		ifTrue: [ 2 - ( self symmetricAcceptance: aNumber abs)]
		ifFalse:[ self symmetricAcceptance: aNumber abs]) / 2! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
incompleteBetaFunction
	"Private - Answers the incomplete beta function used to compute
		 the symmetric acceptance integral of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "

	incompleteBetaFunction isNil 
		ifTrue: 
			[incompleteBetaFunction := DhbIncompleteBetaFunction 
						shape: degreeOfFreedom / 2
						shape: 0.5].
	^incompleteBetaFunction! !

!DhbStudentDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anInteger
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	anInteger > 0
		ifFalse: [ self error: 'Degree of freedom must be positive'].
	degreeOfFreedom := anInteger.
	self computeNorm.
	^self! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 Undefined if the degree of freedom is less than 5.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^degreeOfFreedom > 4 ifTrue: [ 6 / ( degreeOfFreedom - 4)]
						 ifFalse:[ nil]! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: degreeOfFreedom! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
random
		"Answer a random number distributed according to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^DhbNormalDistribution random * ( ( (degreeOfFreedom - 1) / self chiSquareDistribution random ) sqrt)! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
symmetricAcceptance: aNumber
		"Private - Compute the acceptance of the receiver between -aNumber and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "
	^ self incompleteBetaFunction value: ( degreeOfFreedom / ( aNumber squared + degreeOfFreedom))! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( norm - ( ( aNumber squared / degreeOfFreedom + 1) ln * ( ( degreeOfFreedom + 1) / 2))) exp! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the receiver.
		 Undefined if the degree of freedom is less than 3.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^degreeOfFreedom > 2 ifTrue: [ degreeOfFreedom / ( degreeOfFreedom - 2)]
						 ifFalse:[ nil]! !

DhbProbabilityDensity subclass: #DhbTriangularDistribution
	instanceVariableNames: 'lowLimit highLimit peak'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbTriangularDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Triangular distribution'! !

!DhbTriangularDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 Default returns nil (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| b c|
	b := aHistogram standardDeviation * 1.73205080756888 "12 sqrt / 2".
	b = 0
		ifTrue: [ ^nil].
	c := aHistogram average.
	^self new: c from: ( c - b) to: ( c + b).! !

!DhbTriangularDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Create an instance of the receiver with peak at 1/2 and limits 0 and 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^self new: (1 / 2) from: 0 to: 1! !

!DhbTriangularDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: aNumber1 from: aNumber2 to: aNumber3
		"Create an instance of the receiver with peak at aNumber1 and limits aNumber2 and aNumber3.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 from: aNumber2 to: aNumber3! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(lowLimit + peak + highLimit) / 3! !

!DhbTriangularDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	lowLimit := lowLimit + ( aVector at: 1).
	highLimit := highLimit + ( aVector at: 2).
	peak := peak + ( aVector at: 3).! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
		"changed last line so that 1 is returned, if aNumber >highLimit "
	| norm |
	^( aNumber between: lowLimit and: highLimit)
		ifTrue: [ aNumber < peak
						ifTrue: [ norm := ( highLimit - lowLimit) * ( peak - lowLimit).
									 ( aNumber - lowLimit) squared / norm
									]
						ifFalse:[ aNumber > peak
										ifTrue: [ norm := ( highLimit - lowLimit) * ( highLimit - peak).
									  				 1 - ( ( highLimit - aNumber) squared / norm)
													]
										ifFalse:[ ( peak - lowLimit) / ( highLimit - lowLimit)]
									]
				   ]
		ifFalse:[ aNumber >lowLimit ifTrue: [1] ifFalse: [0]]! !

!DhbTriangularDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 from: aNumber2 to: aNumber3
		"Private - Defines the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	( aNumber2 < aNumber3 and: [ aNumber1 between: aNumber2 and: aNumber3])
		ifFalse: [ self error: 'Illegal distribution parameters'].
	peak := aNumber1.
	lowLimit := aNumber2.
	highLimit := aNumber3.
	^self! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inverseAcceptanceAfterPeak: aNumber
		"Private - Compute inverse acceptance function in the region after the peak.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^ highLimit - ( ( ( 1 - aNumber) * ( highLimit - lowLimit) * ( highLimit - peak)) sqrt)! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
inverseAcceptanceBeforePeak: aNumber
		"Private - Compute inverse acceptance function in the region before the peak.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^ ( aNumber * ( highLimit - lowLimit) * ( peak - lowLimit)) sqrt + lowLimit! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^(-6/10)! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: lowLimit with: highLimit with: peak! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( peak - lowLimit) / ( highLimit - lowLimit) > aNumber
			ifTrue: [ self inverseAcceptanceBeforePeak: aNumber]
			ifFalse: [ self inverseAcceptanceAfterPeak: aNumber]! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^(((lowLimit squared * lowLimit + ( peak squared * peak) + ( highLimit squared * highLimit) ) / 135)
	-(((lowLimit squared * peak) + (lowLimit squared * highLimit) + (peak squared * lowLimit) + (peak squared * highLimit) + (highLimit squared * lowLimit) + (highLimit squared * peak))/90)
	+( 2 * lowLimit * peak * highLimit / 45)) / ( self standardDeviation raisedToInteger: 3)! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| norm |
	^( aNumber between: lowLimit and: highLimit)
		ifTrue: [ aNumber < peak
						ifTrue: [ norm := ( highLimit - lowLimit) * ( peak - lowLimit).
									 2 * ( aNumber - lowLimit) / norm
									]
						ifFalse:[ aNumber > peak
										ifTrue: [ norm := ( highLimit - lowLimit) * ( highLimit - peak).
									  				 2 * ( highLimit - aNumber) / norm
													]
										ifFalse:[ 2 / ( highLimit - lowLimit)]
									]
				   ]
		ifFalse:[ 0]! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(lowLimit squared + peak squared + highLimit squared - ( lowLimit * peak) - ( lowLimit * highLimit) - ( peak * highLimit)) / 18! !

DhbProbabilityDensity subclass: #DhbUniformDistribution
	instanceVariableNames: 'lowLimit highLimit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Uniform distribution'! !

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
from: aNumber1 to: aNumber2
		"Create a new instance of the receiver with given limits.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 to: aNumber2! !

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 Default returns nil (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| b c|
	b := aHistogram standardDeviation * 1.73205080756888 "12 sqrt / 2".
	b = 0
		ifTrue: [ ^nil].
	c := aHistogram average.
	^self from: ( c - b) to: ( c + b).! !

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Create a new instance of the receiver with limits 0 and 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self from: 0 to: 1! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( highLimit + lowLimit) / 2! !

!DhbUniformDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	lowLimit := lowLimit + ( aVector at: 1).
	highLimit := highLimit + ( aVector at: 2).! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	aNumber < lowLimit
		ifTrue: [ ^0].
	^aNumber < highLimit
		ifTrue: [ (aNumber - lowLimit) / ( highLimit - lowLimit)]
		ifFalse:[ 1]! !

!DhbUniformDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 to: aNumber2
		"Private - Defines the limits of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	aNumber1 < aNumber2
		ifFalse: [ self error: 'Illegal distribution parameters'].
	lowLimit := aNumber1.
	highLimit := aNumber2.
	^self! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^-12 / 10! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: lowLimit with: highLimit! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(highLimit - lowLimit) * aNumber + lowLimit! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( highLimit - lowLimit) / 3.46410161513775 "12 sqrt"! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( aNumber between: lowLimit and: highLimit)
		ifTrue: [ 1/( highLimit - lowLimit)]
		ifFalse:[ 0]! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( highLimit - lowLimit) squared / 12! !

DhbProbabilityDensity subclass: #DhbWeibullDistribution
	instanceVariableNames: 'alpha beta norm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbWeibullDistribution class methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Weibull distribution'! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	| average xMin xMax accMin accMax |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	average := aHistogram average.
	xMin := ( aHistogram minimum + average) / 2.
	accMin := ( aHistogram countsUpTo: xMin) / aHistogram totalCount.
	xMax := ( aHistogram maximum + average) / 2.
	accMax := ( aHistogram countsUpTo: xMax) / aHistogram totalCount.
	^[self solve: xMin acc: accMin upper: xMax acc: accMax]
			on: Error do: [ :signal | signal return: nil]! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
shape: aNumber1 scale: aNumber2 
	"Create an instance of the receiver with given shape and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
solve: lowX acc: lowAcc upper: highX acc: highAcc 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "

	| lowLnAcc highLnAcc deltaLnAcc lowLnX highLnX |
	lowLnAcc := (1 - lowAcc) ln negated ln.
	highLnAcc := (1 - highAcc) ln negated ln.
	deltaLnAcc := highLnAcc - lowLnAcc.
	lowLnX := lowX ln.
	highLnX := highX ln.
	^self shape: deltaLnAcc / (highLnX - lowLnX)
		scale: ((highLnAcc * lowLnX - (lowLnAcc * highLnX)) / deltaLnAcc) exp! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(1 / alpha) gamma * beta / alpha! !

!DhbWeibullDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	alpha := alpha + ( aVector at: 1).
	beta := beta + ( aVector at: 2).
	self computeNorm.! !

!DhbWeibullDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
computeNorm
		"Private - Compute the norm of the receiver because its parameters have changed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	norm := alpha/ ( beta raisedTo: alpha).! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 Assumes that the value of the receiver is 0 for x < 0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > 0
		ifTrue: [ 1 - ( ( ( aNumber / beta) raisedTo: alpha) negated exp)]
		ifFalse:[ 0]! !

!DhbWeibullDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	( aNumber1 > 0 and: [ aNumber2 > 0])
		ifFalse: [ self error: 'Illegal distribution parameters'].
	alpha := aNumber1.
	beta := aNumber2.
	self computeNorm.
	^self! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha with: beta! !

!DhbWeibullDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( (1 - aNumber) ln negated raisedTo: ( 1 / alpha)) * beta! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( ( aNumber / beta) raisedTo: alpha) negated exp * ( aNumber raisedTo: ( alpha - 1)) * norm! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"Answer the variance of the receiver.
		 NOTE: At least one of the methods variance or standardDeviation must be implemented by the subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( beta squared / alpha) * ( (2 / alpha) gamma * 2 - ( (1 / alpha ) gamma squared / alpha))! !

Object subclass: #DhbProbabilityDistributionFunction
	instanceVariableNames: 'probabilityDensity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbProbabilityDistributionFunction class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
density: aProbabilityDensity
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^self new initialize: aProbabilityDensity! !

!DhbProbabilityDistributionFunction methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aProbabilityDensity
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	probabilityDensity := aProbabilityDensity.
	^self! !

!DhbProbabilityDistributionFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^probabilityDensity distributionValue: aNumber! !

Object subclass: #DhbProjectedOneVariableFunction
	instanceVariableNames: 'index function argument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbProjectedOneVariableFunction class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
function: aVectorFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new initialize: aVectorFunction! !

!DhbProjectedOneVariableFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
argumentWith: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^argument at: index put: aNumber; yourself! !

!DhbProjectedOneVariableFunction methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
bumpIndex
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index isNil
		ifTrue: [ index := 1]
		ifFalse:[ index := index + 1.
				  index > argument size
					ifTrue: [ index := 1].
				].! !

!DhbProjectedOneVariableFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
index
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index isNil
		ifTrue: [ index := 1].
	^index! !

!DhbProjectedOneVariableFunction methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	function := aFunction.
	^self! !

!DhbProjectedOneVariableFunction methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setArgument: anArrayOrVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	argument := anArrayOrVector copy.! !

!DhbProjectedOneVariableFunction methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setIndex: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index := anInteger.! !

!DhbProjectedOneVariableFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^function value: ( self argumentWith: aNumber)! !

DhbProjectedOneVariableFunction subclass: #DhbVectorProjectedFunction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbVectorProjectedFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
argumentWith: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^aNumber * self direction + self origin! !

!DhbVectorProjectedFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
direction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^index! !

!DhbVectorProjectedFunction methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
direction: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index := aVector.! !

!DhbVectorProjectedFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
origin
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^argument! !

!DhbVectorProjectedFunction methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
origin: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	argument := aVector.! !

!DhbVectorProjectedFunction methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	self origin printOn: aStream.
	aStream nextPutAll: ' ('.
	self direction printOn: aStream.
	aStream nextPut: $).! !

Object subclass: #DhbScaledProbabilityDensityFunction
	instanceVariableNames: 'probabilityDensityFunction count binWidth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbScaledProbabilityDensityFunction class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
histogram: aHistogram against: aProbabilityDensityFunction 
	"Create a new instance of the receiver with given probability density function and histogram.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	^self new 
		initialize: aProbabilityDensityFunction
		binWidth: aHistogram binWidth
		count: aHistogram totalCount! !

!DhbScaledProbabilityDensityFunction class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
histogram: aHistogram distributionClass: aProbabilityDensityFunctionClass 
	"Create a new instance of the receiver with given probability density function and histogram. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 3/3/99"

	| dp |
	^(dp := aProbabilityDensityFunctionClass fromHistogram: aHistogram) isNil
		ifTrue: [nil]
		ifFalse: [self histogram: aHistogram against: dp]! !

!DhbScaledProbabilityDensityFunction methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	count := count + aVector last.
	probabilityDensityFunction changeParametersBy: aVector.! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
distributionFunction
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^probabilityDensityFunction distributionFunction! !

!DhbScaledProbabilityDensityFunction methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aProbabilityDensityFunction binWidth: aNumber count: anInteger 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	probabilityDensityFunction := aProbabilityDensityFunction.
	binWidth := aNumber.
	count := anInteger.
	^self! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	^probabilityDensityFunction parameters copyWith: count! !

!DhbScaledProbabilityDensityFunction methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	super printOn: aStream.
	aStream nextPut: $[;
			nextPutAll: probabilityDensityFunction class distributionName;
			nextPut: $].! !

!DhbScaledProbabilityDensityFunction methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setCount: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	count := aNumber.! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
value: aNumber 
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	^(probabilityDensityFunction value: aNumber) * binWidth * count! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
valueAndGradient: aNumber 
	"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "

	| g temp |
	g := probabilityDensityFunction valueAndGradient: aNumber.
	temp := binWidth * count.
	^Array with: g first * temp
		   with: ( (g last collect: [:each | each * temp]) copyWith: g first * binWidth)! !

Object subclass: #DhbSeriesTermServer
	instanceVariableNames: 'x lastTerm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbSeriesTermServer subclass: #DhbIncompleteBetaFractionTermServer
	instanceVariableNames: 'alpha1 alpha2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteBetaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialTerm
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	^1! !

!DhbIncompleteBetaFractionTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setParameter: aNumber1 second: aNumber2 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	alpha1 := aNumber1.
	alpha2 := aNumber2! !

!DhbIncompleteBetaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
termsAt: anInteger 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	| n n2 |
	n := anInteger // 2.
	n2 := 2 * n.
	^Array with: ( n2 < anInteger 
		ifTrue: 
			[x negated * (alpha1 + n) * (alpha1 + alpha2 + n) 
				/ ((alpha1 + n2) * (alpha1 + 1 + n2))]
		ifFalse: [x * n * (alpha2 - n) / ((alpha1 + n2) * (alpha1 - 1 + n2))])
			with: 1! !

DhbSeriesTermServer subclass: #DhbIncompleteGammaFractionTermServer
	instanceVariableNames: 'alpha'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteGammaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialTerm
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	lastTerm := x - alpha + 1.
	^lastTerm! !

!DhbIncompleteGammaFractionTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setParameter: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	alpha := aNumber asFloat! !

!DhbIncompleteGammaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
termsAt: anInteger 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	lastTerm := lastTerm + 2.
	^Array with: (alpha - anInteger) * anInteger with: lastTerm! !

DhbSeriesTermServer subclass: #DhbIncompleteGammaSeriesTermServer
	instanceVariableNames: 'alpha sum'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteGammaSeriesTermServer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialTerm
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	lastTerm := 1 / alpha.
	sum := alpha.
	^lastTerm! !

!DhbIncompleteGammaSeriesTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setParameter: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	alpha := aNumber asFloat! !

!DhbIncompleteGammaSeriesTermServer methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
termAt: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	sum := sum + 1.
	lastTerm := lastTerm * x / sum.
	^lastTerm! !

!DhbSeriesTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
setArgument: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	x := aNumber asFloat.! !

Object subclass: #DhbStatisticalMoments
	instanceVariableNames: 'moments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbStatisticalMoments subclass: #DhbFastStatisticalMoments
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFastStatisticalMoments methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| var |
	var := 1.
	1 to: moments size
		do: 
			[:n | 
			moments at: n put: (moments at: n) + var.
			var := var * aNumber]! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	self count = 0 ifTrue: [^nil].
	^(moments at: 2) / self count! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| var x1 x2 x3 x4 kFact kConst n m4 xSquared |
	n := self count.
	n < 4 ifTrue: [^nil].
	var := self variance.
	var = 0 ifTrue: [^nil].
	x1 := (moments at: 2) / n.
	x2 := (moments at: 3) / n.
	x3 := (moments at: 4) / n.
	x4 := (moments at: 5) / n.
	xSquared := x1 squared.
	m4 := x4 - (4 * x1 * x3) + (6 * x2 * xSquared) - (xSquared squared * 3).
	kFact := n * (n + 1) / (n - 1) / (n - 2) / (n - 3).
	kConst := 3 * (n - 1) * (n - 1) / (n - 2) / (n - 3).
	^kFact * (m4 * n / var squared) - kConst! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| x1 x2 x3 n stdev |
	n := self count.
	n < 3 ifTrue: [^nil].
	stdev := self standardDeviation.
	stdev = 0 ifTrue: [^nil].
	x1 := (moments at: 2) / n.
	x2 := (moments at: 3) / n.
	x3 := (moments at: 4) / n.
	^(x3 - (3 * x1 * x2) + (2 * x1 * x1 * x1)) * n * n 
		/ (stdev squared * stdev * (n - 1) * (n - 2))! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
unnormalizedVariance
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "

	^(moments at: 3) - ((moments at: 2) squared * self count)! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| n |
	n := self count.
	n < 2 ifTrue: [^nil].
	^((moments at: 3) - ((moments at: 2) squared / n)) / (n - 1)! !

DhbStatisticalMoments subclass: #DhbFixedStatisticalMoments
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFixedStatisticalMoments class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "
	^super new: 4! !

!DhbFixedStatisticalMoments class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbFixedStatisticalMoments methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| correction n n1 c2 c3 |
	n := moments at: 1.
	n1 := n + 1.
	correction := ((moments at: 2) - aNumber) / n1.
	c2 := correction squared.
	c3 := c2 * correction.
	moments
		at: 5
			put: ((moments at: 5) + ((moments at: 4) * correction * 4) 
					+ ((moments at: 3) * c2 * 6) + (c2 squared * (n squared * n + 1))) 
					* n / n1;
		at: 4
			put: ((moments at: 4) + ((moments at: 3) * correction * 3) 
					+ (c3 * (1 - n squared))) * n 
					/ n1;
		at: 3 put: ((moments at: 3) + (c2 * (1 + n))) * n / n1;
		at: 2 put: (moments at: 2) - correction;
		at: 1 put: n1! !

!DhbStatisticalMoments class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "
	^self new: 4! !

!DhbStatisticalMoments class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: anInteger
		"anInteger is the degree of the highest central moments
		 accumulated within the created instance.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "
	^super new initialize: anInteger + 1! !

!DhbStatisticalMoments methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	| correction n n1 oldSums pascal nTerm cTerm term |
	n := moments at: 1.
	n1 := n + 1.
	correction := ((moments at: 2) - aNumber) / n1.
	oldSums := moments copyFrom: 1 to: moments size.
	moments
		at: 1 put: n1;
		at: 2 put: (moments at: 2) - correction.
	pascal := Array new: moments size.
	pascal atAllPut: 0.
	pascal
		at: 1 put: 1;
		at: 2 put: 1.
	nTerm := -1.
	cTerm := correction.
	n1 := n / n1.
	n := n negated.
	3 to: moments size
		do: 
			[:k | 
			cTerm := cTerm * correction.
			nTerm := n * nTerm.
			term := cTerm * (1 + nTerm).
			k to: 3
				by: -1
				do: 
					[:l | 
					pascal at: l put: (pascal at: l - 1) + (pascal at: l).
					term := (pascal at: l) * (oldSums at: l) + term.
					oldSums at: l put: (oldSums at: l) * correction].
			pascal at: 2 put: (pascal at: 1) + (pascal at: 2).
			moments at: k put: term * n1]! !

!DhbStatisticalMoments methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asWeightedPoint: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^DhbWeightedPoint point: aNumber @ self average error: self errorOnAverage! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	self count = 0 ifTrue: [^nil].
	^moments at: 2! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
centralMoment: anInteger 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/6/99 "

	^moments at: anInteger + 1! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
count
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/6/99 "

	^moments at: 1! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
errorOnAverage
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 02-Jan-00 "
	^( self variance / self count) sqrt! !

!DhbStatisticalMoments methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fConfidenceLevel: aStatisticalMomentsOrHistogram 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	| fValue |
	fValue := self variance/ aStatisticalMomentsOrHistogram variance.
	^fValue < 1
		ifTrue: [ (DhbFisherSnedecorDistribution degreeOfFreedom: aStatisticalMomentsOrHistogram count
						degreeOfFreedom: self count) 
										confidenceLevel: fValue reciprocal]
		ifFalse:[ (DhbFisherSnedecorDistribution degreeOfFreedom: self count
						degreeOfFreedom: aStatisticalMomentsOrHistogram count) 
										confidenceLevel: fValue]! !

!DhbStatisticalMoments methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anInteger 
	"Private - ( anInteger - 1) is the degree of the highest accumulated central moment.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	moments := Array new: anInteger.
	self reset.
	^self! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
kurtosis
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	| n n1 n23 |
	n := self count.
	n < 4 ifTrue: [^nil].
	n23 := (n - 2) * (n - 3).
	n1 := n - 1.
	^((moments at: 5) * n squared * (n + 1) / (self variance squared * n1) 
		- (n1 squared * 3)) / n23! !

!DhbStatisticalMoments methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	moments atAllPut: 0! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
skewness
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	| n v |
	n := self count.
	n < 3 ifTrue: [^nil].
	v := self variance.
	^(moments at: 4) * n squared / ((n - 1) * (n - 2) * (v sqrt * v))! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
standardDeviation
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "
	^self variance sqrt! !

!DhbStatisticalMoments methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
tConfidenceLevel: aStatisticalMomentsOrHistogram
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	| sbar dof |
	dof := self count + aStatisticalMomentsOrHistogram count - 2.
	sbar := ( ( self unnormalizedVariance + aStatisticalMomentsOrHistogram unnormalizedVariance) / dof) sqrt.
	^( DhbStudentDistribution degreeOfFreedom: dof)
		confidenceLevel: ( self average - (aStatisticalMomentsOrHistogram average))
							/ ( ( 1 / self count + ( 1 / aStatisticalMomentsOrHistogram count)) sqrt * sbar)! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
unnormalizedVariance
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	^( self centralMoment: 2) * self count! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
variance
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "
	| n |
	n := self count.
	n < 2
		ifTrue: [ ^nil].
	^self unnormalizedVariance / ( n - 1)! !

Object subclass: #DhbVectorAccumulator
	instanceVariableNames: 'count average'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbVectorAccumulator subclass: #DhbCovarianceAccumulator
	instanceVariableNames: 'covariance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCovarianceAccumulator methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: anArray
		"Accumulate anArray into the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	| delta count1 r|
	count1 := count.
	delta := super accumulate: anArray.
	r := count1 / count.
	1 to: delta size
		do: [ :n |
			  1 to: n do:
				  [ :m |
					( covariance at: n) at: m put: ( count1 * ( delta at: n) * ( delta at: m) + ( r * ( ( covariance at: n) at: m))).
				  ].
			].! !

!DhbCovarianceAccumulator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
covarianceMatrix
		"Answer a matrix containing the covariance of the accumulated data.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	| rows n |
	n := 0.
	rows := covariance collect:
				[ :row | n := n + 1. row, ( ( ( n + 1) to: covariance size) collect: [ :m | ( covariance at: m) at: n ])].
	^DhbSymmetricMatrix rows: rows! !

!DhbCovarianceAccumulator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anInteger
		"Private - Initialize the receiver to accumulate vectors of dimension anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	covariance := ( ( 1 to: anInteger) collect: [ :n | DhbVector new: n]) asVector.
	^super initialize: anInteger! !

!DhbCovarianceAccumulator methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset
		"Set all accumulators to zero.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	super reset.
	covariance do: [ :each | each atAllPut: 0].! !

!DhbVectorAccumulator class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self new initialize: anInteger! !

!DhbVectorAccumulator methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
accumulate: aVectorOrArray
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	| delta |
	count := count + 1.
	delta := average - aVectorOrArray asVector scaleBy: 1 / count.
	average accumulateNegated: delta.
	^delta! !

!DhbVectorAccumulator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
average
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^average! !

!DhbVectorAccumulator methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
count
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^count! !

!DhbVectorAccumulator methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	average := DhbVector new: anInteger.
	self reset.
	^self! !

!DhbVectorAccumulator methodsFor: 'display' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	super printOn: aStream.
	aStream space.
	count printOn: aStream.
	aStream space.
	average printOn: aStream.! !

!DhbVectorAccumulator methodsFor: 'transformation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reset
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	count := 0.
	average atAllPut: 0.! !

Object subclass: #DhbWeightedPoint
	instanceVariableNames: 'xValue yValue weight error'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
point: aPoint
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^self new initialize: aPoint weight: 1! !

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
point: aNumber count: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^self point: aNumber @ anInteger
		weight: ( anInteger > 0 ifTrue: [ 1 / anInteger]
								ifFalse:[ 1])! !

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
point: aPoint error: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^self new initialize: aPoint error: aNumber! !

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
point: aPoint weight: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^self new initialize: aPoint weight: aNumber! !

!DhbWeightedPoint methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
chi2ComparisonContribution: aWeightedPoint
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^(aWeightedPoint yValue - yValue) squared / ( 1 / aWeightedPoint weight + ( 1 / weight))! !

!DhbWeightedPoint methodsFor: 'information' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
chi2Contribution: aFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^(yValue - ( aFunction value: xValue)) squared * weight! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
error
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	error isNil
		ifTrue: [ error := 1 / weight sqrt].
	^error! !

!DhbWeightedPoint methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aPoint error: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	error := aNumber.
	^self initialize: aPoint weight: 1 / aNumber squared! !

!DhbWeightedPoint methodsFor: 'initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize: aPoint weight: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	xValue := aPoint x.
	yValue := aPoint y.
	weight := aNumber.
	^self! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
point
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^xValue @ yValue! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
weight
	^weight! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
xValue
	^xValue! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
yValue
	^yValue! !

Object subclass: #KDTree
	instanceVariableNames: 'left right dim value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-KDTree'!

!KDTree commentStamp: 'WernerKassens 5/5/2011 15:45' prior: 0!
KDTree is a space partitioning data structure to store points of that space with "KDTree withAll: aCollectionOfVectors", where a vector can be any collection of numbers that understands "at:". you can then find nearest neighbours of aVector with "nnSearch: aVector i: numberOfNearestNeighbours".   !

KDTree subclass: #IndexedKDTree
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-KDTree'!

!IndexedKDTree commentStamp: 'WernerKassens 7/20/2011 13:48' prior: 0!
IndexedKDTree returns the indices of the nearest neighbours instead of the nearest neighbours itself.!

!IndexedKDTree class methodsFor: 'instance creation' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
withAll: aCollectionOfCollections
"make a KDTree from aCollectionOfCollections ,which is a Collection of points in a n-dimensional space"
|c|
c :=aCollectionOfCollections withIndexCollect: [:v :i| {i.v.}]  .
^super withAll: c.  ! !

!IndexedKDTree methodsFor: 'initialize' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
addData: anOrderedCollection at: dimension
|sorted cut nextdim|
dim :=dimension . 
anOrderedCollection size =1 ifTrue: [value :=anOrderedCollection first . index :=value first. value :=value at: 2. ^self ].
sorted :=anOrderedCollection sort:[:a :b| ((a at: 2)  at: dimension) < ((b at: 2) at: dimension) ].
cut :=sorted size + 1// 2.
value :=sorted at: cut .
index :=value first. value :=value at: 2. 
nextdim :=dimension  \\(value size) +1.
cut >1 ifTrue: [left :=self class new addData: (sorted copyFrom: 1 to: cut -1 )  at: nextdim].
right :=self class new addData: (sorted copyFrom: cut +1 to: sorted size)  at: nextdim.! !

!IndexedKDTree methodsFor: 'evaluating' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
nnSearch: aCollection i: anInt
"search for i nearest neighbours of vector aCollection and return the indices; and distances (if i>1, at second position)"
|n|
n :=NNStore new: anInt .
self nnSearch: aCollection near: n .
^anInt =1 ifTrue: [n data first first ] ifFalse:   [n completeData collect: [:i | {(i at:2)first . i first}]  ]  ! !

!IndexedKDTree methodsFor: 'evaluating' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
nnSearch: aCollection near: aNNStore
|dist|
(self nearChild: aCollection)ifNotNilDo:[:c| c nnSearch: aCollection near: aNNStore ].
(aNNStore maxDistance)<( (dist := (value at: dim) - (aCollection at: dim) ) * dist ) ifTrue:[^self ].
dist := ((dist:= aCollection - value)*dist )sum.
(aNNStore maxDistance >dist ) ifTrue: [aNNStore add: {dist . {index . value}} ] .
(self farChild: aCollection)ifNotNilDo:[:c| c nnSearch: aCollection  near: aNNStore].! !

!KDTree class methodsFor: 'instance creation' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
withAll: aCollectionOfCollections
"make a KDTree from aCollectionOfCollections ,which is a Collection of points in a n-dimensional space"
(aCollectionOfCollections allSatisfy: [:c|c isCollection ])ifFalse: 
	[self error: 'KDTree withAll: expects a collection of collections'].    
^self new addData: aCollectionOfCollections asOrderedCollection at:1.     ! !

!KDTree methodsFor: 'initialize' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
addData: anOrderedCollection at: dimension
|sorted cut nextdim|
dim :=dimension . 
anOrderedCollection size =1 ifTrue: [value :=anOrderedCollection first . ^self ].
sorted :=anOrderedCollection sort:[:a :b| (a at: dimension) < (b at: dimension) ].
cut :=sorted size + 1// 2.
value :=sorted at: cut .
nextdim :=dimension  \\(value size) +1.
cut >1 ifTrue: [left :=self class new addData: (sorted copyFrom: 1 to: cut -1 )  at: nextdim].
right :=self class new addData: (sorted copyFrom: cut +1 to: sorted size)  at: nextdim.! !

!KDTree methodsFor: 'accessing' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
farChild: aVector
^((value at: dim) < (aVector at: dim)) ifTrue: [left ] ifFalse:   [right ]! !

!KDTree methodsFor: 'accessing' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
left 
^left ! !

!KDTree methodsFor: 'accessing' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
nearChild: aVector
^((value at: dim) < (aVector at: dim)) ifTrue: [right ] ifFalse:   [left ]! !

!KDTree methodsFor: 'evaluating' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
nnSearch: aCollection i:anInt
"search for i nearest neighbours of vector aCollection "
|n|
n :=NNStore new: anInt .
self nnSearch: aCollection near: n .
n :=n data .
^anInt =1 ifTrue: [n first ] ifFalse:   [n]  ! !

!KDTree methodsFor: 'evaluating' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
nnSearch: aCollection near: aNNStore
|dist|
(self nearChild: aCollection)ifNotNilDo:[:c| c nnSearch: aCollection near: aNNStore ].
(aNNStore maxDistance)<( (dist := (value at: dim) - (aCollection at: dim) ) * dist ) ifTrue:[^self ].
dist := ((dist:= aCollection - value)*dist )sum.
(aNNStore maxDistance >dist ) ifTrue: [aNNStore add: {dist . value} ] .
(self farChild: aCollection)ifNotNilDo:[:c| c nnSearch: aCollection  near: aNNStore].! !

!KDTree methodsFor: 'printing' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
printOn: aStream
	aStream nextPutAll:  'KDTree (dim: ';
	 print: dim ;
	 nextPutAll:  ' value: ';
	 print: value ;
	 nextPutAll: ' left: ';
	 print: left ;
	 nextPutAll: ' right: '; 
	 print: right ;
	 nextPut: $) .! !

!KDTree methodsFor: 'accessing' stamp: 'wernerkassens 5/13/2012 01:18:33.479'!
right  
^right  ! !

Object subclass: #NumberGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!NumberGenerator commentStamp: '<historical>' prior: 0!
A NumberGenerator is a stream of numbers. All NumberGenerators respond to #next.!

NumberGenerator subclass: #BernoulliGenerator
	instanceVariableNames: 'p rng'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!BernoulliGenerator commentStamp: '<historical>' prior: 0!
A BernoulliGenerator is simulates a Bernoulli Process. This is a discrete process, with probability of p for success, and 1-p for failure. 

next
	answer 1 if success event, 0 otherwise
	
generator:
	provide a uniform [0,1] random number generator
	
p: 
	set the probability of success events
	
class methods

withProbability:
	create a generator with probability set to p
	
defaultGenerator
	class used for generator in new instances!

!BernoulliGenerator class methodsFor: 'instance creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
defaultGenerator
	^ PMRandom! !

!BernoulliGenerator class methodsFor: 'instance creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
fair
	^ self withProbability: 0.5! !

!BernoulliGenerator class methodsFor: 'private' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
validProbability: aProbability
"reject probabilities outside of [0,1]"
	self assert: [(aProbability < 0) not and: (aProbability > 1) not].! !

!BernoulliGenerator class methodsFor: 'instance creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
withProbability: p
	self validProbability: p.
	^ (self new) p: p.! !

!BernoulliGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/12/2012 21:27'!
generator
	^ rng ! !

!BernoulliGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
generator: aRandomGenerator
	rng := aRandomGenerator.! !

!BernoulliGenerator methodsFor: 'initialize-release' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
	super initialize.
	self generator: self class defaultGenerator new.
	^ self.! !

!BernoulliGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
	(rng next < p) 
		ifTrue: [^1]
		ifFalse:[^0].! !

!BernoulliGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
p: aProbability
	BernoulliGenerator validProbability: aProbability.
	p := aProbability.! !

NumberGenerator subclass: #BinomialGenerator
	instanceVariableNames: 'n p rng'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!BinomialGenerator commentStamp: '<historical>' prior: 0!
A BinomialGenerator yields results from a binomial distribution with probability p and n trials. The generator is the underlying random source.

If each independent event has probability 0<p<1, and n trials are performed, next returns number of successes.!

!BinomialGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
generator: aRandomNumberGenerator 
	rng := aRandomNumberGenerator! !

!BinomialGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
n: numberOfTrials 
	n := numberOfTrials! !

!BinomialGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
	| x |
	x := 0.
	n timesRepeat: [ rng next <= p ifTrue: [ x := x + 1 ] ].
	^ x! !

!BinomialGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
p: aProbability 
	p := aProbability! !

NumberGenerator subclass: #ConstantGenerator
	instanceVariableNames: 'constant'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!ConstantGenerator commentStamp: 'SergeStinckwich 3/30/2010 15:31' prior: 0!
A ConstantGenerator is not really a random number generator.!

!ConstantGenerator class methodsFor: 'as yet unclassified' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
constant: aNumber
	^ self new constant: aNumber ! !

!ConstantGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
constant: aConstant
	constant := aConstant! !

!ConstantGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
	^ constant! !

NumberGenerator subclass: #GaussianGenerator
	instanceVariableNames: 'rng mean standardDeviation hasNextGaussian nextGaussian'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!GaussianGenerator commentStamp: '<historical>' prior: 0!
A GaussianGenerator is the Normal Distribution.

Instance Variables
	hasNextGaussian:		<Object>
	mean:		<Object>
	nextGaussian:		<Object>
	rng:		<Object>
	standardDeviation:		<Object>

hasNextGaussian
	- xxxxx

mean
	- xxxxx

nextGaussian
	- xxxxx

rng
	- xxxxx

standardDeviation
	- xxxxx
!

!GaussianGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
generator: aRandomNumberGenerator 
	rng := aRandomNumberGenerator! !

!GaussianGenerator methodsFor: 'initialize-release' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
	hasNextGaussian := false! !

!GaussianGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
mean: aMeanValue
	mean := aMeanValue! !

!GaussianGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
	^ self nextGaussianNormalized * standardDeviation + mean! !

!GaussianGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextGaussianNormalized
	| x1 x2 w y1 y2 |
	hasNextGaussian
		ifTrue: [ 
			hasNextGaussian := false.
			^ nextGaussian ].
	[ 
	x1 := 2.0 * rng nextFloat - 1.0.
	x2 := 2.0 * rng nextFloat - 1.0.
	w := x1 * x1 + (x2 * x2).
	w >= 1.0 ] whileTrue.
	w := (-2.0 * w ln / w) sqrt.
	y1 := x1 * w.
	y2 := x2 * w.
	nextGaussian := y2.
	^ y1! !

!GaussianGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
standardDeviation: aStandardDeviation
	standardDeviation := aStandardDeviation! !

!NumberGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
	^ self subclassResponsibility! !

NumberGenerator subclass: #PoissonGenerator
	instanceVariableNames: 'rate rng nextMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!PoissonGenerator commentStamp: '<historical>' prior: 0!
A PoissonGeneratorsimulates a Poisson Process with parameter rate.!

!PoissonGenerator class methodsFor: 'as yet unclassified' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
thresholdForApproximation
	"if rate is larger than this threshhold, GaussianApproximation will be used."
	^ 40! !

!PoissonGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:41'!
determineNextMethod
		nextMethod :=  (rate < self class thresholdForApproximation)
		ifTrue: [  #nextKnuth]
		ifFalse: [ #nextGaussian].! !

!PoissonGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
generator: aRandomGenerator
	rng := aRandomGenerator .! !

!PoissonGenerator methodsFor: 'as yet unclassified' stamp: 'DanielUber 5/10/2012 23:39'!
next
	^ (nextMethod cull: self).
	! !

!PoissonGenerator methodsFor: 'as yet unclassified' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextGaussian
	"For large rate, the poisson is approximable by a Gaussian Distribution "
	^ ((GaussianGenerator new ) 
		mean: rate; 
		standardDeviation:  rate sqrt; 
		generator: rng;
		next) asInteger.! !

!PoissonGenerator methodsFor: 'as yet unclassified' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextKnuth
"appropriate for small rate. This method is linear in rate."
	| L k p |
	
	L := Float e raisedTo: rate negated.
	k := 0.
	p := 1.
	[p > L] whileTrue:
		[k := k + 1.
		p := p * rng next].
	^ k - 1.
! !

!PoissonGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
rate
	^ rate! !

!PoissonGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:40'!
rate: anObject
	rate := anObject.
	self determineNextMethod.! !

!Object methodsFor: '*Math-Quaternion-testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isQuaternion
	"Answer true if receiver is a Quaternion. False by default."

	^ false
! !

Object subclass: #PointSeries
	instanceVariableNames: 'points'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!PointSeries class methodsFor: 'creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
	"Create a new instance and initialize it"
	^super new initialize! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
add: anObject
	"Add a point to the receiver"
	self primitiveAdd: anObject.
	self changed: self changedSymbol.
	^anObject! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
at: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^points at: anInteger! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
changedSymbol
	"Answers the symbol of the event sent when the points of the receiver change"
	^#pointsChanged! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
collectPoints: aBlock
	^points collect: aBlock! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
do: aBlock
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	self pointsDo: aBlock.! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
firstPoint
	"Answers the first point stored in the receiver"
	^self at: 1! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
	"Create the point collection"
	points := SortedCollection sortBlock: self sortBlock.
	^self! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isEmpty
		"
		 (c) Copyrights Didier BESSET, 1998, all rights reserved.
		 Initial code: 28/9/98 "
	^points isEmpty! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
notEmpty
	^points notEmpty! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
pointCollection
		" Answer the collection of points.
		 (c) Copyrights Didier BESSET, 1998, all rights reserved.
		 Initial code: 28/9/98 "
	^self collectPoints: [ :each | each]! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
pointsDo: aBlock
	points do: aBlock.! !

!PointSeries methodsFor: 'privateMethods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
primitiveAdd: aPoint
	"Private - Add a point to the receiver"
	points add: aPoint.! !

!PointSeries methodsFor: 'privateMethods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
primitiveRemove: aPoint
	"Private - Removes a point from the receiver"
	points remove: aPoint.! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
remove: anObject
	"Add a point to the receiver"
	self primitiveRemove: anObject.
	self changed: self changedSymbol.
	^anObject! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
size
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^points size! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sort
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	points := ( points asSortedCollection: self sortBlock) asOrderedCollection.! !

!PointSeries methodsFor: 'privateMethods' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sortBlock
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^[ :a :b | a x < b x]! !

Object subclass: #Quaternion
	instanceVariableNames: 'qr qi qj qk'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Quaternion'!

!Quaternion commentStamp: '<historical>' prior: 0!
Quaternion are to 3D rotations what Complex are to plane rotations.

They are formed with one real part and three unreal parts.
We will note the real part qr, and the unreal part qi*i + qj*j + qk*k, where :
	i*i=j*j=k*k=-1
	i*j=-j*i=k
	j*k=-k*j=i
	k*i=-i*k=j

A 3D rotation of angle theta around axis [u v w] can be associated to quaternion :
	cos(theta/2) + sin(theta/2) * (u i + v j + w k)
Such a quaternion is unitary (have a unit norm)
Product of quaternion is equivalent to product of rotation matrix.
Quaternion conjugate is equivalent to rotation matrix transpose.

A quaternion can also be decomposed into two complex numbers
(qr i: qi) + (qj i: qk) j

Instance Variables:
	qr	<Number>	real part
	qi	<Number>	first unreal part
	qj	<Number>	second unreal part
	qk	<Number>	third unreal part

!

!Quaternion class methodsFor: 'constants access' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
one
	^self 
		qr: 1
		qi: 0
		qj: 0
		qk: 0! !

!Quaternion class methodsFor: 'instance creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
qr: qr qi: qi qj: qj qk: qk 
	^self basicNew 
		qr: qr
		qi: qi
		qj: qj
		qk: qk! !

!Quaternion class methodsFor: 'constants access' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
zero
	^self 
		qr: 0
		qi: 0
		qj: 0
		qk: 0! !

!Quaternion methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
* aQuaternion 
	"Answer the result of multiplying self with aQuaternion.
	Distribute the product (qr + qi i +qj j + qk k)*(tr+ti i + tj j + tk k)
	with rules i*i=j*j=k*k=-1 and i*j=-k, j*k=-i, k*i=-j"
	^ aQuaternion isQuaternion
		ifTrue: [| tr ti tj tk | 
			tr := aQuaternion qr.
			ti := aQuaternion qi.
			tj := aQuaternion qj.
			tk := aQuaternion qk.
			^ self species
				qr: qr * tr - (qi * ti) - (qj * tj) - (qk * tk)
				qi: qr * ti + (qi * tr) + (qj * tk) - (qk * tj)
				qj: qr * tj + (qj * tr) + (qk * ti) - (qi * tk)
				qk: qr * tk + (qk * tr) + (qi * tj) - (qj * ti)]
		ifFalse: [(aQuaternion isNumber and: [aQuaternion isComplex not])
				ifTrue: ["handle case much faster than adapt..."
					self species
						qr: qr * aQuaternion
						qi: qi * aQuaternion
						qj: qj * aQuaternion
						qk: qk * aQuaternion]
				ifFalse: [aQuaternion adaptToQuaternion: self andSend: #*]]! !

!Quaternion methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
+ aQuaternion 
	^ aQuaternion isQuaternion
		ifTrue: [self species
				qr: qr + aQuaternion qr
				qi: qi + aQuaternion qi
				qj: qj + aQuaternion qj
				qk: qk + aQuaternion qk]
		ifFalse: [aQuaternion adaptToQuaternion: self andSend: #+]! !

!Quaternion methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
- aQuaternion 
	^ aQuaternion isQuaternion
		ifTrue: [self species
				qr: qr - aQuaternion qr
				qi: qi - aQuaternion qi
				qj: qj - aQuaternion qj
				qk: qk - aQuaternion qk]
		ifFalse: [aQuaternion adaptToQuaternion: self andSend: #-]! !

!Quaternion methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
/ aQuaternion
	^self * aQuaternion reciprocal! !

!Quaternion methodsFor: 'comparing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
= aQuaternion 
	"Answer true if both quaternion are equals.
	A quaternion might also equal a Complex or a Number or other kind of numbers.
	Implementation note: only work if Complex Octonion or other kind answer true to isNumber."

	^ aQuaternion isQuaternion
		ifTrue: [qi = aQuaternion qi
				and: [qj = aQuaternion qj
						and: [qk = aQuaternion qk
								and: [qr = aQuaternion qr]]]]
		ifFalse: [aQuaternion isNumber
				and: [aQuaternion adaptToQuaternion: self andSend: #=]]! !

!Quaternion methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
abs
	"Return the magnitude (or absolute value) of the quaternion.
	Care for overflow or underflow"

	| scale |
	scale := (qr abs max: qi abs) max: (qj abs max: qk abs).
	^scale isZero 
		ifTrue: [scale]
		ifFalse: 
			[(self species 
				qr: qr / scale
				qi: qi / scale
				qj: qj / scale
				qk: qk / scale) squaredNorm sqrt * scale]! !

!Quaternion methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToComplex: rcvr andSend: selector 
	"If I am involved in arithmetic with a Complex, convert it to a Quaternion."
	^ rcvr asQuaternion perform: selector with: self! !

!Quaternion methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToFloat: rcvr andSend: selector 
	"If I am involved in arithmetic with a Float, convert it to a Quaternion."
	^ rcvr asQuaternion perform: selector with: self! !

!Quaternion methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToFraction: rcvr andSend: selector 
	"If I am involved in arithmetic with a Fraction, convert it to a Quaternion."
	^ rcvr asQuaternion perform: selector with: self! !

!Quaternion methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
adaptToInteger: rcvr andSend: selector 
	"If I am involved in arithmetic with an Integer, convert it to a Quaternion."
	^ rcvr asQuaternion perform: selector with: self! !

!Quaternion methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/11/2012 18:07'!
addPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99
		
		added to Quaternion 11 May 2012 Daniel Uber
		 "
	^aPolynomial addNumber: self! !

!Quaternion methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
angle
	"answer the rotation angle associated with the receiver"

	| sinThetaSur2 cosThetaSur2 |
	sinThetaSur2 := self unreal abs.
	cosThetaSur2 := qr.
	^(sinThetaSur2 arcTan: cosThetaSur2) * 2! !

!Quaternion methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
angleInDegrees
	"answer the rotation angle in degrees associated with the receiver"

	^self angle radiansToDegrees! !

!Quaternion methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
asQuaternion
	"Convert the receiver into a quaternion.
	Do nothing, i am already a quaternion"

	^self! !

!Quaternion methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
conjugated
	"Return the conjugate of this quaternion number."

	^self species 
		qr: qr
		qi: qi negated
		qj: qj negated
		qk: qk negated! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
cos
	"Answer the receiver cosine"

	| z w |
	z := self unreal abs.
	w := z isZero
		ifTrue: [z]
		ifFalse: [qr sin negated * z sinh / z].
	^(self species 
				qr: qr cos * z cosh
				qi: qi * w
				qj: qj * w
				qk: qk * w)! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
cosh
	"Answer the receiver hyperbolic cosine"

	^(self exp + self negated exp) / 2! !

!Quaternion methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/11/2012 18:10'!
dividingPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: (1 / self)! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
exp
	"Answer the receiver exponential"

	| z w |
	z := self unreal abs.
	w := z isZero
		ifTrue: [z]
		ifFalse: [z sin / z].
	^qr exp * (self species 
				qr: z cos
				qi: qi * w
				qj: qj * w
				qk: qk * w)! !

!Quaternion methodsFor: 'private' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
floatClass
	"Answer the class suitable for doing floating point operations.
	In default Squeak, this is Float.
	In an image with single and double IEEE 754 floating point numbers,
	this would depend on the class of real and imaginary parts"

	^Float! !

!Quaternion methodsFor: 'comparing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
hash
	^(((qi hash bitXor: qj hash) bitXor: qk hash) bitXor: qr hash)! !

!Quaternion methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
i
	"Answer the product of the receiver with pure imaginary i.
	This is the same as self * 1 i"

	^self species 
		qr: qi negated
		qi: qr
		qj: qk
		qk: qj negated! !

!Quaternion methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isNumber
	"Answer true just like Complex, our unreal part is just a bit more complex.
	Note that this message is needed to make equality tests pass due to Number>>= implementation."

	^true! !

!Quaternion methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isQuaternion
	"always answer true since the receiver is a quaternion"

	^true! !

!Quaternion methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isZero
	"answer true if the receiver is null, false otherwise"

	^qr isZero and: [qi isZero and: [qj isZero and: [qk isZero]]]! !

!Quaternion methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
j
	"Answer the product of the receiver with pure imaginary j.
	This is the same as self * 1 j"


	^self species 
		qr: qj negated
		qi: qk negated
		qj: qr
		qk: qi! !

!Quaternion methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
k
	"Answer the product of the receiver with pure imaginary k.
	This is the same as self * 1 k"


	^self species 
		qr: qk negated
		qi: qj
		qj: qi negated
		qk: qr! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
ln
	"Answer the receiver natural logarithm"

	| z |
	z := self unreal abs.
	^z isZero 
		ifTrue: 
			[self species 
				qr: self abs ln
				qi: (z arcTan: qr)
				qj: 0
				qk: 0]
		ifFalse: 
			[| theta |
			theta := (z arcTan: qr) / z.
			self species 
				qr: self abs ln
				qi: qi * theta
				qj: qj * theta
				qk: qk * theta]! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
log
	"Answer receiver base 10 logarithm."

	^self ln / 10 ln! !

!Quaternion methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
negated
	^self species 
		qr: qr negated
		qi: qi negated
		qj: qj negated
		qk: qk negated! !

!Quaternion methodsFor: 'printing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
printOn: aStream 
	"Print a representation of the receiver onto aStream"

	self storeOn: aStream! !

!Quaternion methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/11/2012 18:12'!
productWithVector: aVector
		"Answers a new vector product of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aVector collect: [ :each | each * self]! !

!Quaternion methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
qi
	^qi! !

!Quaternion methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
qj
	^qj! !

!Quaternion methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
qk
	^qk! !

!Quaternion methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
qr
	^qr! !

!Quaternion methodsFor: 'private' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
qr: a0 qi: a1 qj: a2 qk: a3 
	qr := a0.
	qi := a1.
	qj := a2.
	qk := a3! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
raisedTo: aNumber 
	"Answer the receiver raised to aNumber."

	aNumber isInteger ifTrue:
		["Do the special case of integer power"
		^ self raisedToInteger: aNumber].
	
	0 = aNumber ifTrue: [^ self class one].	"Special case of exponent=0"
	1 = aNumber ifTrue: [^ self].	"Special case of exponent=1"
	0 = self ifTrue: [				"Special case of self = 0"
		aNumber < 0
			ifTrue: [^ (ZeroDivide dividend: self) signal]
			ifFalse: [^ self]].
	^ (aNumber * self ln) exp		"Otherwise use logarithms"! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
raisedToInteger: operand 
	"Answer the receiver raised to the power operand, an Integer."

	"implementation note: this code is copied from Number.
	This suggest that both Number and Quaternion should have an
	ArithmeticValue common superclass like in Visualworks.
	Or maybe should it be a Traits (a property of fields ?)"

	| count result |
	operand isInteger ifFalse: [^ ArithmeticError signal: 'parameter is not an Integer'].
	operand = 0 ifTrue: [^ self class one].
	operand = 1 ifTrue: [^ self].
	operand < 0 ifTrue: [^ (self raisedToInteger: operand negated) reciprocal].
	count := 1 bitShift: (operand-1) highBit.
	result := self class one.
	[count > 0]
		whileTrue: 
			[result := result * result.
			(operand bitAnd: count)
				= 0 ifFalse: [result := result * self].
			count := count bitShift: -1].
	^ result! !

!Quaternion methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
real
	"answer the real part of the receiver"

	^qr! !

!Quaternion methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reciprocal
	"Trivial algorithm:
		^self conjugated / self squaredNorm
	may overflow"

	| qn qrn qin qjn qkn qn2 |
	qn := (qr abs max: qi abs) max: (qj abs max: qk abs).
	qn isZero 
		ifTrue: 
			[^ZeroDivide signal: 'Cannot take reciprocal of null quaternion'].
	qrn := qr / qn.
	qin := qi / qn.
	qjn := qj / qn.
	qkn := qk / qn.
	qn2 := qrn squared + qin squared + qjn squared + qkn squared.
	^(qrn / (qn*qn2) i: qin negated / (qn*qn2) j: qjn negated / (qn*qn2) k: qkn negated / (qn*qn2))! !

!Quaternion methodsFor: 'converting' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
reduce
	"Answer the receiver transformed to a lower generality, if
	such a transformation is possible without losing information.
	If not,answer the receiver"
	^ self unreal isZero
		ifTrue: [qr reduce]
		ifFalse: [(qj isZero
					and: [qk isZero])
				ifTrue: [qr i: qi]
				ifFalse: [self]]! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sin
	"Answer the receiver sine"

	| z w |
	z := self unreal abs.
	w := z isZero
		ifTrue: [z]
		ifFalse: [qr cos * z sinh / z].
	^(self species 
				qr: qr sin * z cosh
				qi: qi * w
				qj: qj * w
				qk: qk * w)! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
sinh
	"Answer the receiver hyperbolic sine"

	^(self exp - self negated exp) / 2! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
squared
	^self * self! !

!Quaternion methodsFor: 'arithmetic' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
squaredNorm
	^qr squared + qi squared + qj squared + qk squared! !

!Quaternion methodsFor: 'printing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
storeOn: aStream 
	"Store a representation of the receiver onto aStream that can be interpreted.
	Two possible forms are:
	a + b i + c j + d k
	a i: b j: c k: d
	Second one is preferred for efficiency"

	aStream nextPut: $(.
	qr storeOn: aStream.
	aStream
		space;
		nextPutAll: #i:;
		space.
	qi storeOn: aStream.
	aStream
		space;
		nextPutAll: #j:;
		space.
	qj storeOn: aStream.
	aStream
		space;
		nextPutAll: #k:;
		space.
	qk storeOn: aStream.
	aStream nextPut: $)! !

!Quaternion methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/11/2012 18:08'!
subtractToPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self negated! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
tan
	"Answer the receiver tangent"

	^self sin / self cos! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
tanh
	"Answer the hyperbolic tangent"

	| ep em |
	ep := self exp.
	em := self negated exp.
	^(ep - em) / (ep + em)! !

!Quaternion methodsFor: '*Math-DHB-Extensions' stamp: 'DanielUber 5/11/2012 18:02'!
timesPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 
		added to Complex class 12 May 2012, Daniel Uber
		"
	^aPolynomial timesNumber: self! !

!Quaternion methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
unreal
	"answer the unreal part of the receiver"

	^self species 
		qr: 0
		qi: qi
		qj: qj
		qk: qk! !

Object subclass: #RandomGenerator
	instanceVariableNames: 'seed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!RandomGenerator commentStamp: 'hfm 3/23/2009 21:02' prior: 0!
This is the top abstract class of the pseudo-random numbers generator.

The applications of PRNG's are many ones:
-Simulations
-Sampling
-Cryptography
-Numeric Analysis
-Games
-Aesthetics

Internally, one can classify RNG's according to several views, so choosing one or another is fully dependent of your requirements:

-From the production view, there is always a tradeoff between:
--Fast or
--Secure

Another view, based on the seed they use, is:
-Deterministic: They take the seed value from a specific number. These ones are called "Pseudo random number generators"
-Non-deterministic: They take the seed value from a physical source non-predictable and outside the human control. These are called "true Random number generators"

Along these terms, deterministic RNG's are divided between:
-Normal PRNG's
--Linear Congruential
--Non-Linear Congruential
-Cryptography safe PRNG's (CSPRNG's)
--DSA
--ECDSA

And finally, there's a taxonomy more related with the internal implementation, based on how the pseudo-generator make its random variables following a distribution function.


!

RandomGenerator subclass: #LCRandom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!LCRandom commentStamp: 'hfm 3/23/2009 21:22' prior: 0!
This uses a quick and dirty Linear Congruential generator.

LCRandom new next.!

!LCRandom methodsFor: 'initialize-release' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
  " Private - Answer the receiver "! !

!LCRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
	"Private - Answer a pseudo-Random floating point number
	between 0 and 1.
	Uses a quick and dirty Linear congruential generator,"
	^ self nextFloat! !

!LCRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextFloat
    "Private - Answer a pseudo-Random floating point number between 0 and 1.
    Uses a quick and dirty Linear congruential generator,"

    seed := self peekSeed.
    ^ seed / 120050.0! !

!LCRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
peek
    "Private - Answer a pseudo-Random floating point number between 0 and 1.
    Uses a simple, but fast, Linear Congruential generator."

    ^ self peekSeed / 120050.0! !

!LCRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
peekSeed
    "Private - Answer the next Integer seed calculated 
	using a quick and dirty Linear congruential generator."

    ^ seed * 2311 + 25367 \\ 120050! !

RandomGenerator subclass: #LehmerRandom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!LehmerRandom commentStamp: 'hfm 3/23/2009 21:22' prior: 0!
This generator uses the Lehmer's Linear Congruential method.

LehmerRandom new next!

!LehmerRandom methodsFor: 'initialize-release' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
  " Private - Answer the receiver "! !

!LehmerRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
    "Private - Answer a Random Float between 0 and 1. 
	Uses Lehmer's linear congruential method."

    ^ self nextFloat! !

!LehmerRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextFloat
    "Private - Answer a Random Float between 0 and 1. Uses Lehmer's linear congruential method."

    seed := self peekSeed.
    ^ seed / SmallInteger maxVal asFloat! !

!LehmerRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
peek
    "Private - Answer a pseudo-Random floating point number between 0 and 1."

    ^ self peekSeed / SmallInteger maxVal asFloat! !

!LehmerRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
peekSeed
    "Private - Answer the next Integer seed calculated using Lehmer's linear congruential
    method. The seed is in the range 0..16r3FFFFFFF."

    ^ ( seed * 31415821 ) + 1 bitAnd: SmallInteger maxVal! !

RandomGenerator subclass: #MersenneTwisterRandom
	instanceVariableNames: 'states mti'
	classVariableNames: 'DefaultLengthVector Mt19937LowerMask Mt19937Mask Mt19937MatrixA Mt19937UpperMask PeriodParameter TemperingMaskB TemperingMaskC'
	poolDictionaries: ''
	category: 'Math-Random'!

!MersenneTwisterRandom commentStamp: 'hfm 3/24/2009 08:27' prior: 0!
See http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html

" To get a pseudo-random float "
MersenneTwisterRandom new next. 

" To get a pseudo-random integer "
MersenneTwisterRandom new nextInteger.

!

!MersenneTwisterRandom class methodsFor: 'class initialization' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
	" self initialize "

	DefaultLengthVector := 624.
	Mt19937LowerMask := 2147483647.
	Mt19937Mask := 4294967295.
	Mt19937MatrixA := 16r9908B0DF.	" 2567483615 "
	Mt19937UpperMask := 16r80000000. " 2147483648 "
	PeriodParameter := 397.
	TemperingMaskB := 16r9D2C5680.	" 2636928640 "
	TemperingMaskC := 16rEFC60000.  " 4022730752 "

! !

!MersenneTwisterRandom methodsFor: 'initialize-release' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
	" Private - Initialize the receiver state vector and multiplier "

	states := Array new: DefaultLengthVector.
	mti := 1.

	states at: 1 put: ( self seed bitAnd: Mt19937Mask ).
		
	2 to: DefaultLengthVector
		do: [: multIndex | 
			states at: multIndex 
				put: ( 16r6C078965 * 
						( ( states at: multIndex - 1 ) 
							bitXor: ( ( states at: multIndex - 1 ) bitShift: 30 negated ) ) + multIndex ).
								
			states at: multIndex 
				put: ( ( states at: multIndex ) bitAnd: Mt19937Mask )
      ]
! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
    "Private - Answer a pseudo-Random float number
	four times faster than rand() ff period 2**19937-1.  "

   ^ self nextFloat
! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextFloat
  " Answer a random float number in the interval [ 0 , 1 ] "

  ^ self nextInteger * ( 1.0 / (4294967295.0 * 2 ))! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextFloat: limit
  " Answer a random float number in the interval [ 0 , limit ] "

  ^ self nextFloat * limit! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextFloatExcludeUpper
  " Answer a random float number in the interval [ 0 , 1 ) "

  ^ self nextInteger * ( 1.0 / 4294967296.0 )! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextFloatExcludeUpper: limit
  " Answer a random float number in the interval [ 0 , limit ) "

  ^ self nextFloatExcludeUpper * limit! !

!MersenneTwisterRandom methodsFor: 'private' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextInteger
	" Answer a random integer on [0,0xffffffff] interval "

	| tempArray y |
	tempArray := Array withAll: #(0 16r9908B0DF).
	mti >= DefaultLengthVector
		ifTrue: [ 
			mti = (DefaultLengthVector + 1)
				ifTrue: [ 
					self seed: 5489.
					self initialize ].
			1 to: DefaultLengthVector - PeriodParameter do: [ :kk | 
				y := ((states at: kk) bitAnd: Mt19937UpperMask) bitOr: ((states at: kk + 1) bitAnd: Mt19937LowerMask).
				states
					at: kk
					put:
						(((states at: kk + PeriodParameter) bitXor: (y bitShift: 1 negated)) bitXor: (tempArray at: (y bitAnd: 16r1) + 1)) ].
			DefaultLengthVector - PeriodParameter + 1 to: DefaultLengthVector - 2 do: [ :kk | 
				y := ((states at: kk) bitAnd: Mt19937UpperMask) bitOr: ((states at: kk + 1) bitAnd: Mt19937LowerMask).
				states
					at: kk
					put:
						(((states at: kk + PeriodParameter - DefaultLengthVector + 1) bitXor: (y bitShift: 1 negated))
							bitXor: (tempArray at: (y bitAnd: 16r1) + 1)) ].
			y := ((states at: DefaultLengthVector) bitAnd: Mt19937UpperMask) bitOr: ((states at: 1) bitAnd: Mt19937LowerMask).
			states
				at: DefaultLengthVector
				put: ((states at: PeriodParameter) bitXor: ((y bitShift: 1) bitXor: (tempArray at: (y bitAnd: 16r1) + 1))).
			mti := 0 ].
	y := states at: mti + 1.
	mti := mti + 1.
	y := y bitXor: (y bitShift: 11 negated).
	y := y bitXor: ((y bitShift: 7 negated) bitAnd: TemperingMaskB).
	y := y bitXor: ((y bitShift: 15 negated) bitAnd: TemperingMaskC).
	y := y bitXor: (y bitShift: 18 negated).
	^ y! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
peek
	"Answer the same as next without changing state"
	
	^ self copy next.
	! !

RandomGenerator subclass: #PMRandom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!PMRandom commentStamp: 'hfm 3/24/2009 09:19' prior: 0!
Uses Park and Miller's 'Minimum Standard' Congruential generator. See 'Numerical Recipes in C', 2nd Edition; Press, Flannery, Teukolsky and Vetterling; Cambridge University Press, 1992.

PMRandom new next!

!PMRandom methodsFor: 'initialize-release' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
  " Private - Answer the receiver "! !

!PMRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
    "Private - Answer a pseudo-Random floating point number between 0 and 1.
    Uses Park and Miller's 'Minimum Standard' congruential generator
    (see 'Numerical Recipes in C', 2nd Edition; Press, Flannery,
    Teukolsky and Vetterling; Cambridge University Press, 1992)."

	^ self nextFloat! !

!PMRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
nextFloat
    "Private - Answer a pseudo-Random floating point number between 0 and 1.
    Uses Park and Miller's 'Minimum Standard' congruential generator
    (see 'Numerical Recipes in C', 2nd Edition; Press, Flannery,
    Teukolsky and Vetterling; Cambridge University Press, 1992)."

    seed := self peekSeed.
    ^ seed / 16r7FFFFFFF asFloat ! !

!PMRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
peek
    "Private - Answer a pseudo-Random floating point number between 0 and 1.
    Uses Park and Miller's 'Minimum Standard' congruential generator."

    ^ self peekSeed / 16r7FFFFFFF asFloat
! !

!PMRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
peekSeed
    "Private - Answer the next Integer seed calculated using Park and Miller's 'Minimum Standard'
    congruential generator. The seed is in the range 0..16r7FFFFFFF."

    ^ ( seed * ( 7 raisedToInteger: 5 ) ) rem: 16r7FFFFFFF
! !

!PMRandom methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
seed: anInteger
    "Set the seed of the Random stream to anInteger (or 1 if zero)."

    seed := anInteger max: 1
! !

!RandomGenerator class methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
chiSquare: range repeating: anInteger
	"Run a 'chi-square' test on a random number generator, 
      over a range of integers given by range, repeating anInteger times.  
      Answer with an Array	 containing the result of the chi-square test 
      (which should be the same as the range), and the upper and lower 
      bounds for 'good' randomness (assuming that anInteger
      is at least 10 * range)."
	"Random chiSquare: 100 repeating: 4000."
	
	| array rand ave |
    array := Array new: range withAll: 0.
	rand := self new.
    anInteger timesRepeat: [  
            | temp |
            temp := (rand next * range) truncated + 1.
     		array at: temp put: (array at: temp) + 1].
			ave := 0.
			array do: [ :each | ave := ave + (each * each)].
			^ Array	with: ( range - ( 2 * range sqrt ) )
						with: ( ( ave * range ) / anInteger) - anInteger asFloat
						with: ( range + ( 2 * range sqrt ) )
! !

!RandomGenerator class methodsFor: 'instance creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
new
  " Answer a new instance of the receiver "

	^ ( self seed: Time millisecondClockValue ) initialize! !

!RandomGenerator class methodsFor: 'instance creation' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
seed: anInteger
	"Anwer a new Random stream with the initial seed anInteger.."

	^ self basicNew seed: anInteger; yourself! !

!RandomGenerator methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
atEnd
	"Answer whether the receiver is at its end - 
	never true for a stream of Random numbers"

	^ false
! !

!RandomGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
contents
	"Answer all of the objects in the collection accessed by the receiver.
	Implementation Notes: Random streams are infinite, 
	so there is no implementation possible."

	^ self shouldNotImplement
! !

!RandomGenerator methodsFor: 'initialize-release' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
initialize
  " Private - Initialize the receiver "

   ^ self subclassResponsibility! !

!RandomGenerator methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isReadable
	"Answer whether the receiver can be read from 
	(i.e. it implements the gettableStream protocol)."

	^ true
! !

!RandomGenerator methodsFor: 'testing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
isWriteable
	"Answer whether the receiver can be written to 
	(i.e. it implements the puttableStream protocol)."

	^ false
! !

!RandomGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next
  " Answer a pseudo-Random number "

   ^ self subclassResponsibility! !

!RandomGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
next: anInteger
	" Answer an Array of anInteger pseudo-random 
	Floats between 0 and 1. "

	^ ( 1 to: anInteger ) 
			collect: [ :i | self next ]
! !

!RandomGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
peek
	" Answer a pseudo-Random number generated from the next seed, 
	but do not advance down the stream (i.e. self peek = self peek). "

   ^ self subclassResponsibility! !

!RandomGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
seed
    "Returns the instance var of the receiver
     for external manipulation"

    ^ seed! !

!RandomGenerator methodsFor: 'accessing' stamp: 'DanielUber 5/10/2012 23:35:53.96'!
seed: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"

    seed := anObject! !
MersenneTwisterRandom initialize!
